@@ -14,9 +14,11 @@
 # under the License.
 import functools
 import hashlib
+import os
 import time
 import warnings
 
+from oslo_context import context as oslo_context
 from oslo_log import log as logging
 import requests
 import simplejson
@@ -30,6 +32,7 @@ from octavia.amphorae.drivers.keepalived import vrrp_rest_driver
 from octavia.common.config import cfg
 from octavia.common import constants as consts
 from octavia.common.jinja.haproxy import jinja_cfg
+from octavia.common.jinja.lvs import jinja_cfg as jinja_udp_cfg
 from octavia.common.tls_utils import cert_parser
 from octavia.common import utils
 
@@ -53,10 +56,25 @@ class HaproxyAmphoraLoadBalancerDriver(
             name=CONF.certificates.cert_manager,
             invoke_on_load=True,
         ).driver
+
         self.jinja = jinja_cfg.JinjaTemplater(
             base_amp_path=CONF.haproxy_amphora.base_path,
             base_crt_dir=CONF.haproxy_amphora.base_cert_dir,
-            haproxy_template=CONF.haproxy_amphora.haproxy_template)
+            haproxy_template=CONF.haproxy_amphora.haproxy_template,
+            connection_logging=CONF.haproxy_amphora.connection_logging)
+        self.udp_jinja = jinja_udp_cfg.LvsJinjaTemplater()
+
+    def _get_haproxy_versions(self, amphora):
+        """Get major and minor version number from haproxy
+
+        Example: ['1', '6']
+
+        :returns version_list: A list with the major and minor numbers
+        """
+
+        version_string = self.client.get_info(amphora)['haproxy_version']
+
+        return version_string.split('.')[:2]
 
     def update_amphora_listeners(self, listeners, amphora_index,
                                  amphorae, timeout_dict=None):
@@ -64,8 +82,10 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         :param listeners: List of listeners to update.
         :type listener: list
-        :param amphora_id: The ID of the amphora to update
-        :type amphora_id: string
+        :param amphora_index: The index of the amphora to update
+        :type amphora_index: integer
+        :param amphorae: List of amphorae
+        :type amphorae: list
         :param timeout_dict: Dictionary of timeout values for calls to the
                              amphora. May contain: req_conn_timeout,
                              req_read_timeout, conn_max_retries,
@@ -81,41 +101,87 @@ class HaproxyAmphoraLoadBalancerDriver(
         amp = amphorae[amphora_index]
         if amp is None or amp.status == consts.DELETED:
             return
+
+        haproxy_versions = self._get_haproxy_versions(amp)
+
         # TODO(johnsom) remove when we don't have a process per listener
         for listener in listeners:
             LOG.debug("%s updating listener %s on amphora %s",
                       self.__class__.__name__, listener.id, amp.id)
-            certs = self._process_tls_certificates(listener)
-            # Generate HaProxy configuration from listener object
-            config = self.jinja.build_config(
-                host_amphora=amp,
-                listener=listener,
-                tls_cert=certs['tls_cert'],
-                user_group=CONF.haproxy_amphora.user_group)
-            self.client.upload_config(amp, listener.id, config,
-                                      timeout_dict=timeout_dict)
-            self.client.reload_listener(amp, listener.id,
-                                        timeout_dict=timeout_dict)
+            if listener.protocol == 'UDP':
+                # Generate Keepalived LVS configuration from listener object
+                config = self.udp_jinja.build_config(listener=listener)
+                self.client.upload_udp_config(amp, listener.id, config,
+                                              timeout_dict=timeout_dict)
+                self.client.reload_listener(amp, listener.id,
+                                            timeout_dict=timeout_dict)
+            else:
+                certs = self._process_tls_certificates(listener)
+                client_ca_filename = self._process_secret(
+                    listener, listener.client_ca_tls_certificate_id)
+                crl_filename = self._process_secret(
+                    listener, listener.client_crl_container_id)
+                pool_tls_certs = self._process_listener_pool_certs(listener)
 
-    def update(self, listener, vip):
-        LOG.debug("Amphora %s haproxy, updating listener %s, vip %s",
+                # Generate HaProxy configuration from listener object
+                config = self.jinja.build_config(
+                    host_amphora=amp, listener=listener,
+                    tls_cert=certs['tls_cert'],
+                    haproxy_versions=haproxy_versions,
+                    client_ca_filename=client_ca_filename,
+                    client_crl=crl_filename,
+                    pool_tls_certs=pool_tls_certs)
+                self.client.upload_config(amp, listener.id, config,
+                                          timeout_dict=timeout_dict)
+                self.client.reload_listener(amp, listener.id,
+                                            timeout_dict=timeout_dict)
+
+    def _udp_update(self, listener, vip):
+        LOG.debug("Amphora %s keepalivedlvs, updating "
+                  "listener %s, vip %s",
                   self.__class__.__name__, listener.protocol_port,
                   vip.ip_address)
 
-        # Process listener certificate info
-        certs = self._process_tls_certificates(listener)
-
         for amp in listener.load_balancer.amphorae:
             if amp.status != consts.DELETED:
-                # Generate HaProxy configuration from listener object
-                config = self.jinja.build_config(
-                    host_amphora=amp,
-                    listener=listener,
-                    tls_cert=certs['tls_cert'],
-                    user_group=CONF.haproxy_amphora.user_group)
-                self.client.upload_config(amp, listener.id, config)
+                # Generate Keepalived LVS configuration from listener object
+                config = self.udp_jinja.build_config(listener=listener)
+                self.client.upload_udp_config(amp, listener.id, config)
                 self.client.reload_listener(amp, listener.id)
 
+    def update(self, listener, vip):
+        if listener.protocol == 'UDP':
+            self._udp_update(listener, vip)
+        else:
+            LOG.debug("Amphora %s haproxy, updating listener %s, "
+                      "vip %s", self.__class__.__name__,
+                      listener.protocol_port,
+                      vip.ip_address)
+
+            # Process listener certificate info
+            certs = self._process_tls_certificates(listener)
+            client_ca_filename = self._process_secret(
+                listener, listener.client_ca_tls_certificate_id)
+            crl_filename = self._process_secret(
+                listener, listener.client_crl_container_id)
+            pool_tls_certs = self._process_listener_pool_certs(listener)
+
+            for amp in listener.load_balancer.amphorae:
+                if amp.status != consts.DELETED:
+
+                    haproxy_versions = self._get_haproxy_versions(amp)
+
+                    # Generate HaProxy configuration from listener object
+                    config = self.jinja.build_config(
+                        host_amphora=amp, listener=listener,
+                        tls_cert=certs['tls_cert'],
+                        haproxy_versions=haproxy_versions,
+                        client_ca_filename=client_ca_filename,
+                        client_crl=crl_filename,
+                        pool_tls_certs=pool_tls_certs)
+                    self.client.upload_config(amp, listener.id, config)
+                    self.client.reload_listener(amp, listener.id)
+
     def upload_cert_amp(self, amp, pem):
         LOG.debug("Amphora %s updating cert in REST driver "
                   "with amphora id %s,",
@@ -231,15 +297,103 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         return {'tls_cert': tls_cert, 'sni_certs': sni_certs}
 
+    def _process_secret(self, listener, secret_ref):
+        """Get the secret from the cert manager and upload it to the amp.
+
+        :returns: The filename of the secret in the amp.
+        """
+        if not secret_ref:
+            return None
+        context = oslo_context.RequestContext(project_id=listener.project_id)
+        secret = self.cert_manager.get_secret(context, secret_ref)
+        try:
+            secret = secret.encode('utf-8')
+        except AttributeError:
+            pass
+        md5 = hashlib.md5(secret).hexdigest()  # nosec
+        id = hashlib.sha1(secret).hexdigest()  # nosec
+        name = '{id}.pem'.format(id=id)
+        self._apply(self._upload_cert, listener, None, secret, md5, name)
+        return name
+
+    def _process_listener_pool_certs(self, listener):
+        #     {'POOL-ID': {
+        #         'client_cert': client_full_filename,
+        #         'ca_cert': ca_cert_full_filename,
+        #         'crl': crl_full_filename}}
+        pool_certs_dict = dict()
+        for pool in listener.pools:
+            if pool.id not in pool_certs_dict:
+                pool_certs_dict[pool.id] = self._process_pool_certs(listener,
+                                                                    pool)
+        for l7policy in listener.l7policies:
+            if (l7policy.redirect_pool and
+                    l7policy.redirect_pool.id not in pool_certs_dict):
+                pool_certs_dict[l7policy.redirect_pool.id] = (
+                    self._process_pool_certs(listener, l7policy.redirect_pool))
+        return pool_certs_dict
+
+    def _process_pool_certs(self, listener, pool):
+        pool_cert_dict = dict()
+
+        # Handle the cleint cert(s) and key
+        if pool.tls_certificate_id:
+            data = cert_parser.load_certificates_data(self.cert_manager, pool)
+            pem = cert_parser.build_pem(data)
+            try:
+                pem = pem.encode('utf-8')
+            except AttributeError:
+                pass
+            md5 = hashlib.md5(pem).hexdigest()  # nosec
+            name = '{id}.pem'.format(id=data.id)
+            self._apply(self._upload_cert, listener, None, pem, md5, name)
+            pool_cert_dict['client_cert'] = os.path.join(
+                CONF.haproxy_amphora.base_cert_dir, listener.id, name)
+        if pool.ca_tls_certificate_id:
+            name = self._process_secret(listener, pool.ca_tls_certificate_id)
+            pool_cert_dict['ca_cert'] = os.path.join(
+                CONF.haproxy_amphora.base_cert_dir, listener.id, name)
+        if pool.crl_container_id:
+            name = self._process_secret(listener, pool.crl_container_id)
+            pool_cert_dict['crl'] = os.path.join(
+                CONF.haproxy_amphora.base_cert_dir, listener.id, name)
+
+        return pool_cert_dict
+
     def _upload_cert(self, amp, listener_id, pem, md5, name):
         try:
-            if self.client.get_cert_md5sum(amp, listener_id, name) == md5:
+            if self.client.get_cert_md5sum(
+                    amp, listener_id, name, ignore=(404,)) == md5:
                 return
         except exc.NotFound:
             pass
 
-        self.client.upload_cert_pem(
-            amp, listener_id, name, pem)
+        self.client.upload_cert_pem(amp, listener_id, name, pem)
+
+    def update_amphora_agent_config(self, amphora, agent_config,
+                                    timeout_dict=None):
+        """Update the amphora agent configuration file.
+
+        :param amphora: The amphora to update.
+        :type amphora: object
+        :param agent_config: The new amphora agent configuration.
+        :type agent_config: string
+        :param timeout_dict: Dictionary of timeout values for calls to the
+                             amphora. May contain: req_conn_timeout,
+                             req_read_timeout, conn_max_retries,
+                             conn_retry_interval
+        :returns: None
+
+        Note: This will mutate the amphora agent config and adopt the
+              new values.
+        """
+        try:
+            self.client.update_agent_config(amphora, agent_config,
+                                            timeout_dict=timeout_dict)
+        except exc.NotFound:
+            LOG.debug('Amphora {} does not support the update_agent_config '
+                      'API.'.format(amphora.id))
+            raise driver_except.AmpDriverNotImplementedError()
 
 
 # Check a custom hostname
@@ -377,6 +531,7 @@ class AmphoraAPIClient(object):
             'listeners/{listener_id}'.format(listener_id=listener_id))
         if exc.check_exception(r):
             return r.json()
+        return None
 
     def _action(self, action, amp, listener_id, timeout_dict=None):
         r = self.put(amp, 'listeners/{listener_id}/{action}'.format(
@@ -385,50 +540,51 @@ class AmphoraAPIClient(object):
 
     def upload_cert_pem(self, amp, listener_id, pem_filename, pem_file):
         r = self.put(
-            amp,
-            'listeners/{listener_id}/certificates/{filename}'.format(
+            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
                 listener_id=listener_id, filename=pem_filename),
             data=pem_file)
         return exc.check_exception(r)
 
+    def get_cert_md5sum(self, amp, listener_id, pem_filename, ignore=tuple()):
+        r = self.get(
+            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
+                listener_id=listener_id, filename=pem_filename))
+        if exc.check_exception(r, ignore):
+            return r.json().get("md5sum")
+        return None
+
+    def delete_cert_pem(self, amp, listener_id, pem_filename):
+        r = self.delete(
+            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
+                listener_id=listener_id, filename=pem_filename))
+        return exc.check_exception(r, (404,))
+
     def update_cert_for_rotation(self, amp, pem_file):
         r = self.put(amp, 'certificate', data=pem_file)
         return exc.check_exception(r)
 
-    def get_cert_md5sum(self, amp, listener_id, pem_filename):
-        r = self.get(amp,
-                     'listeners/{listener_id}/certificates/{filename}'.format(
-                         listener_id=listener_id, filename=pem_filename))
-        if exc.check_exception(r):
-            return r.json().get("md5sum")
-
     def delete_listener(self, amp, listener_id):
         r = self.delete(
             amp, 'listeners/{listener_id}'.format(listener_id=listener_id))
-
         return exc.check_exception(r, (404,))
 
     def get_info(self, amp):
         r = self.get(amp, "info")
         if exc.check_exception(r):
             return r.json()
+        return None
 
     def get_details(self, amp):
         r = self.get(amp, "details")
         if exc.check_exception(r):
             return r.json()
+        return None
 
     def get_all_listeners(self, amp):
         r = self.get(amp, "listeners")
         if exc.check_exception(r):
             return r.json()
-
-    def delete_cert_pem(self, amp, listener_id, pem_filename):
-        r = self.delete(
-            amp,
-            'listeners/{listener_id}/certificates/{filename}'.format(
-                listener_id=listener_id, filename=pem_filename))
-        return exc.check_exception(r, (404,))
+        return None
 
     def plug_network(self, amp, port):
         r = self.post(amp, 'plug/network',
@@ -454,3 +610,16 @@ class AmphoraAPIClient(object):
                      timeout_dict=timeout_dict)
         if exc.check_exception(r):
             return r.json()
+        return None
+
+    def upload_udp_config(self, amp, listener_id, config, timeout_dict=None):
+        r = self.put(
+            amp,
+            'listeners/{amphora_id}/{listener_id}/udp_listener'.format(
+                amphora_id=amp.id, listener_id=listener_id), timeout_dict,
+            data=config)
+        return exc.check_exception(r)
+
+    def update_agent_config(self, amp, agent_config, timeout_dict=None):
+        r = self.put(amp, 'config', timeout_dict, data=agent_config)
+        return exc.check_exception(r)
