@@ -12,10 +12,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from concurrent import futures
 import socket
 import time
 
-from concurrent import futures
 from oslo_config import cfg
 from oslo_log import log as logging
 from stevedore import driver as stevedore_driver
@@ -29,22 +29,22 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
-def update_health(obj):
+def update_health(obj, srcaddr):
     handler = stevedore_driver.DriverManager(
         namespace='octavia.amphora.health_update_drivers',
         name=CONF.health_manager.health_update_driver,
         invoke_on_load=True
     ).driver
-    handler.update_health(obj)
+    handler.update_health(obj, srcaddr)
 
 
-def update_stats(obj):
+def update_stats(obj, srcaddr):
     handler = stevedore_driver.DriverManager(
         namespace='octavia.amphora.stats_update_drivers',
         name=CONF.health_manager.stats_update_driver,
         invoke_on_load=True
     ).driver
-    handler.update_stats(obj)
+    handler.update_stats(obj, srcaddr)
 
 
 class UDPStatusGetter(object):
@@ -63,8 +63,10 @@ class UDPStatusGetter(object):
         self.sock = None
         self.update(self.key, self.ip, self.port)
 
-        self.executor = futures.ProcessPoolExecutor(
-            max_workers=cfg.CONF.health_manager.status_update_threads)
+        self.health_executor = futures.ProcessPoolExecutor(
+            max_workers=CONF.health_manager.health_update_threads)
+        self.stats_executor = futures.ProcessPoolExecutor(
+            max_workers=CONF.health_manager.stats_update_threads)
         self.repo = repositories.Repositories().amphorahealth
 
     def update(self, key, ip, port):
@@ -82,6 +84,7 @@ class UDPStatusGetter(object):
             if self.sock is not None:
                 self.sock.close()
             self.sock = socket.socket(ai_family, socket.SOCK_DGRAM)
+            self.sock.settimeout(1)
             self.sock.bind(self.sockaddr)
             if cfg.CONF.health_manager.sock_rlimit > 0:
                 rlimit = cfg.CONF.health_manager.sock_rlimit
@@ -184,16 +187,29 @@ class UDPStatusGetter(object):
         """
         (data, srcaddr) = self.sock.recvfrom(UDP_MAX_SIZE)
         LOG.debug('Received packet from %s', srcaddr)
-        obj = status_message.unwrap_envelope(data, self.key)
+        try:
+            obj = status_message.unwrap_envelope(data, self.key)
+        except Exception as e:
+            LOG.warning('Health Manager experienced an exception processing a '
+                        'heartbeat message from %s. Ignoring this packet. '
+                        'Exception: %s', srcaddr, e)
+            raise exceptions.InvalidHMACException()
         obj['recv_time'] = time.time()
-        return obj, srcaddr
+        return obj, srcaddr[0]
 
     def check(self):
         try:
             obj, srcaddr = self.dorecv()
+        except socket.timeout:
+            # Pass here as this is an expected cycling of the listen socket
+            pass
         except exceptions.InvalidHMACException:
             # Pass here as the packet was dropped and logged already
             pass
+        except Exception as e:
+            LOG.warning('Health Manager experienced an exception processing a '
+                        'heartbeat packet. Ignoring this packet. '
+                        'Exception: %s', e)
         else:
-            self.executor.submit(update_health, obj)
-            self.executor.submit(update_stats, obj)
+            self.health_executor.submit(update_health, obj, srcaddr)
+            self.stats_executor.submit(update_stats, obj, srcaddr)
