@@ -14,7 +14,7 @@
 
 from oslo_log import log as logging
 
-from octavia.amphorae.drivers import driver_base as driver_base
+from octavia.amphorae.drivers import driver_base
 
 LOG = logging.getLogger(__name__)
 
@@ -37,6 +37,16 @@ class NoopManager(object):
         super(NoopManager, self).__init__()
         self.amphoraconfig = {}
 
+    def update_amphora_listeners(self, listeners, amphora_index,
+                                 amphorae, timeout_dict):
+        amphora_id = amphorae[amphora_index].id
+        for listener in listeners:
+            LOG.debug("Amphora noop driver update_amphora_listeners, "
+                      "listener %s, amphora %s, timeouts %s", listener.id,
+                      amphora_id, timeout_dict)
+            self.amphoraconfig[(listener.id, amphora_id)] = (
+                listener, amphora_id, timeout_dict, "update_amp")
+
     def update(self, listener, vip):
         LOG.debug("Amphora %s no-op, update listener %s, vip %s",
                   self.__class__.__name__, listener.protocol_port,
@@ -51,12 +61,13 @@ class NoopManager(object):
         self.amphoraconfig[(listener.protocol_port,
                             vip.ip_address)] = (listener, vip, 'stop')
 
-    def start(self, listener, vip):
-        LOG.debug("Amphora %s no-op, start listener %s, vip %s",
+    def start(self, listener, vip, amphora=None):
+        LOG.debug("Amphora %s no-op, start listener %s, vip %s, amp %s",
                   self.__class__.__name__,
-                  listener.protocol_port, vip.ip_address)
+                  listener.protocol_port, vip.ip_address, amphora)
         self.amphoraconfig[(listener.protocol_port,
-                            vip.ip_address)] = (listener, vip, 'start')
+                            vip.ip_address, amphora)] = (listener, vip,
+                                                         amphora, 'start')
 
     def delete(self, listener, vip):
         LOG.debug("Amphora %s no-op, delete listener %s, vip %s",
@@ -93,11 +104,18 @@ class NoopManager(object):
             load_balancer.id, amphorae_network_config, 'post_vip_plug')
 
     def upload_cert_amp(self, amphora, pem_file):
-        LOG.debug("Amphora %s no-op, upload cert amphora %s,with pem fle %s",
+        LOG.debug("Amphora %s no-op, upload cert amphora %s,with pem file %s",
                   self.__class__.__name__, amphora.id, pem_file)
         self.amphoraconfig[amphora.id, pem_file] = (amphora.id, pem_file,
                                                     'update_amp_cert_file')
 
+    def update_agent_config(self, amphora, agent_config):
+        LOG.debug("Amphora %s no-op, update agent config amphora "
+                  "%s, with agent config %s",
+                  self.__class__.__name__, amphora.id, agent_config)
+        self.amphoraconfig[amphora.id, agent_config] = (
+            amphora.id, agent_config, 'update_agent_config')
+
 
 class NoopAmphoraLoadBalancerDriver(
     driver_base.AmphoraLoadBalancerDriver,
@@ -106,6 +124,12 @@ class NoopAmphoraLoadBalancerDriver(
         super(NoopAmphoraLoadBalancerDriver, self).__init__()
         self.driver = NoopManager()
 
+    def update_amphora_listeners(self, listeners, amphora_index,
+                                 amphorae, timeout_dict):
+
+        self.driver.update_amphora_listeners(listeners, amphora_index,
+                                             amphorae, timeout_dict)
+
     def update(self, listener, vip):
 
         self.driver.update(listener, vip)
@@ -114,9 +138,9 @@ class NoopAmphoraLoadBalancerDriver(
 
         self.driver.stop(listener, vip)
 
-    def start(self, listener, vip):
+    def start(self, listener, vip, amphora=None):
 
-        self.driver.start(listener, vip)
+        self.driver.start(listener, vip, amphora)
 
     def delete(self, listener, vip):
 
@@ -147,6 +171,9 @@ class NoopAmphoraLoadBalancerDriver(
 
         self.driver.upload_cert_amp(amphora, pem_file)
 
+    def update_agent_config(self, amphora, agent_config):
+        self.driver.update_agent_config(amphora, agent_config)
+
     def update_vrrp_conf(self, loadbalancer):
         pass
 
