@@ -117,7 +117,8 @@ class HealthMonitorController(base.BaseController):
             raise exceptions.InvalidOption(value='None',
                                            option=consts.MAX_RETRIES)
 
-        if hm_dict[consts.TYPE] != consts.HEALTH_MONITOR_HTTP:
+        if hm_dict[consts.TYPE] not in (consts.HEALTH_MONITOR_HTTP,
+                                        consts.HEALTH_MONITOR_HTTPS):
             if hm_dict.get(consts.HTTP_METHOD, None):
                 raise exceptions.InvalidOption(
                     value=consts.HTTP_METHOD, option='health monitors of '
@@ -141,6 +142,17 @@ class HealthMonitorController(base.BaseController):
                 hm_dict[consts.EXPECTED_CODES] = (
                     consts.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES)
 
+        if hm_dict.get('domain_name') and not hm_dict.get('http_version'):
+            raise exceptions.ValidationException(
+                detail=_("'http_version' must be specified when 'domain_name' "
+                         "is provided."))
+
+        if hm_dict.get('http_version') and hm_dict.get('domain_name'):
+            if hm_dict['http_version'] < 1.1:
+                raise exceptions.InvalidOption(
+                    value='http_version %s' % hm_dict['http_version'],
+                    option='health monitors HTTP 1.1 domain name health check')
+
         try:
             return self.repositories.health_monitor.create(
                 lock_session, **hm_dict)
@@ -259,7 +271,8 @@ class HealthMonitorController(base.BaseController):
         return db_hm
 
     def _validate_update_hm(self, db_hm, health_monitor):
-        if db_hm.type != consts.HEALTH_MONITOR_HTTP:
+        if db_hm.type not in (consts.HEALTH_MONITOR_HTTP,
+                              consts.HEALTH_MONITOR_HTTPS):
             if health_monitor.http_method != wtypes.Unset:
                 raise exceptions.InvalidOption(
                     value=consts.HTTP_METHOD, option='health monitors of '
@@ -270,7 +283,7 @@ class HealthMonitorController(base.BaseController):
                     'type {}'.format(db_hm.type))
             if health_monitor.expected_codes != wtypes.Unset:
                 raise exceptions.InvalidOption(
-                    value=consts.URL_PATH, option='health monitors of '
+                    value=consts.EXPECTED_CODES, option='health monitors of '
                     'type {}'.format(db_hm.type))
         else:
             # For HTTP health monitor these cannot be null/None
@@ -281,6 +294,20 @@ class HealthMonitorController(base.BaseController):
             if health_monitor.expected_codes is None:
                 health_monitor.expected_codes = wtypes.Unset
 
+        if health_monitor.domain_name and not (
+                db_hm.http_version or health_monitor.http_version):
+            raise exceptions.ValidationException(
+                detail=_("'http_version' must be specified when 'domain_name' "
+                         "is provided."))
+
+        if ((db_hm.http_version or health_monitor.http_version) and
+                (db_hm.domain_name or health_monitor.domain_name)):
+            http_version = health_monitor.http_version or db_hm.http_version
+            if http_version < 1.1:
+                raise exceptions.InvalidOption(
+                    value='http_version %s' % http_version,
+                    option='health monitors HTTP 1.1 domain name health check')
+
     @wsme_pecan.wsexpose(hm_types.HealthMonitorRootResponse, wtypes.text,
                          body=hm_types.HealthMonitorRootPUT, status_code=200)
     def put(self, id, health_monitor_):
