@@ -19,6 +19,7 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import strutils
 import pecan
+from sqlalchemy.orm import exc as sa_exception
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
@@ -135,29 +136,9 @@ class LoadBalancersController(base.BaseController):
                         "VIP address specified."
                     ))
             else:
-                # If subnet and IP are not provided, pick the first subnet with
-                # enough available IPs, preferring ipv4
-                if not network.subnets:
-                    raise exceptions.ValidationException(detail=_(
-                        "Supplied network does not contain a subnet."
-                    ))
-                ip_avail = network_driver.get_network_ip_availability(
-                    network)
-                if (CONF.controller_worker.loadbalancer_topology ==
-                        constants.TOPOLOGY_SINGLE):
-                    num_req_ips = 2
-                if (CONF.controller_worker.loadbalancer_topology ==
-                        constants.TOPOLOGY_ACTIVE_STANDBY):
-                    num_req_ips = 3
-                subnets = [subnet_id for subnet_id in network.subnets if
-                           utils.subnet_ip_availability(ip_avail, subnet_id,
-                                                        num_req_ips)]
-                if not subnets:
-                    raise exceptions.ValidationException(detail=_(
-                        "Subnet(s) in the supplied network do not contain "
-                        "enough available IPs."
-                    ))
-                for subnet_id in subnets:
+                # If subnet and IP are not provided, pick the first subnet,
+                # preferring ipv4
+                for subnet_id in network.subnets:
                     # Use the first subnet, in case there are no ipv4 subnets
                     if not load_balancer.vip_subnet_id:
                         load_balancer.vip_subnet_id = subnet_id
@@ -165,6 +146,10 @@ class LoadBalancersController(base.BaseController):
                     if subnet.ip_version == 4:
                         load_balancer.vip_subnet_id = subnet_id
                         break
+                if not load_balancer.vip_subnet_id:
+                    raise exceptions.ValidationException(detail=_(
+                        "Supplied network does not contain a subnet."
+                    ))
 
     @staticmethod
     def _validate_port_and_fill_or_validate_subnet(load_balancer):
@@ -262,6 +247,71 @@ class LoadBalancersController(base.BaseController):
                 e.msg = e.orig_msg
             raise e
 
+    def _get_provider(self, session, load_balancer):
+        """Decide on the provider for this load balancer."""
+
+        provider = None
+        if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
+            try:
+                provider = self.repositories.flavor.get_flavor_provider(
+                    session, load_balancer.flavor_id)
+            except sa_exception.NoResultFound:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id."))
+
+        # No provider specified and no flavor specified, use conf default
+        if (isinstance(load_balancer.provider, wtypes.UnsetType) and
+                not provider):
+            provider = CONF.api_settings.default_provider_driver
+        # Both provider and flavor specified, they must match
+        elif (not isinstance(load_balancer.provider, wtypes.UnsetType) and
+                provider):
+            if provider != load_balancer.provider:
+                raise exceptions.ProviderFlavorMismatchError(
+                    flav=load_balancer.flavor_id, prov=load_balancer.provider)
+        # No flavor, but provider, use the provider specified
+        elif not provider:
+            provider = load_balancer.provider
+        # Otherwise, use the flavor provider we found above
+
+        return provider
+
+    def _apply_flavor_to_lb_dict(self, lock_session, driver, lb_dict):
+
+        flavor_dict = {}
+        if 'flavor_id' in lb_dict:
+            try:
+                flavor_dict = (
+                    self.repositories.flavor.get_flavor_metadata_dict(
+                        lock_session, lb_dict['flavor_id']))
+            except sa_exception.NoResultFound:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id."))
+
+        # Make sure the driver will still accept the flavor metadata
+        if flavor_dict:
+            driver_utils.call_provider(driver.name, driver.validate_flavor,
+                                       flavor_dict)
+
+        # Apply the flavor settings to the load balanacer
+        # Use the configuration file settings as defaults
+        lb_dict[constants.TOPOLOGY] = flavor_dict.get(
+            constants.LOADBALANCER_TOPOLOGY,
+            CONF.controller_worker.loadbalancer_topology)
+
+        return flavor_dict
+
+    def _validate_flavor(self, session, load_balancer):
+        if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
+            flavor = self.repositories.flavor.get(session,
+                                                  id=load_balancer.flavor_id)
+            if not flavor:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id."))
+            if not flavor.enabled:
+                raise exceptions.DisabledOption(option='flavor',
+                                                value=load_balancer.flavor_id)
+
     @wsme_pecan.wsexpose(lb_types.LoadBalancerFullRootResponse,
                          body=lb_types.LoadBalancerRootPOST, status_code=201)
     def post(self, load_balancer):
@@ -274,17 +324,19 @@ class LoadBalancersController(base.BaseController):
 
         if not load_balancer.project_id:
             raise exceptions.ValidationException(detail=_(
-                "Missing project ID in request where one is required. "
-                "An administrator should check the keystone settings "
-                "in the Octavia configuration."))
+                "Missing project ID in request where one is required."))
 
         self._auth_validate_action(context, load_balancer.project_id,
                                    constants.RBAC_POST)
 
         self._validate_vip_request_object(load_balancer)
 
+        self._validate_flavor(context.session, load_balancer)
+
+        provider = self._get_provider(context.session, load_balancer)
+
         # Load the driver early as it also provides validation
-        driver = driver_factory.get_driver(load_balancer.provider)
+        driver = driver_factory.get_driver(provider)
 
         lock_session = db_api.get_session(autocommit=False)
         try:
@@ -310,14 +362,17 @@ class LoadBalancersController(base.BaseController):
             listeners = lb_dict.pop('listeners', []) or []
             pools = lb_dict.pop('pools', []) or []
 
-            # TODO(johnsom) Remove flavor from the lb_dict
-            # as it has not been implemented beyond the API yet.
-            # Remove this line when it is implemented.
-            lb_dict.pop('flavor', None)
+            flavor_dict = self._apply_flavor_to_lb_dict(lock_session, driver,
+                                                        lb_dict)
 
             db_lb = self.repositories.create_load_balancer_and_vip(
                 lock_session, lb_dict, vip_dict)
 
+            # Pass the flavor dictionary through for the provider drivers
+            # This is a "virtual" lb_dict item that includes the expanded
+            # flavor dict instead of just the flavor_id we store in the DB.
+            lb_dict['flavor'] = flavor_dict
+
             # See if the provider driver wants to create the VIP port
             octavia_owned = False
             try:
@@ -387,8 +442,9 @@ class LoadBalancersController(base.BaseController):
                     detail='Pools must be named when creating a fully '
                            'populated loadbalancer.')
             # If a pool has more than a name, assume it's a full specification
-            # (but use >2 because it will also have "enabled" as default)
-            if default_pool and len(default_pool) > 2:
+            # (but use >3 because it will also have "enabled" and "tls_enabled"
+            # as default)
+            if default_pool and len(default_pool) > 3:
                 pools.append(default_pool)
                 l['default_pool'] = {'name': pool_name}
             # Otherwise, it's a reference and we record it and move on
@@ -405,8 +461,9 @@ class LoadBalancersController(base.BaseController):
                         detail='Pools must be named when creating a fully '
                                'populated loadbalancer.')
                 # If a pool has more than a name, assume it's a full spec
-                # (but use >2 because it will also have "enabled" as default)
-                if redirect_pool and len(redirect_pool) > 2:
+                # (but use >2 because it will also have "enabled" and
+                # "tls_enabled" as default)
+                if redirect_pool and len(redirect_pool) > 3:
                     pool_name = redirect_pool['name']
                     policy['redirect_pool'] = {'name': pool_name}
                     pools.append(redirect_pool)
@@ -563,8 +620,7 @@ class LoadBalancersController(base.BaseController):
             LOG.info("Sending delete Load Balancer %s to provider %s",
                      id, driver.name)
             provider_loadbalancer = (
-                driver_utils.db_loadbalancer_to_provider_loadbalancer(
-                    db_lb, for_delete=True))
+                driver_utils.db_loadbalancer_to_provider_loadbalancer(db_lb))
             driver_utils.call_provider(driver.name, driver.loadbalancer_delete,
                                        provider_loadbalancer, cascade)
 
