@@ -44,6 +44,9 @@ class HealthMonitorResponse(BaseHealthMonitorType):
     operating_status = wtypes.wsattr(wtypes.StringType())
     created_at = wtypes.wsattr(wtypes.datetime.datetime)
     updated_at = wtypes.wsattr(wtypes.datetime.datetime)
+    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType()))
+    http_version = wtypes.wsattr(float)
+    domain_name = wtypes.wsattr(wtypes.StringType())
 
     @classmethod
     def from_data_model(cls, data_model, children=False):
@@ -89,18 +92,21 @@ class HealthMonitorPOST(BaseHealthMonitorType):
                            maximum=constants.MAX_HM_RETRIES),
         mandatory=True)
     http_method = wtypes.wsattr(
-        wtypes.Enum(str, *constants.SUPPORTED_HEALTH_MONITOR_HTTP_METHODS),
-        default=constants.HEALTH_MONITOR_HTTP_DEFAULT_METHOD)
+        wtypes.Enum(str, *constants.SUPPORTED_HEALTH_MONITOR_HTTP_METHODS))
     url_path = wtypes.wsattr(
-        types.URLPathType(),
-        default=constants.HEALTH_MONITOR_DEFAULT_URL_PATH)
+        types.URLPathType())
     expected_codes = wtypes.wsattr(
-        wtypes.StringType(pattern=r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$'),
-        default=constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES)
+        wtypes.StringType(pattern=r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$'))
     admin_state_up = wtypes.wsattr(bool, default=True)
     # TODO(johnsom) Remove after deprecation (R series)
     project_id = wtypes.wsattr(wtypes.StringType(max_length=36))
     pool_id = wtypes.wsattr(wtypes.UuidType(), mandatory=True)
+    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))
+    http_version = wtypes.wsattr(
+        wtypes.Enum(float, *constants.SUPPORTED_HTTP_VERSIONS))
+    domain_name = wtypes.wsattr(
+        wtypes.StringType(min_length=1, max_length=255,
+                          pattern=constants.DOMAIN_NAME_REGEX))
 
 
 class HealthMonitorRootPOST(types.BaseType):
@@ -124,6 +130,12 @@ class HealthMonitorPUT(BaseHealthMonitorType):
     expected_codes = wtypes.wsattr(
         wtypes.StringType(pattern=r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$'))
     admin_state_up = wtypes.wsattr(bool)
+    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))
+    http_version = wtypes.wsattr(
+        wtypes.Enum(float, *constants.SUPPORTED_HTTP_VERSIONS))
+    domain_name = wtypes.wsattr(
+        wtypes.StringType(min_length=1, max_length=255,
+                          pattern=constants.DOMAIN_NAME_REGEX))
 
 
 class HealthMonitorRootPUT(types.BaseType):
@@ -146,15 +158,17 @@ class HealthMonitorSingleCreate(BaseHealthMonitorType):
                            maximum=constants.MAX_HM_RETRIES),
         mandatory=True)
     http_method = wtypes.wsattr(
-        wtypes.Enum(str, *constants.SUPPORTED_HEALTH_MONITOR_HTTP_METHODS),
-        default=constants.HEALTH_MONITOR_HTTP_DEFAULT_METHOD)
-    url_path = wtypes.wsattr(
-        types.URLPathType(),
-        default=constants.HEALTH_MONITOR_DEFAULT_URL_PATH)
+        wtypes.Enum(str, *constants.SUPPORTED_HEALTH_MONITOR_HTTP_METHODS))
+    url_path = wtypes.wsattr(types.URLPathType())
     expected_codes = wtypes.wsattr(
-        wtypes.StringType(pattern=r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$'),
-        default=constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES)
+        wtypes.StringType(pattern=r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$'))
     admin_state_up = wtypes.wsattr(bool, default=True)
+    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))
+    http_version = wtypes.wsattr(
+        wtypes.Enum(float, *constants.SUPPORTED_HTTP_VERSIONS))
+    domain_name = wtypes.wsattr(
+        wtypes.StringType(min_length=1, max_length=255,
+                          pattern=constants.DOMAIN_NAME_REGEX))
 
 
 class HealthMonitorStatusResponse(BaseHealthMonitorType):
