@@ -18,10 +18,10 @@ Common classes for Barbican certificate handling
 """
 
 import abc
-import six
 
-from barbicanclient import client as barbican_client
+from barbicanclient.v1 import containers
 from oslo_utils import encodeutils
+import six
 
 from octavia.certificates.common import cert
 from octavia.common.tls_utils import cert_parser
@@ -31,8 +31,7 @@ from octavia.i18n import _
 class BarbicanCert(cert.Cert):
     """Representation of a Cert based on the Barbican CertificateContainer."""
     def __init__(self, cert_container):
-        if not isinstance(cert_container,
-                          barbican_client.containers.CertificateContainer):
+        if not isinstance(cert_container, containers.CertificateContainer):
             raise TypeError(_("Retrieved Barbican Container is not of the "
                               "correct type (certificate)."))
         self._cert_container = cert_container
@@ -41,6 +40,7 @@ class BarbicanCert(cert.Cert):
         if self._cert_container.certificate:
             return encodeutils.to_utf8(
                 self._cert_container.certificate.payload)
+        return None
 
     def get_intermediates(self):
         if self._cert_container.intermediates:
@@ -48,16 +48,19 @@ class BarbicanCert(cert.Cert):
                 self._cert_container.intermediates.payload)
             return [imd for imd in cert_parser.get_intermediates_pems(
                 intermediates)]
+        return None
 
     def get_private_key(self):
         if self._cert_container.private_key:
             return encodeutils.to_utf8(
                 self._cert_container.private_key.payload)
+        return None
 
     def get_private_key_passphrase(self):
         if self._cert_container.private_key_passphrase:
             return encodeutils.to_utf8(
                 self._cert_container.private_key_passphrase.payload)
+        return None
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -70,3 +73,19 @@ class BarbicanAuth(object):
         :return: a Barbican Client object
         :raises Exception: if the client cannot be created
         """
+
+    @abc.abstractmethod
+    def ensure_secret_access(self, context, ref):
+        """Do whatever steps are necessary to ensure future access to a secret.
+
+        :param context: pecan context object
+        :param ref: Reference to a Barbican object
+        """
+
+    @abc.abstractmethod
+    def revoke_secret_access(self, context, ref):
+        """Revoke access of Octavia keystone user to a secret.
+
+        :param context: pecan context object
+        :param ref: Reference to a Barbican object
+        """
