@@ -31,14 +31,14 @@ class LocalCertManager(cert_mgr.CertManager):
     """Cert Manager Interface that stores data locally."""
 
     @staticmethod
-    def store_cert(project_id, certificate, private_key, intermediates=None,
+    def store_cert(context, certificate, private_key, intermediates=None,
                    private_key_passphrase=None, **kwargs):
         """Stores (i.e., registers) a cert with the cert manager.
 
         This method stores the specified cert to the filesystem and returns
         a UUID that can be used to retrieve it.
 
-        :param project_id: Ignored in this implementation
+        :param context: Ignored in this implementation
         :param certificate: PEM encoded TLS certificate
         :param private_key: private key for the supplied certificate
         :param intermediates: ordered and concatenated intermediate certs
@@ -82,10 +82,10 @@ class LocalCertManager(cert_mgr.CertManager):
         return cert_ref
 
     @staticmethod
-    def get_cert(project_id, cert_ref, **kwargs):
+    def get_cert(context, cert_ref, **kwargs):
         """Retrieves the specified cert.
 
-        :param project_id: Ignored in this implementation
+        :param context: Ignored in this implementation
         :param cert_ref: the UUID of the cert to retrieve
 
         :return: octavia.certificates.common.Cert representation of the
@@ -134,10 +134,10 @@ class LocalCertManager(cert_mgr.CertManager):
         return local_common.LocalCert(**cert_data)
 
     @staticmethod
-    def delete_cert(project_id, cert_ref, **kwargs):
+    def delete_cert(context, cert_ref, **kwargs):
         """Deletes the specified cert.
 
-        :param project_id: Ignored in this implementation
+        :param context: Ignored in this implementation
         :param cert_ref: the UUID of the cert to delete
 
         :raises CertificateStorageException: if certificate deletion fails
@@ -160,3 +160,40 @@ class LocalCertManager(cert_mgr.CertManager):
         except IOError as ioe:
             LOG.error("Failed to delete certificate %s", cert_ref)
             raise exceptions.CertificateStorageException(message=ioe.message)
+
+    def set_acls(self, context, cert_ref):
+        # There is no security on this store, because it's really dumb
+        pass
+
+    def unset_acls(self, context, cert_ref):
+        # There is no security on this store, because it's really dumb
+        pass
+
+    @staticmethod
+    def get_secret(context, secret_ref):
+        """Retrieves a secret payload by reference.
+
+        :param context: Ignored in this implementation
+        :param secret_ref: The secret reference ID
+
+        :return: The secret payload
+        :raises CertificateStorageException: if secret retrieval fails
+        """
+        LOG.info("Loading secret %s from the local filesystem.", secret_ref)
+
+        filename_base = os.path.join(CONF.certificates.storage_path,
+                                     secret_ref)
+
+        filename_secret = "{0}.pem".format(filename_base)
+
+        secret_data = None
+
+        flags = os.O_RDONLY
+        try:
+            with os.fdopen(os.open(filename_secret, flags)) as secret_file:
+                secret_data = secret_file.read()
+        except IOError:
+            LOG.error("Failed to read secret for %s.", secret_ref)
+            raise exceptions.CertificateRetrievalException(ref=secret_ref)
+
+        return secret_data
