@@ -12,47 +12,183 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-LB_ALGORITHM_ROUND_ROBIN = 'ROUND_ROBIN'
-LB_ALGORITHM_LEAST_CONNECTIONS = 'LEAST_CONNECTIONS'
-LB_ALGORITHM_SOURCE_IP = 'SOURCE_IP'
-SUPPORTED_LB_ALGORITHMS = (LB_ALGORITHM_LEAST_CONNECTIONS,
-                           LB_ALGORITHM_ROUND_ROBIN,
-                           LB_ALGORITHM_SOURCE_IP)
+from octavia_lib.common import constants as lib_consts
 
-SESSION_PERSISTENCE_SOURCE_IP = 'SOURCE_IP'
-SESSION_PERSISTENCE_HTTP_COOKIE = 'HTTP_COOKIE'
-SESSION_PERSISTENCE_APP_COOKIE = 'APP_COOKIE'
-SUPPORTED_SP_TYPES = (SESSION_PERSISTENCE_SOURCE_IP,
-                      SESSION_PERSISTENCE_HTTP_COOKIE,
-                      SESSION_PERSISTENCE_APP_COOKIE)
+##############################################################################
+# Constants common to the provider drivers moved to
+#     octavia_lib.common.constants
+# These are deprecated, to be removed in the 'U' release
+##############################################################################
+# 'loadbalancers'
+LOADBALANCERS = lib_consts.LOADBALANCERS
+# 'listeners'
+LISTENERS = lib_consts.LISTENERS
+# 'pools'
+POOLS = lib_consts.POOLS
+# HEALTHMONITORS = 'healthmonitors'
+HEALTHMONITORS = lib_consts.HEALTHMONITORS
+# 'members'
+MEMBERS = lib_consts.MEMBERS
+# 'l7policies'
+L7POLICIES = lib_consts.L7POLICIES
+# 'l7rules'
+L7RULES = lib_consts.L7RULES
 
-HEALTH_MONITOR_PING = 'PING'
-HEALTH_MONITOR_TCP = 'TCP'
-HEALTH_MONITOR_HTTP = 'HTTP'
-HEALTH_MONITOR_HTTPS = 'HTTPS'
-HEALTH_MONITOR_TLS_HELLO = 'TLS-HELLO'
-HEALTH_MONITOR_UDP_CONNECT = 'UDP-CONNECT'
-SUPPORTED_HEALTH_MONITOR_TYPES = (HEALTH_MONITOR_HTTP, HEALTH_MONITOR_HTTPS,
-                                  HEALTH_MONITOR_PING, HEALTH_MONITOR_TCP,
-                                  HEALTH_MONITOR_TLS_HELLO,
-                                  HEALTH_MONITOR_UDP_CONNECT)
-HEALTH_MONITOR_HTTP_METHOD_GET = 'GET'
-HEALTH_MONITOR_HTTP_METHOD_HEAD = 'HEAD'
-HEALTH_MONITOR_HTTP_METHOD_POST = 'POST'
-HEALTH_MONITOR_HTTP_METHOD_PUT = 'PUT'
-HEALTH_MONITOR_HTTP_METHOD_DELETE = 'DELETE'
-HEALTH_MONITOR_HTTP_METHOD_TRACE = 'TRACE'
-HEALTH_MONITOR_HTTP_METHOD_OPTIONS = 'OPTIONS'
-HEALTH_MONITOR_HTTP_METHOD_CONNECT = 'CONNECT'
-HEALTH_MONITOR_HTTP_METHOD_PATCH = 'PATCH'
-HEALTH_MONITOR_HTTP_DEFAULT_METHOD = HEALTH_MONITOR_HTTP_METHOD_GET
+# 'PING'
+HEALTH_MONITOR_PING = lib_consts.HEALTH_MONITOR_PING
+# 'TCP'
+HEALTH_MONITOR_TCP = lib_consts.HEALTH_MONITOR_TCP
+# 'HTTP'
+HEALTH_MONITOR_HTTP = lib_consts.HEALTH_MONITOR_HTTP
+# 'HTTPS'
+HEALTH_MONITOR_HTTPS = lib_consts.HEALTH_MONITOR_HTTPS
+# 'TLS-HELLO'
+HEALTH_MONITOR_TLS_HELLO = lib_consts.HEALTH_MONITOR_TLS_HELLO
+# 'UDP-CONNECT'
+HEALTH_MONITOR_UDP_CONNECT = lib_consts.HEALTH_MONITOR_UDP_CONNECT
+SUPPORTED_HEALTH_MONITOR_TYPES = lib_consts.SUPPORTED_HEALTH_MONITOR_TYPES
+
+# 'GET'
+HEALTH_MONITOR_HTTP_METHOD_GET = lib_consts.HEALTH_MONITOR_HTTP_METHOD_GET
+# 'HEAD'
+HEALTH_MONITOR_HTTP_METHOD_HEAD = lib_consts.HEALTH_MONITOR_HTTP_METHOD_HEAD
+# 'POST'
+HEALTH_MONITOR_HTTP_METHOD_POST = lib_consts.HEALTH_MONITOR_HTTP_METHOD_POST
+# 'PUT'
+HEALTH_MONITOR_HTTP_METHOD_PUT = lib_consts.HEALTH_MONITOR_HTTP_METHOD_PUT
+# 'DELETE'
+HEALTH_MONITOR_HTTP_METHOD_DELETE = (
+    lib_consts.HEALTH_MONITOR_HTTP_METHOD_DELETE)
+# 'TRACE'
+HEALTH_MONITOR_HTTP_METHOD_TRACE = lib_consts.HEALTH_MONITOR_HTTP_METHOD_TRACE
+# 'OPTIONS'
+HEALTH_MONITOR_HTTP_METHOD_OPTIONS = (
+    lib_consts.HEALTH_MONITOR_HTTP_METHOD_OPTIONS)
+# 'CONNECT'
+HEALTH_MONITOR_HTTP_METHOD_CONNECT = (
+    lib_consts.HEALTH_MONITOR_HTTP_METHOD_CONNECT)
+# 'PATCH'
+HEALTH_MONITOR_HTTP_METHOD_PATCH = lib_consts.HEALTH_MONITOR_HTTP_METHOD_PATCH
 SUPPORTED_HEALTH_MONITOR_HTTP_METHODS = (
-    HEALTH_MONITOR_HTTP_METHOD_GET, HEALTH_MONITOR_HTTP_METHOD_HEAD,
-    HEALTH_MONITOR_HTTP_METHOD_POST, HEALTH_MONITOR_HTTP_METHOD_PUT,
-    HEALTH_MONITOR_HTTP_METHOD_DELETE, HEALTH_MONITOR_HTTP_METHOD_TRACE,
-    HEALTH_MONITOR_HTTP_METHOD_OPTIONS, HEALTH_MONITOR_HTTP_METHOD_CONNECT,
-    HEALTH_MONITOR_HTTP_METHOD_PATCH)
+    lib_consts.SUPPORTED_HEALTH_MONITOR_HTTP_METHODS)
+
+# 'REJECT'
+L7POLICY_ACTION_REJECT = lib_consts.L7POLICY_ACTION_REJECT
+# 'REDIRECT_TO_URL'
+L7POLICY_ACTION_REDIRECT_TO_URL = lib_consts.L7POLICY_ACTION_REDIRECT_TO_URL
+# 'REDIRECT_TO_POOL'
+L7POLICY_ACTION_REDIRECT_TO_POOL = lib_consts.L7POLICY_ACTION_REDIRECT_TO_POOL
+# 'REDIRECT_PREFIX'
+L7POLICY_ACTION_REDIRECT_PREFIX = lib_consts.L7POLICY_ACTION_REDIRECT_PREFIX
+SUPPORTED_L7POLICY_ACTIONS = lib_consts.SUPPORTED_L7POLICY_ACTIONS
+
+# 'REGEX'
+L7RULE_COMPARE_TYPE_REGEX = lib_consts.L7RULE_COMPARE_TYPE_REGEX
+# 'STARTS_WITH'
+L7RULE_COMPARE_TYPE_STARTS_WITH = lib_consts.L7RULE_COMPARE_TYPE_STARTS_WITH
+# 'ENDS_WITH'
+L7RULE_COMPARE_TYPE_ENDS_WITH = lib_consts.L7RULE_COMPARE_TYPE_ENDS_WITH
+# 'CONTAINS'
+L7RULE_COMPARE_TYPE_CONTAINS = lib_consts.L7RULE_COMPARE_TYPE_CONTAINS
+# 'EQUAL_TO'
+L7RULE_COMPARE_TYPE_EQUAL_TO = lib_consts.L7RULE_COMPARE_TYPE_EQUAL_TO
+SUPPORTED_L7RULE_COMPARE_TYPES = lib_consts.SUPPORTED_L7RULE_COMPARE_TYPES
+
+# 'HOST_NAME'
+L7RULE_TYPE_HOST_NAME = lib_consts.L7RULE_TYPE_HOST_NAME
+# 'PATH'
+L7RULE_TYPE_PATH = lib_consts.L7RULE_TYPE_PATH
+# 'FILE_TYPE'
+L7RULE_TYPE_FILE_TYPE = lib_consts.L7RULE_TYPE_FILE_TYPE
+# 'HEADER'
+L7RULE_TYPE_HEADER = lib_consts.L7RULE_TYPE_HEADER
+# 'COOKIE'
+L7RULE_TYPE_COOKIE = lib_consts.L7RULE_TYPE_COOKIE
+# 'SSL_CONN_HAS_CERT'
+L7RULE_TYPE_SSL_CONN_HAS_CERT = lib_consts.L7RULE_TYPE_SSL_CONN_HAS_CERT
+# 'SSL_VERIFY_RESULT'
+L7RULE_TYPE_SSL_VERIFY_RESULT = lib_consts.L7RULE_TYPE_SSL_VERIFY_RESULT
+# 'SSL_DN_FIELD'
+L7RULE_TYPE_SSL_DN_FIELD = lib_consts.L7RULE_TYPE_SSL_DN_FIELD
+SUPPORTED_L7RULE_TYPES = lib_consts.SUPPORTED_L7RULE_TYPES
+
+# 'ROUND_ROBIN'
+LB_ALGORITHM_ROUND_ROBIN = lib_consts.LB_ALGORITHM_ROUND_ROBIN
+# 'LEAST_CONNECTIONS'
+LB_ALGORITHM_LEAST_CONNECTIONS = lib_consts.LB_ALGORITHM_LEAST_CONNECTIONS
+# 'SOURCE_IP'
+LB_ALGORITHM_SOURCE_IP = lib_consts.LB_ALGORITHM_SOURCE_IP
+SUPPORTED_LB_ALGORITHMS = lib_consts.SUPPORTED_LB_ALGORITHMS
+
+# 'operating_status'
+OPERATING_STATUS = lib_consts.OPERATING_STATUS
+# 'ONLINE'
+ONLINE = lib_consts.ONLINE
+# 'OFFLINE'
+OFFLINE = lib_consts.OFFLINE
+# 'DEGRADED'
+DEGRADED = lib_consts.DEGRADED
+# 'ERROR'
+ERROR = lib_consts.ERROR
+# 'DRAINING'
+DRAINING = lib_consts.DRAINING
+# 'NO_MONITOR'
+NO_MONITOR = lib_consts.NO_MONITOR
+# 'operating_status'
+SUPPORTED_OPERATING_STATUSES = lib_consts.SUPPORTED_OPERATING_STATUSES
+
+# 'TCP'
+PROTOCOL_TCP = lib_consts.PROTOCOL_TCP
+# 'UDP'
+PROTOCOL_UDP = lib_consts.PROTOCOL_UDP
+# 'HTTP'
+PROTOCOL_HTTP = lib_consts.PROTOCOL_HTTP
+# 'HTTPS'
+PROTOCOL_HTTPS = lib_consts.PROTOCOL_HTTPS
+# 'TERMINATED_HTTPS'
+PROTOCOL_TERMINATED_HTTPS = lib_consts.PROTOCOL_TERMINATED_HTTPS
+# 'PROXY'
+PROTOCOL_PROXY = lib_consts.PROTOCOL_PROXY
+SUPPORTED_PROTOCOLS = lib_consts.SUPPORTED_PROTOCOLS
+
+# 'provisioning_status'
+PROVISIONING_STATUS = lib_consts.PROVISIONING_STATUS
+# Amphora has been allocated to a load balancer 'ALLOCATED'
+AMPHORA_ALLOCATED = lib_consts.AMPHORA_ALLOCATED
+# Amphora is being built 'BOOTING'
+AMPHORA_BOOTING = lib_consts.AMPHORA_BOOTING
+# Amphora is ready to be allocated to a load balancer 'READY'
+AMPHORA_READY = lib_consts.AMPHORA_READY
+# 'ACTIVE'
+ACTIVE = lib_consts.ACTIVE
+# 'PENDING_DELETE'
+PENDING_DELETE = lib_consts.PENDING_DELETE
+# 'PENDING_UPDATE'
+PENDING_UPDATE = lib_consts.PENDING_UPDATE
+# 'PENDING_CREATE'
+PENDING_CREATE = lib_consts.PENDING_CREATE
+# 'DELETED'
+DELETED = lib_consts.DELETED
+SUPPORTED_PROVISIONING_STATUSES = lib_consts.SUPPORTED_PROVISIONING_STATUSES
+
+# 'SOURCE_IP'
+SESSION_PERSISTENCE_SOURCE_IP = lib_consts.SESSION_PERSISTENCE_SOURCE_IP
+# 'HTTP_COOKIE'
+SESSION_PERSISTENCE_HTTP_COOKIE = lib_consts.SESSION_PERSISTENCE_HTTP_COOKIE
+# 'APP_COOKIE'
+SESSION_PERSISTENCE_APP_COOKIE = lib_consts.SESSION_PERSISTENCE_APP_COOKIE
+SUPPORTED_SP_TYPES = lib_consts.SUPPORTED_SP_TYPES
+
+# List of HTTP headers which are supported for insertion
+SUPPORTED_HTTP_HEADERS = lib_consts.SUPPORTED_HTTP_HEADERS
+
+# List of SSL headers for client certificate
+SUPPORTED_SSL_HEADERS = lib_consts.SUPPORTED_SSL_HEADERS
+
+###############################################################################
+
 HEALTH_MONITOR_DEFAULT_EXPECTED_CODES = '200'
+HEALTH_MONITOR_HTTP_DEFAULT_METHOD = lib_consts.HEALTH_MONITOR_HTTP_METHOD_GET
 HEALTH_MONITOR_DEFAULT_URL_PATH = '/'
 TYPE = 'type'
 URL_PATH = 'url_path'
@@ -66,23 +202,12 @@ RISE_THRESHOLD = 'rise_threshold'
 UPDATE_STATS = 'UPDATE_STATS'
 UPDATE_HEALTH = 'UPDATE_HEALTH'
 
-PROTOCOL_TCP = 'TCP'
-PROTOCOL_UDP = 'UDP'
-PROTOCOL_HTTP = 'HTTP'
-PROTOCOL_HTTPS = 'HTTPS'
-PROTOCOL_TERMINATED_HTTPS = 'TERMINATED_HTTPS'
-PROTOCOL_PROXY = 'PROXY'
-SUPPORTED_PROTOCOLS = (PROTOCOL_TCP, PROTOCOL_HTTPS, PROTOCOL_HTTP,
-                       PROTOCOL_TERMINATED_HTTPS, PROTOCOL_PROXY, PROTOCOL_UDP)
-
 # API Integer Ranges
 MIN_PORT_NUMBER = 1
 MAX_PORT_NUMBER = 65535
 
-DEFAULT_CONNECTION_LIMIT = -1
 MIN_CONNECTION_LIMIT = -1
 
-DEFAULT_WEIGHT = 1
 MIN_WEIGHT = 0
 MAX_WEIGHT = 256
 
@@ -98,74 +223,26 @@ DEFAULT_TIMEOUT_MEMBER_CONNECT = 5000
 DEFAULT_TIMEOUT_MEMBER_DATA = 50000
 DEFAULT_TIMEOUT_TCP_INSPECT = 0
 
+MUTABLE_STATUSES = (lib_consts.ACTIVE,)
+DELETABLE_STATUSES = (lib_consts.ACTIVE, lib_consts.ERROR)
+FAILOVERABLE_STATUSES = (lib_consts.ACTIVE, lib_consts.ERROR)
+
 # Note: The database Amphora table has a foreign key constraint against
 #       the provisioning_status table
-# Amphora has been allocated to a load balancer
-AMPHORA_ALLOCATED = 'ALLOCATED'
-# Amphora is being built
-AMPHORA_BOOTING = 'BOOTING'
-# Amphora is ready to be allocated to a load balancer
-AMPHORA_READY = 'READY'
-
-ACTIVE = 'ACTIVE'
-PENDING_DELETE = 'PENDING_DELETE'
-PENDING_UPDATE = 'PENDING_UPDATE'
-PENDING_CREATE = 'PENDING_CREATE'
-DELETED = 'DELETED'
-ERROR = 'ERROR'
-SUPPORTED_PROVISIONING_STATUSES = (ACTIVE, AMPHORA_ALLOCATED,
-                                   AMPHORA_BOOTING, AMPHORA_READY,
-                                   PENDING_DELETE, PENDING_CREATE,
-                                   PENDING_UPDATE, DELETED, ERROR)
-MUTABLE_STATUSES = (ACTIVE,)
-DELETABLE_STATUSES = (ACTIVE, ERROR)
-FAILOVERABLE_STATUSES = (ACTIVE, ERROR)
-
-SUPPORTED_AMPHORA_STATUSES = (AMPHORA_ALLOCATED, AMPHORA_BOOTING, ERROR,
-                              AMPHORA_READY, DELETED,
-                              PENDING_CREATE, PENDING_DELETE)
-
-ONLINE = 'ONLINE'
-OFFLINE = 'OFFLINE'
-DEGRADED = 'DEGRADED'
-ERROR = 'ERROR'
-DRAINING = 'DRAINING'
-NO_MONITOR = 'NO_MONITOR'
-OPERATING_STATUS = 'operating_status'
-PROVISIONING_STATUS = 'provisioning_status'
-SUPPORTED_OPERATING_STATUSES = (ONLINE, OFFLINE, DEGRADED, ERROR, DRAINING,
-                                NO_MONITOR)
+SUPPORTED_AMPHORA_STATUSES = (
+    lib_consts.AMPHORA_ALLOCATED, lib_consts.AMPHORA_BOOTING, lib_consts.ERROR,
+    lib_consts.AMPHORA_READY, lib_consts.DELETED, lib_consts.PENDING_CREATE,
+    lib_consts.PENDING_DELETE)
 
 AMPHORA_VM = 'VM'
 SUPPORTED_AMPHORA_TYPES = (AMPHORA_VM,)
 
-# L7 constants
-L7RULE_TYPE_HOST_NAME = 'HOST_NAME'
-L7RULE_TYPE_PATH = 'PATH'
-L7RULE_TYPE_FILE_TYPE = 'FILE_TYPE'
-L7RULE_TYPE_HEADER = 'HEADER'
-L7RULE_TYPE_COOKIE = 'COOKIE'
-SUPPORTED_L7RULE_TYPES = (L7RULE_TYPE_HOST_NAME, L7RULE_TYPE_PATH,
-                          L7RULE_TYPE_FILE_TYPE, L7RULE_TYPE_HEADER,
-                          L7RULE_TYPE_COOKIE)
+DISTINGUISHED_NAME_FIELD_REGEX = lib_consts.DISTINGUISHED_NAME_FIELD_REGEX
 
-L7RULE_COMPARE_TYPE_REGEX = 'REGEX'
-L7RULE_COMPARE_TYPE_STARTS_WITH = 'STARTS_WITH'
-L7RULE_COMPARE_TYPE_ENDS_WITH = 'ENDS_WITH'
-L7RULE_COMPARE_TYPE_CONTAINS = 'CONTAINS'
-L7RULE_COMPARE_TYPE_EQUAL_TO = 'EQUAL_TO'
-SUPPORTED_L7RULE_COMPARE_TYPES = (L7RULE_COMPARE_TYPE_REGEX,
-                                  L7RULE_COMPARE_TYPE_STARTS_WITH,
-                                  L7RULE_COMPARE_TYPE_ENDS_WITH,
-                                  L7RULE_COMPARE_TYPE_CONTAINS,
-                                  L7RULE_COMPARE_TYPE_EQUAL_TO)
+# For redirect, only codes 301, 302, 303, 307 and 308 are # supported.
+SUPPORTED_L7POLICY_REDIRECT_HTTP_CODES = [301, 302, 303, 307, 308]
 
-L7POLICY_ACTION_REJECT = 'REJECT'
-L7POLICY_ACTION_REDIRECT_TO_URL = 'REDIRECT_TO_URL'
-L7POLICY_ACTION_REDIRECT_TO_POOL = 'REDIRECT_TO_POOL'
-SUPPORTED_L7POLICY_ACTIONS = (L7POLICY_ACTION_REJECT,
-                              L7POLICY_ACTION_REDIRECT_TO_URL,
-                              L7POLICY_ACTION_REDIRECT_TO_POOL)
+SUPPORTED_HTTP_VERSIONS = [1.0, 1.1]
 
 MIN_POLICY_POSITION = 1
 # Largest a 32-bit integer can be, which is a limitation
@@ -194,6 +271,9 @@ HTTP_HEADER_VALUE_REGEX = (r'\A[a-zA-Z0-9'
 HTTP_QUOTED_HEADER_VALUE_REGEX = (r'\A"[a-zA-Z0-9 \t'
                                   r'!"#$%&\'()*+,-./:;<=>?@[\]^_`{|}~\\]*"\Z')
 
+DOMAIN_NAME_REGEX = (
+    r'^(?=.{1,253}\.?$)(?:(?!-|[^.]+_)[A-Za-z0-9-_]{1,63}(?<!-)(?:\.|$))+$')
+
 # Task/Flow constants
 AMPHORA = 'amphora'
 FAILED_AMPHORA = 'failed_amphora'
@@ -207,17 +287,14 @@ DELTAS = 'deltas'
 HEALTH_MON = 'health_mon'
 HEALTH_MONITOR = 'health_monitor'
 LISTENER = 'listener'
-LISTENERS = 'listeners'
 LISTENER_ID = 'listener_id'
 LOADBALANCER = 'loadbalancer'
-LOADBALANCERS = 'loadbalancers'
 LOADBALANCER_ID = 'loadbalancer_id'
 LOAD_BALANCER_ID = 'load_balancer_id'
 SERVER_GROUP_ID = 'server_group_id'
 ANTI_AFFINITY = 'anti-affinity'
 SOFT_ANTI_AFFINITY = 'soft-anti-affinity'
 MEMBER = 'member'
-MEMBERS = 'members'
 MEMBER_ID = 'member_id'
 COMPUTE_ID = 'compute_id'
 COMPUTE_OBJ = 'compute_obj'
@@ -226,7 +303,6 @@ AMPS_DATA = 'amps_data'
 NICS = 'nics'
 VIP = 'vip'
 POOL = 'pool'
-POOLS = 'pools'
 POOL_CHILD_COUNT = 'pool_child_count'
 POOL_ID = 'pool_id'
 L7POLICY = 'l7policy'
@@ -235,16 +311,14 @@ OBJECT = 'object'
 SERVER_PEM = 'server_pem'
 UPDATE_DICT = 'update_dict'
 VIP_NETWORK = 'vip_network'
+AMPHORA_NETWORK_CONFIG = 'amphora_network_config'
 AMPHORAE_NETWORK_CONFIG = 'amphorae_network_config'
 ADDED_PORTS = 'added_ports'
 PORTS = 'ports'
 MEMBER_PORTS = 'member_ports'
-LOADBALANCER_TOPOLOGY = 'topology'
-HEALTHMONITORS = 'healthmonitors'
+TOPOLOGY = 'topology'
 HEALTH_MONITOR_ID = 'health_monitor_id'
-L7POLICIES = 'l7policies'
 L7POLICY_ID = 'l7policy_id'
-L7RULES = 'l7rules'
 L7RULE_ID = 'l7rule_id'
 LOAD_BALANCER_UPDATES = 'load_balancer_updates'
 LISTENER_UPDATES = 'listener_updates'
@@ -258,6 +332,13 @@ REQ_CONN_TIMEOUT = 'req_conn_timeout'
 REQ_READ_TIMEOUT = 'req_read_timeout'
 CONN_MAX_RETRIES = 'conn_max_retries'
 CONN_RETRY_INTERVAL = 'conn_retry_interval'
+SUBNET = 'subnet'
+AMP_DATA = 'amp_data'
+ACTIVE_CONNECTIONS = 'active_connections'
+BYTES_IN = 'bytes_in'
+BYTES_OUT = 'bytes_out'
+REQUEST_ERRORS = 'request_errors'
+TOTAL_CONNECTIONS = 'total_connections'
 
 CERT_ROTATE_AMPHORA_FLOW = 'octavia-cert-rotate-amphora-flow'
 CREATE_AMPHORA_FLOW = 'octavia-create-amphora-flow'
@@ -292,9 +373,11 @@ UPDATE_POOL_FLOW = 'octavia-update-pool-flow'
 UPDATE_L7POLICY_FLOW = 'octavia-update-l7policy-flow'
 UPDATE_L7RULE_FLOW = 'octavia-update-l7rule-flow'
 UPDATE_AMPS_SUBFLOW = 'octavia-update-amps-subflow'
+UPDATE_AMPHORA_CONFIG_FLOW = 'octavia-update-amp-config-flow'
 
 POST_MAP_AMP_TO_LB_SUBFLOW = 'octavia-post-map-amp-to-lb-subflow'
 CREATE_AMP_FOR_LB_SUBFLOW = 'octavia-create-amp-for-lb-subflow'
+AMP_PLUG_NET_SUBFLOW = 'octavia-plug-net-subflow'
 GET_AMPHORA_FOR_LB_SUBFLOW = 'octavia-get-amphora-for-lb-subflow'
 POST_LB_AMP_ASSOCIATION_SUBFLOW = (
     'octavia-post-loadbalancer-amp_association-subflow')
@@ -327,11 +410,15 @@ CREATE_VRRP_GROUP_FOR_LB = 'octavia-create-vrrp-group-for-lb'
 CREATE_VRRP_SECURITY_RULES = 'octavia-create-vrrp-security-rules'
 AMP_COMPUTE_CONNECTIVITY_WAIT = 'octavia-amp-compute-connectivity-wait'
 AMP_LISTENER_UPDATE = 'octavia-amp-listeners-update'
-
+PLUG_VIP_AMPHORA = 'octavia-amp-plug-vip'
+APPLY_QOS_AMP = 'octavia-amp-apply-qos'
+UPDATE_AMPHORA_VIP_DATA = 'ocatvia-amp-update-vip-data'
+GET_AMP_NETWORK_CONFIG = 'octavia-amp-get-network-config'
+AMP_POST_VIP_PLUG = 'octavia-amp-post-vip-plug'
 GENERATE_SERVER_PEM_TASK = 'GenerateServerPEMTask'
+AMPHORA_CONFIG_UPDATE_TASK = 'AmphoraConfigUpdateTask'
 
 # Batch Member Update constants
-MEMBERS = 'members'
 UNORDERED_MEMBER_UPDATES_FLOW = 'octavia-unordered-member-updates-flow'
 UNORDERED_MEMBER_ACTIVE_FLOW = 'octavia-unordered-member-active-flow'
 UPDATE_ATTRIBUTES_FLOW = 'octavia-update-attributes-flow'
@@ -344,9 +431,11 @@ UPDATE_MEMBER_INDB = 'octavia-update-member-indb'
 DELETE_MEMBER_INDB = 'octavia-delete-member-indb'
 
 # Task Names
+RELOAD_AMP_AFTER_PLUG_VIP = 'reload-amp-after-plug-vip'
 RELOAD_LB_AFTER_AMP_ASSOC = 'reload-lb-after-amp-assoc'
 RELOAD_LB_AFTER_AMP_ASSOC_FULL_GRAPH = 'reload-lb-after-amp-assoc-full-graph'
 RELOAD_LB_AFTER_PLUG_VIP = 'reload-lb-after-plug-vip'
+RELOAD_LB_BEFOR_ALLOCATE_VIP = "reload-lb-before-allocate-vip"
 
 NOVA_1 = '1.1'
 NOVA_21 = '2.1'
@@ -391,7 +480,6 @@ DEFAULT_VRRP_ID = 1
 VRRP_PROTOCOL_NUM = 112
 AUTH_HEADER_PROTOCOL_NUMBER = 51
 
-
 TEMPLATES = '/templates'
 AGENT_API_TEMPLATES = '/templates'
 
@@ -447,11 +535,6 @@ PLUGGED_INTERFACES = '/var/lib/octavia/plugged_interfaces'
 HAPROXY_USER_GROUP_CFG = '/var/lib/octavia/haproxy-default-user-group.conf'
 AMPHORA_NAMESPACE = 'amphora-haproxy'
 
-# List of HTTP headers which are supported for insertion
-SUPPORTED_HTTP_HEADERS = ['X-Forwarded-For',
-                          'X-Forwarded-Port',
-                          'X-Forwarded-Proto']
-
 FLOW_DOC_TITLES = {'AmphoraFlows': 'Amphora Flows',
                    'LoadBalancerFlows': 'Load Balancer Flows',
                    'ListenerFlows': 'Listener Flows',
@@ -467,7 +550,6 @@ SYSCTL_CMD = '/sbin/sysctl'
 AMP_ACTION_START = 'start'
 AMP_ACTION_STOP = 'stop'
 AMP_ACTION_RELOAD = 'reload'
-AMP_ACTION_RESTART = 'restart'
 GLANCE_IMAGE_ACTIVE = 'active'
 
 INIT_SYSTEMD = 'systemd'
@@ -531,8 +613,12 @@ RBAC_L7RULE = '{}:l7rule:'.format(LOADBALANCER_API)
 RBAC_QUOTA = '{}:quota:'.format(LOADBALANCER_API)
 RBAC_AMPHORA = '{}:amphora:'.format(LOADBALANCER_API)
 RBAC_PROVIDER = '{}:provider:'.format(LOADBALANCER_API)
+RBAC_PROVIDER_FLAVOR = '{}:provider-flavor:'.format(LOADBALANCER_API)
+RBAC_FLAVOR = '{}:flavor:'.format(LOADBALANCER_API)
+RBAC_FLAVOR_PROFILE = '{}:flavor-profile:'.format(LOADBALANCER_API)
 RBAC_POST = 'post'
 RBAC_PUT = 'put'
+RBAC_PUT_CONFIG = 'put_config'
 RBAC_PUT_FAILOVER = 'put_failover'
 RBAC_DELETE = 'delete'
 RBAC_GET_ONE = 'get_one'
@@ -543,16 +629,31 @@ RBAC_GET_STATS = 'get_stats'
 RBAC_GET_STATUS = 'get_status'
 
 # PROVIDERS
-# TODO(johnsom) When providers are implemented, this should be removed.
 OCTAVIA = 'octavia'
 
-# FLAVORS
-# TODO(johnsom) When flavors are implemented, this should be removed.
-SUPPORTED_FLAVORS = ()
-
 # systemctl commands
 DISABLE = 'disable'
 ENABLE = 'enable'
 
 # systemd amphora netns service prefix
 AMP_NETNS_SVC_PREFIX = 'amphora-netns'
+
+# Amphora Feature Compatibility
+HTTP_REUSE = 'has_http_reuse'
+
+# TODO(johnsom) convert this to octavia_lib constant flavor
+# once octavia is transitioned to use octavia_lib
+FLAVOR = 'flavor'
+FLAVOR_DATA = 'flavor_data'
+
+# Flavor metadata
+LOADBALANCER_TOPOLOGY = 'loadbalancer_topology'
+COMPUTE_FLAVOR = 'compute_flavor'
+
+# TODO(johnsom) move to octavia_lib
+# client certification authorization option
+CLIENT_AUTH_NONE = 'NONE'
+CLIENT_AUTH_OPTIONAL = 'OPTIONAL'
+CLIENT_AUTH_MANDATORY = 'MANDATORY'
+SUPPORTED_CLIENT_AUTH_MODES = [CLIENT_AUTH_NONE, CLIENT_AUTH_OPTIONAL,
+                               CLIENT_AUTH_MANDATORY]
