@@ -26,6 +26,7 @@ import netaddr
 from oslo_config import cfg
 import rfc3986
 import six
+from wsme import types as wtypes
 
 from octavia.common import constants
 from octavia.common import exceptions
@@ -106,8 +107,6 @@ def regex(regex):
 # L7 rules must be internally consistent.
 def l7rule_data(l7rule):
     """Raises an error if the l7rule given is invalid in some way."""
-    if not l7rule.value:
-        raise exceptions.InvalidL7Rule(msg=_('L7 rule type requires a value'))
     if l7rule.type == constants.L7RULE_TYPE_HEADER:
         if not l7rule.key:
             raise exceptions.InvalidL7Rule(msg='L7 rule type requires a key')
@@ -163,11 +162,72 @@ def l7rule_data(l7rule):
             raise exceptions.InvalidL7Rule(msg='invalid comparison type '
                                            'for rule type')
 
+    elif l7rule.type in [constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+                         constants.L7RULE_TYPE_SSL_VERIFY_RESULT,
+                         constants.L7RULE_TYPE_SSL_DN_FIELD]:
+        validate_l7rule_ssl_types(l7rule)
+
     else:
         raise exceptions.InvalidL7Rule(msg='invalid rule type')
     return True
 
 
+def validate_l7rule_ssl_types(l7rule):
+    if not l7rule.type or l7rule.type not in [
+       constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+       constants.L7RULE_TYPE_SSL_VERIFY_RESULT,
+       constants.L7RULE_TYPE_SSL_DN_FIELD]:
+        return
+
+    rule_type = None if l7rule.type == wtypes.Unset else l7rule.type
+    req_key = None if l7rule.key == wtypes.Unset else l7rule.key
+    req_value = None if l7rule.value == wtypes.Unset else l7rule.value
+    compare_type = (None if l7rule.compare_type == wtypes.Unset else
+                    l7rule.compare_type)
+    msg = None
+    if rule_type == constants.L7RULE_TYPE_SSL_CONN_HAS_CERT:
+        # key and value are not allowed
+        if req_key:
+            # log error or raise
+            msg = 'L7rule type {0} does not use the "key" field.'.format(
+                rule_type)
+        elif req_value.lower() != 'true':
+            msg = 'L7rule value {0} is not a boolean True string.'.format(
+                req_value)
+        elif compare_type != constants.L7RULE_COMPARE_TYPE_EQUAL_TO:
+            msg = 'L7rule type {0} only supports the {1} compare type.'.format(
+                rule_type, constants.L7RULE_COMPARE_TYPE_EQUAL_TO)
+
+    if rule_type == constants.L7RULE_TYPE_SSL_VERIFY_RESULT:
+        if req_key:
+            # log or raise req_key not used
+            msg = 'L7rule type {0} does not use the "key" field.'.format(
+                rule_type)
+        elif not req_value.isdigit() or int(req_value) < 0:
+            # log or raise req_value must be int
+            msg = 'L7rule type {0} needs a int value, which is >= 0'.format(
+                rule_type)
+        elif compare_type != constants.L7RULE_COMPARE_TYPE_EQUAL_TO:
+            msg = 'L7rule type {0} only supports the {1} compare type.'.format(
+                rule_type, constants.L7RULE_COMPARE_TYPE_EQUAL_TO)
+
+    if rule_type == constants.L7RULE_TYPE_SSL_DN_FIELD:
+        dn_regex = re.compile(constants.DISTINGUISHED_NAME_FIELD_REGEX)
+        if compare_type == constants.L7RULE_COMPARE_TYPE_REGEX:
+            regex(l7rule.value)
+
+        if not req_key or not req_value:
+            # log or raise key and value must be specified.
+            msg = 'L7rule type {0} needs to specify a key and a value.'.format(
+                rule_type)
+        # log or raise the key must be splited by '-'
+        elif not dn_regex.match(req_key):
+            msg = ('Invalid L7rule distinguished name field.')
+
+    if msg:
+        raise exceptions.InvalidL7Rule(msg=msg)
+
+
 def sanitize_l7policy_api_args(l7policy, create=False):
     """Validate and make consistent L7Policy API arguments.
 
@@ -196,30 +256,50 @@ def sanitize_l7policy_api_args(l7policy, create=False):
                 raise exceptions.InvalidL7PolicyArgs(
                     msg='redirect_pool_id or redirect_pool must not be None')
             l7policy.update({'redirect_url': None})
+        elif l7policy['action'] == constants.L7POLICY_ACTION_REDIRECT_PREFIX:
+            if not l7policy.get('redirect_prefix'):
+                raise exceptions.InvalidL7PolicyArgs(
+                    msg='redirect_prefix must not be None')
         else:
             raise exceptions.InvalidL7PolicyAction(
                 action=l7policy['action'])
-    if ((l7policy.get('redirect_pool_id') or
-            l7policy.get('redirect_pool')) and l7policy.get('redirect_url')):
+    if ((l7policy.get('redirect_pool_id') or l7policy.get('redirect_pool')) and
+            (l7policy.get('redirect_url') or l7policy.get('redirect_prefix'))):
         raise exceptions.InvalidL7PolicyArgs(
-            msg='Cannot specify redirect_pool_id and redirect_url '
-                'at the same time')
+            msg='Cannot specify redirect_pool_id and redirect_url or '
+                'redirect_prefix at the same time')
     if l7policy.get('redirect_pool_id'):
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL})
         l7policy.update({'redirect_url': None})
         l7policy.pop('redirect_pool', None)
+        l7policy.update({'redirect_prefix': None})
+        l7policy.update({'redirect_http_code': None})
     if l7policy.get('redirect_pool'):
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL})
         l7policy.update({'redirect_url': None})
         l7policy.pop('redirect_pool_id', None)
+        l7policy.update({'redirect_prefix': None})
+        l7policy.update({'redirect_http_code': None})
     if l7policy.get('redirect_url'):
         url(l7policy['redirect_url'])
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL})
         l7policy.update({'redirect_pool_id': None})
+        l7policy.update({'redirect_prefix': None})
         l7policy.pop('redirect_pool', None)
+        if not l7policy.get('redirect_http_code'):
+            l7policy.update({'redirect_http_code': 302})
+    if l7policy.get('redirect_prefix'):
+        url(l7policy['redirect_prefix'])
+        l7policy.update({
+            'action': constants.L7POLICY_ACTION_REDIRECT_PREFIX})
+        l7policy.update({'redirect_pool_id': None})
+        l7policy.update({'redirect_url': None})
+        l7policy.pop('redirect_pool', None)
+        if not l7policy.get('redirect_http_code'):
+            l7policy.update({'redirect_http_code': 302})
 
     # If we are creating, we need an action at this point
     if create and 'action' not in l7policy.keys():
@@ -341,3 +421,11 @@ def ip_not_reserved(ip_address):
     if ip_address in CONF.networking.reserved_ips:
         raise exceptions.InvalidOption(value=ip_address,
                                        option='member address')
+
+
+def is_flavor_spares_compatible(flavor):
+    if flavor:
+        # If a compute flavor is specified, the flavor is not spares compatible
+        if flavor.get(constants.COMPUTE_FLAVOR, None):
+            return False
+    return True
