@@ -12,14 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from concurrent import futures
 import datetime
 
-from concurrent import futures
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import timeutils
 from sqlalchemy.orm import exc as sqlalchemy_exceptions
 
-from octavia.common import constants
 from octavia.controller.worker import controller_worker as cw
 from octavia.db import api as db_api
 from octavia.db import repositories as repo
@@ -31,6 +31,7 @@ CONF = cfg.CONF
 class SpareAmphora(object):
     def __init__(self):
         self.amp_repo = repo.AmphoraRepository()
+        self.spares_repo = repo.SparesPoolRepository()
         self.cw = cw.ControllerWorker()
 
     def spare_check(self):
@@ -38,24 +39,42 @@ class SpareAmphora(object):
 
         If it's less than the requirement, starts new amphora.
         """
+        lock_session = db_api.get_session(autocommit=False)
         session = db_api.get_session()
-        conf_spare_cnt = CONF.house_keeping.spare_amphora_pool_size
-        curr_spare_cnt = self.amp_repo.get_spare_amphora_count(session)
-        LOG.debug("Required Spare Amphora count : %d", conf_spare_cnt)
-        LOG.debug("Current Spare Amphora count : %d", curr_spare_cnt)
-        diff_count = conf_spare_cnt - curr_spare_cnt
+        try:
+            # Lock the spares_pool record for read and write
+            spare_amp_row = self.spares_repo.get_for_update(lock_session)
 
-        # When the current spare amphora is less than required
-        if diff_count > 0:
-            LOG.info("Initiating creation of %d spare amphora.", diff_count)
+            conf_spare_cnt = CONF.house_keeping.spare_amphora_pool_size
+            curr_spare_cnt = self.amp_repo.get_spare_amphora_count(session)
+            LOG.debug("Required Spare Amphora count : %d", conf_spare_cnt)
+            LOG.debug("Current Spare Amphora count : %d", curr_spare_cnt)
+            diff_count = conf_spare_cnt - curr_spare_cnt
 
-            # Call Amphora Create Flow diff_count times
-            for i in range(1, diff_count + 1):
-                LOG.debug("Starting amphorae number %d ...", i)
-                self.cw.create_amphora()
+            # When the current spare amphora is less than required
+            amp_booting = []
+            if diff_count > 0:
+                LOG.info("Initiating creation of %d spare amphora.",
+                         diff_count)
 
-        else:
-            LOG.debug("Current spare amphora count satisfies the requirement")
+                # Call Amphora Create Flow diff_count times
+                with futures.ThreadPoolExecutor(
+                        max_workers=CONF.house_keeping.spare_amphora_pool_size
+                ) as executor:
+                    for i in range(1, diff_count + 1):
+                        LOG.debug("Starting amphorae number %d ...", i)
+                        amp_booting.append(
+                            executor.submit(self.cw.create_amphora))
+            else:
+                LOG.debug("Current spare amphora count satisfies the "
+                          "requirement")
+
+            # Wait for the amphora boot threads to finish
+            futures.wait(amp_booting)
+            spare_amp_row.updated_at = timeutils.utcnow()
+            lock_session.commit()
+        except Exception:
+            lock_session.rollback()
 
 
 class DatabaseCleanup(object):
@@ -70,18 +89,24 @@ class DatabaseCleanup(object):
             seconds=CONF.house_keeping.amphora_expiry_age)
 
         session = db_api.get_session()
-        amphora, _ = self.amp_repo.get_all(session, status=constants.DELETED)
+        amp_ids = self.amp_repo.get_all_deleted_expiring(session,
+                                                         exp_age=exp_age)
 
-        for amp in amphora:
-            if self.amp_health_repo.check_amphora_expired(session, amp.id,
-                                                          exp_age):
-                LOG.info('Attempting to delete Amphora id : %s', amp.id)
-                self.amp_repo.delete(session, id=amp.id)
+        for amp_id in amp_ids:
+            # If we're here, we already think the amp is expiring according to
+            # the amphora table. Now check it is expired in the health table.
+            # In this way, we ensure that amps aren't deleted unless they are
+            # both expired AND no longer receiving zombie heartbeats.
+            if self.amp_health_repo.check_amphora_health_expired(
+                    session, amp_id, exp_age):
+                LOG.debug('Attempting to purge db record for Amphora ID: %s',
+                          amp_id)
+                self.amp_repo.delete(session, id=amp_id)
                 try:
-                    self.amp_health_repo.delete(session, amphora_id=amp.id)
+                    self.amp_health_repo.delete(session, amphora_id=amp_id)
                 except sqlalchemy_exceptions.NoResultFound:
                     pass  # Best effort delete, this record might not exist
-                LOG.info('Deleted Amphora id : %s', amp.id)
+                LOG.info('Purged db record for Amphora ID: %s', amp_id)
 
     def cleanup_load_balancers(self):
         """Checks the DB for old load balancers and triggers their removal."""
@@ -89,15 +114,13 @@ class DatabaseCleanup(object):
             seconds=CONF.house_keeping.load_balancer_expiry_age)
 
         session = db_api.get_session()
-        load_balancers, _ = self.lb_repo.get_all(
-            session, provisioning_status=constants.DELETED)
+        lb_ids = self.lb_repo.get_all_deleted_expiring(session,
+                                                       exp_age=exp_age)
 
-        for lb in load_balancers:
-            if self.lb_repo.check_load_balancer_expired(session, lb.id,
-                                                        exp_age):
-                LOG.info('Attempting to delete load balancer id : %s', lb.id)
-                self.lb_repo.delete(session, id=lb.id)
-                LOG.info('Deleted load balancer id : %s', lb.id)
+        for lb_id in lb_ids:
+            LOG.info('Attempting to delete load balancer id : %s', lb_id)
+            self.lb_repo.delete(session, id=lb_id)
+            LOG.info('Deleted load balancer id : %s', lb_id)
 
 
 class CertRotation(object):
