@@ -69,6 +69,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         self._pool_repo = repo.PoolRepository()
         self._l7policy_repo = repo.L7PolicyRepository()
         self._l7rule_repo = repo.L7RuleRepository()
+        self._flavor_repo = repo.FlavorRepository()
 
         super(ControllerWorker, self).__init__()
 
@@ -90,16 +91,19 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         :returns: amphora_id
         """
-        create_amp_tf = self._taskflow_load(
-            self._amphora_flows.get_create_amphora_flow(),
-            store={constants.BUILD_TYPE_PRIORITY:
-                   constants.LB_CREATE_SPARES_POOL_PRIORITY}
-        )
-        with tf_logging.DynamicLoggingListener(create_amp_tf, log=LOG):
+        try:
+            create_amp_tf = self._taskflow_load(
+                self._amphora_flows.get_create_amphora_flow(),
+                store={constants.BUILD_TYPE_PRIORITY:
+                       constants.LB_CREATE_SPARES_POOL_PRIORITY,
+                       constants.FLAVOR: None}
+            )
+            with tf_logging.DynamicLoggingListener(create_amp_tf, log=LOG):
+                create_amp_tf.run()
 
-            create_amp_tf.run()
-
-        return create_amp_tf.storage.fetch('amphora')
+            return create_amp_tf.storage.fetch('amphora')
+        except Exception as e:
+            LOG.error('Failed to create an amphora due to: {}'.format(str(e)))
 
     def delete_amphora(self, amphora_id):
         """Deletes an existing Amphora.
@@ -301,7 +305,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         wait=tenacity.wait_incrementing(
             RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
         stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
-    def create_load_balancer(self, load_balancer_id):
+    def create_load_balancer(self, load_balancer_id, flavor=None):
         """Creates a load balancer by allocating Amphorae.
 
         First tries to allocate an existing Amphora in READY state.
@@ -318,14 +322,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                         '60 seconds.', 'load_balancer', load_balancer_id)
             raise db_exceptions.NoResultFound
 
+        # TODO(johnsom) convert this to octavia_lib constant flavor
+        # once octavia is transitioned to use octavia_lib
         store = {constants.LOADBALANCER_ID: load_balancer_id,
                  constants.BUILD_TYPE_PRIORITY:
-                 constants.LB_CREATE_NORMAL_PRIORITY}
+                 constants.LB_CREATE_NORMAL_PRIORITY,
+                 constants.FLAVOR: flavor}
 
-        topology = CONF.controller_worker.loadbalancer_topology
+        topology = lb.topology
 
         store[constants.UPDATE_DICT] = {
-            constants.LOADBALANCER_TOPOLOGY: topology
+            constants.TOPOLOGY: topology
         }
 
         create_lb_flow = self._lb_flows.get_create_load_balancer_flow(
@@ -795,22 +802,6 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                          constants.LOADBALANCER_ID: amp.load_balancer_id,
                          constants.BUILD_TYPE_PRIORITY: priority, }
 
-        if amp.role in (constants.ROLE_MASTER, constants.ROLE_BACKUP):
-            amp_role = 'master_or_backup'
-        elif amp.role == constants.ROLE_STANDALONE:
-            amp_role = 'standalone'
-        elif amp.role is None:
-            amp_role = 'spare'
-        else:
-            amp_role = 'undefined'
-
-        LOG.info("Perform failover for an amphora: %s",
-                 {"id": amp.id,
-                  "load_balancer_id": amp.load_balancer_id,
-                  "lb_network_ip": amp.lb_network_ip,
-                  "compute_id": amp.compute_id,
-                  "role": amp_role})
-
         if amp.status == constants.DELETED:
             LOG.warning('Amphora %s is marked DELETED in the database but '
                         'was submitted for failover. Deleting it from the '
@@ -832,6 +823,12 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             db_apis.get_session(), amp.id)
         if CONF.nova.enable_anti_affinity and lb:
             stored_params[constants.SERVER_GROUP_ID] = lb.server_group_id
+        if lb.flavor_id:
+            stored_params[constants.FLAVOR] = (
+                self._flavor_repo.get_flavor_metadata_dict(
+                    db_apis.get_session(), lb.flavor_id))
+        else:
+            stored_params[constants.FLAVOR] = {}
 
         failover_amphora_tf = self._taskflow_load(
             self._amphora_flows.get_failover_flow(
@@ -841,13 +838,6 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         with tf_logging.DynamicLoggingListener(failover_amphora_tf, log=LOG):
             failover_amphora_tf.run()
 
-        LOG.info("Successfully completed the failover for an amphora: %s",
-                 {"id": amp.id,
-                  "load_balancer_id": amp.load_balancer_id,
-                  "lb_network_ip": amp.lb_network_ip,
-                  "compute_id": amp.compute_id,
-                  "role": amp_role})
-
     def failover_amphora(self, amphora_id):
         """Perform failover operations for an amphora.
 
@@ -878,8 +868,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             except Exception:
                 LOG.error("Unable to revert LB status to ERROR.")
             with excutils.save_and_reraise_exception():
-                LOG.error("Amphora %(id)s failover exception: %(exc)s",
-                          {'id': amphora_id, 'exc': e})
+                LOG.error("Failover exception: %s", e)
 
     def failover_loadbalancer(self, load_balancer_id):
         """Perform failover operations for a load balancer.
@@ -944,3 +933,32 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         with tf_logging.DynamicLoggingListener(certrotation_amphora_tf,
                                                log=LOG):
             certrotation_amphora_tf.run()
+
+    def update_amphora_agent_config(self, amphora_id):
+        """Update the amphora agent configuration.
+
+        Note: This will update the amphora agent configuration file and
+              update the running configuration for mutatable configuration
+              items.
+
+        :param amphora_id: ID of the amphora to update.
+        :returns: None
+        """
+        LOG.info("Start amphora agent configuration update, amphora's id "
+                 "is: %s", amphora_id)
+        amp = self._amphora_repo.get(db_apis.get_session(), id=amphora_id)
+        lb = self._amphora_repo.get_lb_for_amphora(db_apis.get_session(),
+                                                   amphora_id)
+        flavor = {}
+        if lb.flavor_id:
+            flavor = self._flavor_repo.get_flavor_metadata_dict(
+                db_apis.get_session(), lb.flavor_id)
+
+        update_amphora_tf = self._taskflow_load(
+            self._amphora_flows.update_amphora_config_flow(),
+            store={constants.AMPHORA: amp,
+                   constants.FLAVOR: flavor})
+
+        with tf_logging.DynamicLoggingListener(update_amphora_tf,
+                                               log=LOG):
+            update_amphora_tf.run()
