@@ -28,7 +28,6 @@ from octavia.controller.worker.tasks import amphora_driver_tasks
 from octavia.controller.worker.tasks import compute_tasks
 from octavia.controller.worker.tasks import database_tasks
 from octavia.controller.worker.tasks import lifecycle_tasks
-from octavia.controller.worker.tasks import model_tasks
 from octavia.controller.worker.tasks import network_tasks
 
 CONF = cfg.CONF
@@ -56,6 +55,24 @@ class LoadBalancerFlows(object):
         lb_create_flow.add(lifecycle_tasks.LoadBalancerIDToErrorOnRevertTask(
             requires=constants.LOADBALANCER_ID))
 
+        # allocate VIP
+        lb_create_flow.add(database_tasks.ReloadLoadBalancer(
+            name=constants.RELOAD_LB_BEFOR_ALLOCATE_VIP,
+            requires=constants.LOADBALANCER_ID,
+            provides=constants.LOADBALANCER
+        ))
+        lb_create_flow.add(network_tasks.AllocateVIP(
+            requires=constants.LOADBALANCER,
+            provides=constants.VIP))
+        lb_create_flow.add(database_tasks.UpdateVIPAfterAllocation(
+            requires=(constants.LOADBALANCER_ID, constants.VIP),
+            provides=constants.LOADBALANCER))
+        lb_create_flow.add(network_tasks.UpdateVIPSecurityGroup(
+            requires=constants.LOADBALANCER))
+        lb_create_flow.add(network_tasks.GetSubnetFromVIP(
+            requires=constants.LOADBALANCER,
+            provides=constants.SUBNET))
+
         if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
             lb_create_flow.add(*self._create_active_standby_topology())
         elif topology == constants.TOPOLOGY_SINGLE:
@@ -108,7 +125,8 @@ class LoadBalancerFlows(object):
         f_name = constants.CREATE_LOADBALANCER_FLOW
         amps_flow = unordered_flow.Flow(f_name)
         master_amp_sf = self.amp_flows.get_amphora_for_lb_subflow(
-            prefix=constants.ROLE_MASTER, role=constants.ROLE_MASTER)
+            prefix=constants.ROLE_MASTER, role=constants.ROLE_MASTER
+        )
 
         backup_amp_sf = self.amp_flows.get_amphora_for_lb_subflow(
             prefix=constants.ROLE_BACKUP, role=constants.ROLE_BACKUP)
@@ -173,9 +191,6 @@ class LoadBalancerFlows(object):
                 requires=constants.LOADBALANCER_ID,
                 provides=constants.LOADBALANCER))
 
-        new_LB_net_subflow = self.get_new_LB_networking_subflow()
-        post_create_LB_flow.add(new_LB_net_subflow)
-
         if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
             vrrp_subflow = self.amp_flows.get_vrrp_subflow(prefix)
             post_create_LB_flow.add(vrrp_subflow)
@@ -212,32 +227,7 @@ class LoadBalancerFlows(object):
 
         :returns: The flow for deleting a load balancer
         """
-        (listeners_delete, store) = self._get_delete_listeners_flow(lb)
-
-        delete_LB_flow = linear_flow.Flow(constants.DELETE_LOADBALANCER_FLOW)
-        delete_LB_flow.add(lifecycle_tasks.LoadBalancerToErrorOnRevertTask(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(compute_tasks.NovaServerGroupDelete(
-            requires=constants.SERVER_GROUP_ID))
-        delete_LB_flow.add(database_tasks.MarkLBAmphoraeHealthBusy(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(listeners_delete)
-        delete_LB_flow.add(network_tasks.UnplugVIP(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(network_tasks.DeallocateVIP(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(compute_tasks.DeleteAmphoraeOnLoadBalancer(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(database_tasks.MarkLBAmphoraeDeletedInDB(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(database_tasks.DisableLBAmphoraeHealthMonitoring(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(database_tasks.MarkLBDeletedInDB(
-            requires=constants.LOADBALANCER))
-        delete_LB_flow.add(database_tasks.DecrementLoadBalancerQuota(
-            requires=constants.LOADBALANCER))
-
-        return (delete_LB_flow, store)
+        return self._get_delete_load_balancer_flow(lb, False)
 
     def _get_delete_pools_flow(self, lb):
         """Sets up an internal delete flow
@@ -258,16 +248,8 @@ class LoadBalancerFlows(object):
                     pool_name))
         return (pools_delete_flow, store)
 
-    def get_cascade_delete_load_balancer_flow(self, lb):
-        """Creates a flow to delete a load balancer.
-
-        :returns: The flow for deleting a load balancer
-        """
-
-        (listeners_delete, store) = self._get_delete_listeners_flow(lb)
-        (pools_delete, pool_store) = self._get_delete_pools_flow(lb)
-        store.update(pool_store)
-
+    def _get_delete_load_balancer_flow(self, lb, cascade):
+        store = {}
         delete_LB_flow = linear_flow.Flow(constants.DELETE_LOADBALANCER_FLOW)
         delete_LB_flow.add(lifecycle_tasks.LoadBalancerToErrorOnRevertTask(
             requires=constants.LOADBALANCER))
@@ -275,8 +257,12 @@ class LoadBalancerFlows(object):
             requires=constants.SERVER_GROUP_ID))
         delete_LB_flow.add(database_tasks.MarkLBAmphoraeHealthBusy(
             requires=constants.LOADBALANCER))
-        delete_LB_flow.add(pools_delete)
-        delete_LB_flow.add(listeners_delete)
+        if cascade:
+            (listeners_delete, store) = self._get_delete_listeners_flow(lb)
+            (pools_delete, pool_store) = self._get_delete_pools_flow(lb)
+            store.update(pool_store)
+            delete_LB_flow.add(pools_delete)
+            delete_LB_flow.add(listeners_delete)
         delete_LB_flow.add(network_tasks.UnplugVIP(
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(network_tasks.DeallocateVIP(
@@ -291,9 +277,15 @@ class LoadBalancerFlows(object):
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(database_tasks.DecrementLoadBalancerQuota(
             requires=constants.LOADBALANCER))
-
         return (delete_LB_flow, store)
 
+    def get_cascade_delete_load_balancer_flow(self, lb):
+        """Creates a flow to delete a load balancer.
+
+        :returns: The flow for deleting a load balancer
+        """
+        return self._get_delete_load_balancer_flow(lb, True)
+
     def get_new_LB_networking_subflow(self):
         """Create a sub-flow to setup networking.
 
@@ -314,7 +306,7 @@ class LoadBalancerFlows(object):
         new_LB_net_subflow.add(network_tasks.ApplyQos(
             requires=(constants.LOADBALANCER, constants.AMPS_DATA,
                       constants.UPDATE_DICT)))
-        new_LB_net_subflow.add(database_tasks.UpdateAmphoraVIPData(
+        new_LB_net_subflow.add(database_tasks.UpdateAmphoraeVIPData(
             requires=constants.AMPS_DATA))
         new_LB_net_subflow.add(database_tasks.ReloadLoadBalancer(
             name=constants.RELOAD_LB_AFTER_PLUG_VIP,
@@ -337,11 +329,6 @@ class LoadBalancerFlows(object):
         update_LB_flow = linear_flow.Flow(constants.UPDATE_LOADBALANCER_FLOW)
         update_LB_flow.add(lifecycle_tasks.LoadBalancerToErrorOnRevertTask(
             requires=constants.LOADBALANCER))
-        update_LB_flow.add(model_tasks.
-                           UpdateAttributes(
-                               rebind={constants.OBJECT:
-                                       constants.LOADBALANCER},
-                               requires=[constants.UPDATE_DICT]))
         update_LB_flow.add(network_tasks.ApplyQos(
             requires=(constants.LOADBALANCER, constants.UPDATE_DICT)))
         update_LB_flow.add(amphora_driver_tasks.ListenersUpdate(
