@@ -13,13 +13,14 @@
 # under the License.
 #
 
+from cryptography import fernet
 from oslo_config import cfg
 from stevedore import driver as stevedore_driver
 from taskflow import task
 
+from octavia.common import utils
 
 CONF = cfg.CONF
-CONF.import_group('controller_worker', 'octavia.common.config')
 CERT_VALIDITY = 2 * 365 * 24 * 60 * 60
 
 
@@ -30,7 +31,7 @@ class BaseCertTask(task.Task):
         super(BaseCertTask, self).__init__(**kwargs)
         self.cert_generator = stevedore_driver.DriverManager(
             namespace='octavia.cert_generator',
-            name=CONF.controller_worker.cert_generator,
+            name=CONF.certificates.cert_generator,
             invoke_on_load=True,
         ).driver
 
@@ -45,5 +46,7 @@ class GenerateServerPEMTask(BaseCertTask):
         cert = self.cert_generator.generate_cert_key_pair(
             cn=amphora_id,
             validity=CERT_VALIDITY)
+        key = utils.get_six_compatible_server_certs_key_passphrase()
+        fer = fernet.Fernet(key)
 
-        return cert.certificate + cert.private_key
+        return fer.encrypt(cert.certificate + cert.private_key)
