@@ -31,7 +31,7 @@ class OctaviaBase(models.ModelBase):
         # objects.
         if obj.__class__.__name__ in ['Member', 'Pool', 'LoadBalancer',
                                       'Listener', 'Amphora', 'L7Policy',
-                                      'L7Rule']:
+                                      'L7Rule', 'Flavor', 'FlavorProfile']:
             return obj.__class__.__name__ + obj.id
         elif obj.__class__.__name__ in ['SessionPersistence', 'HealthMonitor']:
             return obj.__class__.__name__ + obj.pool_id
@@ -107,14 +107,17 @@ class OctaviaBase(models.ModelBase):
     def apply_filter(query, model, filters):
         translated_filters = {}
         child_map = {}
-        # Convert admin_state_up to proper type
-        if 'admin_state_up' in filters:
-            filters['admin_state_up'] = strutils.bool_from_string(
-                filters['admin_state_up'])
+        # Convert enabled to proper type
+        if 'enabled' in filters:
+            filters['enabled'] = strutils.bool_from_string(
+                filters['enabled'])
         for attr, name_map in model.__v2_wsme__._child_map.items():
             for k, v in name_map.items():
-                if v in filters:
-                    child_map[attr] = {k: filters.pop(v)}
+                if attr in filters and k in filters[attr]:
+                    child_map.setdefault(attr, {}).update(
+                        {k: filters[attr].pop(k)})
+            filters.pop(attr, None)
+
         for k, v in model.__v2_wsme__._type_to_model_map.items():
             if k in filters:
                 translated_filters[v] = filters.pop(k)
@@ -148,4 +151,35 @@ class NameMixin(object):
     name = sa.Column(sa.String(255), nullable=True)
 
 
+class TagMixin(object):
+    """Tags mixin to add to classes which need tags.
+
+    The class must realize the specified db relationship as well.
+    """
+
+    @property
+    def tags(self):
+        if self._tags:
+            return [each_tag.tag for each_tag in self._tags]
+        return []
+
+    @tags.setter
+    def tags(self, values):
+        new_tags = []
+        if values:
+            for tag in values:
+                tag_ref = Tags()
+                tag_ref.resource_id = self.id
+                tag_ref.tag = tag
+                new_tags.append(tag_ref)
+        self._tags = new_tags
+
+
 BASE = declarative.declarative_base(cls=OctaviaBase)
+
+
+class Tags(BASE):
+    __tablename__ = "tags"
+
+    resource_id = sa.Column(sa.String(36), primary_key=True)
+    tag = sa.Column(sa.String(255), primary_key=True, index=True)
