@@ -24,6 +24,7 @@ from oslo_config import cfg
 from oslo_db import api as oslo_db_api
 from oslo_db import exception as db_exception
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
 from sqlalchemy.orm import joinedload
@@ -51,7 +52,18 @@ class BaseRepository(object):
         :param filters: Filters to decide which entities should be retrieved.
         :returns: int
         """
-        return session.query(self.model_class).filter_by(**filters).count()
+        deleted = filters.pop('show_deleted', True)
+        model = session.query(self.model_class).filter_by(**filters)
+
+        if not deleted:
+            if hasattr(self.model_class, 'status'):
+                model = model.filter(
+                    self.model_class.status != consts.DELETED)
+            else:
+                model = model.filter(
+                    self.model_class.provisioning_status != consts.DELETED)
+
+        return model.count()
 
     def create(self, session, **model_kwargs):
         """Base create method for a database entity.
@@ -92,6 +104,10 @@ class BaseRepository(object):
         :returns: octavia.common.data_model
         """
         with session.begin(subtransactions=True):
+            tags = model_kwargs.pop('tags', None)
+            if tags:
+                resource = session.query(self.model_class).get(id)
+                resource.tags = tags
             session.query(self.model_class).filter_by(
                 id=id).update(model_kwargs)
 
@@ -208,6 +224,9 @@ class Repositories(object):
         self.amp_build_slots = AmphoraBuildSlotsRepository()
         self.amp_build_req = AmphoraBuildReqRepository()
         self.quotas = QuotasRepository()
+        self.flavor = FlavorRepository()
+        self.flavor_profile = FlavorProfileRepository()
+        self.spares_pool = SparesPoolRepository()
 
     def create_load_balancer_and_vip(self, session, lb_dict, vip_dict):
         """Inserts load balancer and vip entities into the database.
@@ -698,6 +717,34 @@ class Repositories(object):
         session.expire_all()
         return self.load_balancer.get(session, id=lb_dm.id)
 
+    def get_amphora_stats(self, session, amp_id):
+        """Gets the statistics for all listeners on an amphora.
+
+        :param session: A Sql Alchemy database session.
+        :param amp_id: The amphora ID to query.
+        :returns: An amphora stats dictionary
+        """
+        with session.begin(subtransactions=True):
+            columns = (models.ListenerStatistics.__table__.columns +
+                       [models.Amphora.load_balancer_id])
+            amp_records = (
+                session.query(*columns)
+                .filter(models.ListenerStatistics.amphora_id == amp_id)
+                .filter(models.ListenerStatistics.amphora_id ==
+                        models.Amphora.id).all())
+            amp_stats = []
+            for amp in amp_records:
+                amp_stat = {consts.LOADBALANCER_ID: amp.load_balancer_id,
+                            consts.LISTENER_ID: amp.listener_id,
+                            'id': amp.amphora_id,
+                            consts.ACTIVE_CONNECTIONS: amp.active_connections,
+                            consts.BYTES_IN: amp.bytes_in,
+                            consts.BYTES_OUT: amp.bytes_out,
+                            consts.REQUEST_ERRORS: amp.request_errors,
+                            consts.TOTAL_CONNECTIONS: amp.total_connections}
+                amp_stats.append(amp_stat)
+            return amp_stats
+
 
 class LoadBalancerRepository(BaseRepository):
     model_class = models.LoadBalancer
@@ -722,6 +769,7 @@ class LoadBalancerRepository(BaseRepository):
             subqueryload(models.LoadBalancer.amphorae),
             subqueryload(models.LoadBalancer.pools),
             subqueryload(models.LoadBalancer.listeners),
+            subqueryload(models.LoadBalancer._tags),
             noload('*'))
 
         return super(LoadBalancerRepository, self).get_all(
@@ -818,6 +866,7 @@ class HealthMonitorRepository(BaseRepository):
         # no-load (blank) the tables we don't need
         query_options = (
             subqueryload(models.HealthMonitor.pool),
+            subqueryload(models.HealthMonitor._tags),
             noload('*'))
 
         return super(HealthMonitorRepository, self).get_all(
@@ -869,6 +918,7 @@ class PoolRepository(BaseRepository):
             subqueryload(models.Pool.load_balancer),
             subqueryload(models.Pool.members),
             subqueryload(models.Pool.session_persistence),
+            subqueryload(models.Pool._tags),
             noload('*'))
 
         return super(PoolRepository, self).get_all(
@@ -896,6 +946,7 @@ class MemberRepository(BaseRepository):
         # no-load (blank) the tables we don't need
         query_options = (
             subqueryload(models.Member.pool),
+            subqueryload(models.Member._tags),
             noload('*'))
 
         return super(MemberRepository, self).get_all(
@@ -941,6 +992,7 @@ class ListenerRepository(BaseRepository):
             subqueryload(models.Listener.l7policies),
             subqueryload(models.Listener.load_balancer),
             subqueryload(models.Listener.sni_containers),
+            subqueryload(models.Listener._tags),
             noload('*'))
 
         return super(ListenerRepository, self).get_all(
@@ -987,6 +1039,10 @@ class ListenerRepository(BaseRepository):
 
     def update(self, session, id, **model_kwargs):
         with session.begin(subtransactions=True):
+            tags = model_kwargs.pop('tags', None)
+            if tags:
+                resource = session.query(self.model_class).get(id)
+                resource.tags = tags
             listener_db = session.query(self.model_class).filter_by(
                 id=id).first()
             # Verify any newly specified default_pool_id exists
@@ -1081,7 +1137,6 @@ class AmphoraRepository(BaseRepository):
                 id=amphora_id).first()
             load_balancer.amphorae.append(amphora)
 
-    @oslo_db_api.wrap_db_retry(max_retries=5, retry_on_deadlock=True)
     def allocate_and_associate(self, session, load_balancer_id):
         """Allocate an amphora for a load balancer.
 
@@ -1197,7 +1252,6 @@ class AmphoraRepository(BaseRepository):
             "load_balancer.operating_status AS lb_op_status, "
             "listener.id AS list_id, "
             "listener.operating_status AS list_op_status, "
-            "listener.protocol AS list_protocol, "
             "pool.id AS pool_id, "
             "pool.operating_status AS pool_op_status, "
             "member.id AS member_id, "
@@ -1221,8 +1275,7 @@ class AmphoraRepository(BaseRepository):
                 lb['provisioning_status'] = row['lb_prov_status']
                 lb['operating_status'] = row['lb_op_status']
             if row['list_id'] and row['list_id'] not in listeners:
-                listener = {'operating_status': row['list_op_status'],
-                            'protocol': row['list_protocol']}
+                listener = {'operating_status': row['list_op_status']}
                 listeners[row['list_id']] = listener
             if row['pool_id']:
                 if row['pool_id'] in pools and row['member_id']:
@@ -1438,6 +1491,7 @@ class L7RuleRepository(BaseRepository):
         # no-load (blank) the tables we don't need
         query_options = (
             subqueryload(models.L7Rule.l7policy),
+            subqueryload(models.L7Rule._tags),
             noload('*'))
 
         return super(L7RuleRepository, self).get_all(
@@ -1474,10 +1528,6 @@ class L7RuleRepository(BaseRepository):
         with session.begin(subtransactions=True):
             if not model_kwargs.get('id'):
                 model_kwargs.update(id=uuidutils.generate_uuid())
-            if model_kwargs.get('l7policy_id'):
-                l7policy_db = session.query(models.L7Policy).filter_by(
-                    id=model_kwargs.get('l7policy_id')).first()
-                model_kwargs.update(l7policy=l7policy_db)
             l7rule = self.model_class(**model_kwargs)
             validate.l7rule_data(l7rule)
             session.add(l7rule)
@@ -1537,6 +1587,7 @@ class L7PolicyRepository(BaseRepository):
             subqueryload(models.L7Policy.l7rules),
             subqueryload(models.L7Policy.listener),
             subqueryload(models.L7Policy.redirect_pool),
+            subqueryload(models.L7Policy._tags),
             noload('*'))
 
         if not deleted:
@@ -1577,12 +1628,21 @@ class L7PolicyRepository(BaseRepository):
                 if l7policy.action == consts.L7POLICY_ACTION_REJECT:
                     model_kwargs.update(redirect_url=None)
                     model_kwargs.update(redirect_pool_id=None)
+                    model_kwargs.update(redirect_prefix=None)
+                    model_kwargs.update(redirect_http_code=None)
                 elif (l7policy.action ==
                         consts.L7POLICY_ACTION_REDIRECT_TO_URL):
                     model_kwargs.update(redirect_pool_id=None)
+                    model_kwargs.update(redirect_prefix=None)
                 elif (l7policy.action ==
                         consts.L7POLICY_ACTION_REDIRECT_TO_POOL):
                     model_kwargs.update(redirect_url=None)
+                    model_kwargs.update(redirect_prefix=None)
+                    model_kwargs.update(redirect_http_code=None)
+                elif (l7policy.action ==
+                        consts.L7POLICY_ACTION_REDIRECT_PREFIX):
+                    model_kwargs.update(redirect_url=None)
+                    model_kwargs.update(redirect_pool_id=None)
 
             l7policy_db.update(model_kwargs)
 
@@ -1616,10 +1676,6 @@ class L7PolicyRepository(BaseRepository):
                 pool_db = session.query(models.Pool).filter_by(
                     id=model_kwargs.get('redirect_pool_id')).first()
                 model_kwargs.update(redirect_pool=pool_db)
-            if model_kwargs.get('listener_id'):
-                listener_db = session.query(models.Listener).filter_by(
-                    id=model_kwargs.get('listener_id')).first()
-                model_kwargs.update(listener=listener_db)
             l7policy = self.model_class(
                 **validate.sanitize_l7policy_api_args(model_kwargs,
                                                       create=True))
@@ -1703,3 +1759,44 @@ class QuotasRepository(BaseRepository):
             quotas.member = None
             quotas.pool = None
             session.flush()
+
+
+class FlavorRepository(BaseRepository):
+    model_class = models.Flavor
+
+    def get_flavor_metadata_dict(self, session, flavor_id):
+        with session.begin(subtransactions=True):
+            flavor_metadata_json = (
+                session.query(models.FlavorProfile.flavor_data)
+                .filter(models.Flavor.id == flavor_id)
+                .filter(
+                    models.Flavor.flavor_profile_id == models.FlavorProfile.id)
+                .one()[0])
+            result_dict = ({} if flavor_metadata_json is None
+                           else jsonutils.loads(flavor_metadata_json))
+            return result_dict
+
+    def get_flavor_provider(self, session, flavor_id):
+        with session.begin(subtransactions=True):
+            return (session.query(models.FlavorProfile.provider_name)
+                    .filter(models.Flavor.id == flavor_id)
+                    .filter(models.Flavor.flavor_profile_id ==
+                            models.FlavorProfile.id).one()[0])
+
+
+class FlavorProfileRepository(BaseRepository):
+    model_class = models.FlavorProfile
+
+
+class SparesPoolRepository(BaseRepository):
+    model_class = models.SparesPool
+
+    def get_for_update(self, lock_session):
+        """Queries and locks the SparesPool record.
+
+        This call will query for the SparesPool table record and lock it
+        so that other processes cannot read or write it.
+        :returns: expected_spares_count, updated_at
+        """
+        row = lock_session.query(models.SparesPool).with_for_update().one()
+        return row
