@@ -20,8 +20,8 @@ from novaclient import exceptions as nova_client_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 import six
+from stevedore import driver as stevedore_driver
 
-from octavia.common import clients
 from octavia.common import constants
 from octavia.common import data_models
 from octavia.common import exceptions
@@ -45,14 +45,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
     def __init__(self):
         super(AllowedAddressPairsDriver, self).__init__()
         self._check_aap_loaded()
-        self.nova_client = clients.NovaAuth.get_nova_client(
-            endpoint=CONF.nova.endpoint,
-            region=CONF.nova.region_name,
-            endpoint_type=CONF.nova.endpoint_type,
-            service_name=CONF.nova.service_name,
-            insecure=CONF.nova.insecure,
-            cacert=CONF.nova.ca_certificates_file
-        )
+        self.compute = stevedore_driver.DriverManager(
+            namespace='octavia.compute.drivers',
+            name=CONF.controller_worker.compute_driver,
+            invoke_on_load=True
+        ).driver
 
     def _check_aap_loaded(self):
         if not self._check_extension_enabled(AAP_EXT_ALIAS):
@@ -99,13 +96,6 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             LOG.debug('Created vip port: %(port_id)s for amphora: %(amp)s',
                       {'port_id': new_port.id, 'amp': amphora.id})
 
-        except Exception:
-            message = _('Error creating the base (VRRP) port for the VIP with '
-                        'port details: {}').format(port)
-            LOG.exception(message)
-            raise base.PlugVIPException(message)
-
-        try:
             interface = self.plug_port(amphora, new_port)
         except Exception:
             message = _('Error plugging amphora (compute_id: {compute_id}) '
@@ -113,16 +103,6 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                             compute_id=amphora.compute_id,
                             network_id=subnet.network_id)
             LOG.exception(message)
-            try:
-                if new_port:
-                    self.neutron_client.delete_port(new_port.id)
-                    LOG.debug('Deleted base (VRRP) port %s due to plug_port '
-                              'failure.', new_port.id)
-            except Exception:
-                LOG.exception('Failed to delete base (VRRP) port %s after '
-                              'plug_port failed. This resource is being '
-                              'abandoned and should be manually deleted when '
-                              'neutron is functional.', new_port.id)
             raise base.PlugVIPException(message)
         return interface
 
@@ -205,8 +185,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         # Currently we are using the VIP network for VRRP
         # so we need to open up the protocols for it
-        if (CONF.controller_worker.loadbalancer_topology ==
-                constants.TOPOLOGY_ACTIVE_STANDBY):
+        if load_balancer.topology == constants.TOPOLOGY_ACTIVE_STANDBY:
             try:
                 self._create_security_group_rule(
                     sec_grp_id,
@@ -368,38 +347,45 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             LOG.info("Port %s will not be deleted by Octavia as it was "
                      "not created by Octavia.", vip.port_id)
 
-    def plug_vip(self, load_balancer, vip):
+    def update_vip_sg(self, load_balancer, vip):
         if self.sec_grp_enabled:
             self._update_vip_security_group(load_balancer, vip)
+
+    def plug_aap_port(self, load_balancer, vip, amphora, subnet):
+        interface = self._get_plugged_interface(
+            amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
+        if not interface:
+            interface = self._plug_amphora_vip(amphora, subnet)
+
+        self._add_vip_address_pair(interface.port_id, vip.ip_address)
+        if self.sec_grp_enabled:
+            self._add_vip_security_group_to_port(load_balancer.id,
+                                                 interface.port_id)
+        vrrp_ip = None
+        for fixed_ip in interface.fixed_ips:
+            is_correct_subnet = fixed_ip.subnet_id == subnet.id
+            is_management_ip = fixed_ip.ip_address == amphora.lb_network_ip
+            if is_correct_subnet and not is_management_ip:
+                vrrp_ip = fixed_ip.ip_address
+                break
+        return data_models.Amphora(
+            id=amphora.id,
+            compute_id=amphora.compute_id,
+            vrrp_ip=vrrp_ip,
+            ha_ip=vip.ip_address,
+            vrrp_port_id=interface.port_id,
+            ha_port_id=vip.port_id)
+
+    # todo (xgerman): Delete later
+    def plug_vip(self, load_balancer, vip):
+        self.update_vip_sg(load_balancer, vip)
         plugged_amphorae = []
         subnet = self.get_subnet(vip.subnet_id)
         for amphora in six.moves.filter(
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 load_balancer.amphorae):
-
-            interface = self._get_plugged_interface(
-                amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
-            if not interface:
-                interface = self._plug_amphora_vip(amphora, subnet)
-
-            self._add_vip_address_pair(interface.port_id, vip.ip_address)
-            if self.sec_grp_enabled:
-                self._add_vip_security_group_to_port(load_balancer.id,
-                                                     interface.port_id)
-            vrrp_ip = None
-            for fixed_ip in interface.fixed_ips:
-                is_correct_subnet = fixed_ip.subnet_id == subnet.id
-                is_management_ip = fixed_ip.ip_address == amphora.lb_network_ip
-                if is_correct_subnet and not is_management_ip:
-                    vrrp_ip = fixed_ip.ip_address
-                    break
-            plugged_amphorae.append(data_models.Amphora(
-                id=amphora.id,
-                compute_id=amphora.compute_id,
-                vrrp_ip=vrrp_ip,
-                ha_ip=vip.ip_address,
-                vrrp_port_id=interface.port_id,
-                ha_port_id=vip.port_id))
+            plugged_amphorae.append(self.plug_aap_port(load_balancer, vip,
+                                                       amphora, subnet))
         return plugged_amphorae
 
     def allocate_vip(self, load_balancer):
@@ -446,6 +432,45 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         new_port = utils.convert_port_dict_to_model(new_port)
         return self._port_to_vip(new_port, load_balancer)
 
+    def unplug_aap_port(self, vip, amphora, subnet):
+        interface = self._get_plugged_interface(
+            amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
+        if not interface:
+            # Thought about raising PluggedVIPNotFound exception but
+            # then that wouldn't evaluate all amphorae, so just continue
+            LOG.debug('Cannot get amphora %s interface, skipped',
+                      amphora.compute_id)
+            return
+        try:
+            self.unplug_network(amphora.compute_id, subnet.network_id)
+        except Exception:
+            pass
+        try:
+            aap_update = {'port': {
+                'allowed_address_pairs': []
+            }}
+            self.neutron_client.update_port(interface.port_id,
+                                            aap_update)
+        except Exception:
+            message = _('Error unplugging VIP. Could not clear '
+                        'allowed address pairs from port '
+                        '{port_id}.').format(port_id=vip.port_id)
+            LOG.exception(message)
+            raise base.UnplugVIPException(message)
+
+        # Delete the VRRP port if we created it
+        try:
+            port = self.get_port(amphora.vrrp_port_id)
+            if port.name.startswith('octavia-lb-vrrp-'):
+                self.neutron_client.delete_port(amphora.vrrp_port_id)
+        except (neutron_client_exceptions.NotFound,
+                neutron_client_exceptions.PortNotFoundClient):
+            pass
+        except Exception as e:
+            LOG.error('Failed to delete port.  Resources may still be in '
+                      'use for port: %(port)s due to error: %s(except)s',
+                      {'port': amphora.vrrp_port_id, 'except': e})
+
     def unplug_vip(self, load_balancer, vip):
         try:
             subnet = self.get_subnet(vip.subnet_id)
@@ -455,52 +480,15 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             LOG.exception(msg)
             raise base.PluggedVIPNotFound(msg)
         for amphora in six.moves.filter(
-            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
+                lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 load_balancer.amphorae):
-
-            interface = self._get_plugged_interface(
-                amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
-            if not interface:
-                # Thought about raising PluggedVIPNotFound exception but
-                # then that wouldn't evaluate all amphorae, so just continue
-                LOG.debug('Cannot get amphora %s interface, skipped',
-                          amphora.compute_id)
-                continue
-            try:
-                self.unplug_network(amphora.compute_id, subnet.network_id)
-            except Exception:
-                pass
-            try:
-                aap_update = {'port': {
-                    'allowed_address_pairs': []
-                }}
-                self.neutron_client.update_port(interface.port_id,
-                                                aap_update)
-            except Exception:
-                message = _('Error unplugging VIP. Could not clear '
-                            'allowed address pairs from port '
-                            '{port_id}.').format(port_id=vip.port_id)
-                LOG.exception(message)
-                raise base.UnplugVIPException(message)
-
-            # Delete the VRRP port if we created it
-            try:
-                port = self.get_port(amphora.vrrp_port_id)
-                if port.name.startswith('octavia-lb-vrrp-'):
-                    self.neutron_client.delete_port(amphora.vrrp_port_id)
-            except (neutron_client_exceptions.NotFound,
-                    neutron_client_exceptions.PortNotFoundClient):
-                pass
-            except Exception as e:
-                LOG.error('Failed to delete port.  Resources may still be in '
-                          'use for port: %(port)s due to error: %s(except)s',
-                          {'port': amphora.vrrp_port_id, 'except': e})
+            self.unplug_aap_port(vip, amphora, subnet)
 
     def plug_network(self, compute_id, network_id, ip_address=None):
         try:
-            interface = self.nova_client.servers.interface_attach(
-                server=compute_id, net_id=network_id, fixed_ip=ip_address,
-                port_id=None)
+            interface = self.compute.attach_network_or_port(
+                compute_id=compute_id, network_id=network_id,
+                ip_address=ip_address)
         except nova_client_exceptions.NotFound as e:
             if 'Instance' in str(e):
                 raise base.AmphoraNotFound(str(e))
@@ -528,14 +516,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         unpluggers = self._get_interfaces_to_unplug(interfaces, network_id,
                                                     ip_address=ip_address)
         for index, unplugger in enumerate(unpluggers):
-            try:
-                self.nova_client.servers.interface_detach(
-                    server=compute_id, port_id=unplugger.port_id)
-            except Exception:
-                LOG.warning('Error unplugging port {port_id} from amphora '
-                            'with compute ID {compute_id}. '
-                            'Skipping.'.format(port_id=unplugger.port_id,
-                                               compute_id=compute_id))
+            self.compute.detach_port(
+                compute_id=compute_id, port_id=unplugger.port_id)
 
     def update_vip(self, load_balancer, for_delete=False):
         sec_grp = self._get_lb_security_group(load_balancer.id)
@@ -578,9 +560,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
     def plug_port(self, amphora, port):
         try:
-            interface = self.nova_client.servers.interface_attach(
-                server=amphora.compute_id, net_id=None,
-                fixed_ip=None, port_id=port.id)
+            interface = self.compute.attach_network_or_port(
+                compute_id=amphora.compute_id, network_id=None,
+                ip_address=None, port_id=port.id)
             plugged_interface = self._nova_interface_to_octavia_interface(
                 amphora.compute_id, interface)
         except nova_client_exceptions.NotFound as e:
@@ -609,30 +591,38 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         return plugged_interface
 
-    def get_network_configs(self, loadbalancer):
+    def _get_amp_net_configs(self, amp, amp_configs, vip_subnet, vip_port):
+        if amp.status != constants.DELETED:
+            LOG.debug("Retrieving network details for amphora %s", amp.id)
+            vrrp_port = self.get_port(amp.vrrp_port_id)
+            vrrp_subnet = self.get_subnet(
+                vrrp_port.get_subnet_id(amp.vrrp_ip))
+            vrrp_port.network = self.get_network(vrrp_port.network_id)
+            ha_port = self.get_port(amp.ha_port_id)
+            ha_subnet = self.get_subnet(
+                ha_port.get_subnet_id(amp.ha_ip))
+
+            amp_configs[amp.id] = n_data_models.AmphoraNetworkConfig(
+                amphora=amp,
+                vip_subnet=vip_subnet,
+                vip_port=vip_port,
+                vrrp_subnet=vrrp_subnet,
+                vrrp_port=vrrp_port,
+                ha_subnet=ha_subnet,
+                ha_port=ha_port
+            )
+
+    def get_network_configs(self, loadbalancer, amphora=None):
         vip_subnet = self.get_subnet(loadbalancer.vip.subnet_id)
         vip_port = self.get_port(loadbalancer.vip.port_id)
         amp_configs = {}
-        for amp in loadbalancer.amphorae:
-            if amp.status != constants.DELETED:
-                LOG.debug("Retrieving network details for amphora %s", amp.id)
-                vrrp_port = self.get_port(amp.vrrp_port_id)
-                vrrp_subnet = self.get_subnet(
-                    vrrp_port.get_subnet_id(amp.vrrp_ip))
-                vrrp_port.network = self.get_network(vrrp_port.network_id)
-                ha_port = self.get_port(amp.ha_port_id)
-                ha_subnet = self.get_subnet(
-                    ha_port.get_subnet_id(amp.ha_ip))
-
-                amp_configs[amp.id] = n_data_models.AmphoraNetworkConfig(
-                    amphora=amp,
-                    vip_subnet=vip_subnet,
-                    vip_port=vip_port,
-                    vrrp_subnet=vrrp_subnet,
-                    vrrp_port=vrrp_port,
-                    ha_subnet=ha_subnet,
-                    ha_port=ha_port
-                )
+        if amphora:
+            self._get_amp_net_configs(amphora, amp_configs,
+                                      vip_subnet, vip_port)
+        else:
+            for amp in loadbalancer.amphorae:
+                self._get_amp_net_configs(amp, amp_configs,
+                                          vip_subnet, vip_port)
         return amp_configs
 
     def wait_for_port_detach(self, amphora):
