@@ -18,6 +18,7 @@ from oslo_log import log as logging
 
 from octavia.common import clients
 from octavia.common import data_models
+from octavia.i18n import _
 from octavia.network import base
 from octavia.network import data_models as network_models
 from octavia.network.drivers.neutron import utils
@@ -26,6 +27,7 @@ from octavia.network.drivers.neutron import utils
 LOG = logging.getLogger(__name__)
 DNS_INT_EXT_ALIAS = 'dns-integration'
 SEC_GRP_EXT_ALIAS = 'security-group'
+QOS_EXT_ALIAS = 'qos'
 
 CONF = cfg.CONF
 
@@ -45,6 +47,9 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         self.sec_grp_enabled = self._check_extension_enabled(SEC_GRP_EXT_ALIAS)
         self.dns_integration_enabled = self._check_extension_enabled(
             DNS_INT_EXT_ALIAS)
+        self._qos_enabled = self._check_extension_enabled(QOS_EXT_ALIAS)
+        self.project_id = self.neutron_client.get_auth_info().get(
+            'auth_tenant_id')
 
     def _check_extension_enabled(self, extension_alias):
         if extension_alias in self._check_extension_cache:
@@ -110,10 +115,18 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         try:
             self.neutron_client.update_port(port_id, port_update)
         except neutron_client_exceptions.PortNotFoundClient as e:
-            raise base.PortNotFound(e.message)
+            raise base.PortNotFound(str(e))
         except Exception as e:
             raise base.NetworkException(str(e))
 
+    def _get_ports_by_security_group(self, sec_grp_id):
+        all_ports = self.neutron_client.list_ports(project=self.project_id)
+        filtered_ports = []
+        for port in all_ports.get('ports', []):
+            if sec_grp_id in port.get('security_groups', []):
+                filtered_ports.append(port)
+        return filtered_ports
+
     def _create_security_group(self, name):
         new_sec_grp = {'security_group': {'name': name}}
         sec_grp = self.neutron_client.create_security_group(new_sec_grp)
@@ -134,6 +147,18 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         }
         self.neutron_client.create_security_group_rule(rule)
 
+    def apply_qos_on_port(self, qos_id, port_id):
+        body = {
+            'port':
+                {'qos_policy_id': qos_id}
+        }
+        try:
+            self.neutron_client.update_port(port_id, body)
+        except neutron_client_exceptions.PortNotFoundClient as e:
+            raise base.PortNotFound(str(e))
+        except Exception as e:
+            raise base.NetworkException(str(e))
+
     def get_plugged_networks(self, compute_id):
         # List neutron ports associated with the Amphora
         try:
@@ -155,8 +180,8 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
             message = _('{resource_type} not found '
                         '({resource_type} id: {resource_id}).').format(
                 resource_type=resource_type, resource_id=resource_id)
-            raise getattr(base, '%sNotFound' %
-                          resource_type.capitalize())(message)
+            raise getattr(base, '%sNotFound' % ''.join(
+                [w.capitalize() for w in resource_type.split('_')]))(message)
         except Exception:
             message = _('Error retrieving {resource_type} '
                         '({resource_type} id: {resource_id}.').format(
@@ -164,21 +189,32 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
             LOG.exception(message)
             raise base.NetworkException(message)
 
-    def _get_resource_by_filters(self, resource_type, **filters):
-        # Filter items are unique, return the filtered item from the list.
+    def _get_resources_by_filters(self, resource_type, unique_item=False,
+                                  **filters):
+        """Retrieves item(s) from filters. By default, a list is returned.
+
+        If unique_item set to True, only the first resource is returned.
+        """
         try:
             resource = getattr(self.neutron_client, 'list_%ss' %
                                resource_type)(**filters)
-            resource = resource['%ss' % resource_type][0]
-            return getattr(utils, 'convert_%s_dict_to_model' %
-                           resource_type)(resource)
+            conversion_function = getattr(
+                utils,
+                'convert_%s_dict_to_model' % resource_type)
+            if not resource['%ss' % resource_type]:
+                # no items found
+                raise neutron_client_exceptions.NotFound()
+            elif unique_item:
+                return conversion_function(resource['%ss' % resource_type][0])
+            else:
+                return list(map(conversion_function,
+                                resource['%ss' % resource_type]))
         except neutron_client_exceptions.NotFound:
             message = _('{resource_type} not found '
                         '({resource_type} Filters: {filters}.').format(
                 resource_type=resource_type, filters=filters)
-            LOG.exception(message)
-            raise getattr(base, '%sNotFound' %
-                          resource_type.capitalize())(message)
+            raise getattr(base, '%sNotFound' % ''.join(
+                [w.capitalize() for w in resource_type.split('_')]))(message)
         except Exception:
             message = _('Error retrieving {resource_type} '
                         '({resource_type} Filters: {filters}.').format(
@@ -196,15 +232,24 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         return self._get_resource('port', port_id)
 
     def get_network_by_name(self, network_name):
-        return self._get_resource_by_filters('network', name=network_name)
+        return self._get_resources_by_filters(
+            'network', unique_item=True, name=network_name)
 
     def get_subnet_by_name(self, subnet_name):
-        return self._get_resource_by_filters('subnet', name=subnet_name)
+        return self._get_resources_by_filters(
+            'subnet', unique_item=True, name=subnet_name)
 
     def get_port_by_name(self, port_name):
-        return self._get_resource_by_filters('port', name=port_name)
+        return self._get_resources_by_filters(
+            'port', unique_item=True, name=port_name)
 
     def get_port_by_net_id_device_id(self, network_id, device_id):
-        return self._get_resource_by_filters('port',
-                                             network_id=network_id,
-                                             device_id=device_id)
+        return self._get_resources_by_filters(
+            'port', unique_item=True,
+            network_id=network_id, device_id=device_id)
+
+    def get_qos_policy(self, qos_policy_id):
+        return self._get_resource('qos_policy', qos_policy_id)
+
+    def qos_enabled(self):
+        return self._qos_enabled
