@@ -26,6 +26,7 @@ class NoopManager(object):
     def __init__(self):
         super(NoopManager, self).__init__()
         self.networkconfigconfig = {}
+        self._qos_extension_enabled = True
 
     def allocate_vip(self, loadbalancer):
         LOG.debug("Network %s no-op, allocate_vip loadbalancer %s",
@@ -57,21 +58,41 @@ class NoopManager(object):
         LOG.debug("Network %s no-op, plug_vip loadbalancer %s, vip %s",
                   self.__class__.__name__,
                   loadbalancer.id, vip.ip_address)
+        self.update_vip_sg(loadbalancer, vip)
+        amps = []
+        for amphora in loadbalancer.amphorae:
+            amps.append(self.plug_aap_port(loadbalancer, vip, amphora, None))
         self.networkconfigconfig[(loadbalancer.id,
                                   vip.ip_address)] = (loadbalancer, vip,
                                                       'plug_vip')
-        amps = []
-        for amphora in loadbalancer.amphorae:
-            amps.append(data_models.Amphora(
-                id=amphora.id,
-                compute_id=amphora.compute_id,
-                vrrp_ip='198.51.100.1',
-                ha_ip='198.51.100.1',
-                vrrp_port_id=uuidutils.generate_uuid(),
-                ha_port_id=uuidutils.generate_uuid()
-            ))
         return amps
 
+    def update_vip_sg(self, load_balancer, vip):
+        LOG.debug("Network %s no-op, update_vip_sg loadbalancer %s, vip %s",
+                  self.__class__.__name__,
+                  load_balancer.id, vip.ip_address)
+        self.networkconfigconfig[(load_balancer.id,
+                                  vip.ip_address)] = (load_balancer, vip,
+                                                      'update_vip_sg')
+
+    def plug_aap_port(self, load_balancer, vip, amphora, subnet):
+        LOG.debug("Network %s no-op, plug_aap_port loadbalancer %s, vip %s,"
+                  " amphora %s, subnet %s",
+                  self.__class__.__name__,
+                  load_balancer.id, vip.ip_address, amphora, subnet)
+        self.networkconfigconfig[(amphora.id,
+                                  vip.ip_address)] = (
+            load_balancer, vip, amphora, subnet,
+            'plug_aap_port')
+        return data_models.Amphora(
+            id=amphora.id,
+            compute_id=amphora.compute_id,
+            vrrp_ip='198.51.100.1',
+            ha_ip='198.51.100.1',
+            vrrp_port_id=uuidutils.generate_uuid(),
+            ha_port_id=uuidutils.generate_uuid()
+        )
+
     def unplug_vip(self, loadbalancer, vip):
         LOG.debug("Network %s no-op, unplug_vip loadbalancer %s, vip %s",
                   self.__class__.__name__,
@@ -80,6 +101,15 @@ class NoopManager(object):
                                   vip.ip_address)] = (loadbalancer, vip,
                                                       'unplug_vip')
 
+    def unplug_aap_port(self, vip, amphora, subnet):
+        LOG.debug("Network %s no-op, unplug_aap_port vip %s amp: %s "
+                  "subnet: %s",
+                  self.__class__.__name__,
+                  vip.ip_address, amphora.id, subnet.id)
+        self.networkconfigconfig[(amphora.id,
+                                  vip.ip_address)] = (vip, amphora, subnet,
+                                                      'unplug_aap_port')
+
     def plug_network(self, compute_id, network_id, ip_address=None):
         LOG.debug("Network %s no-op, plug_network compute_id %s, network_id "
                   "%s, ip_address %s", self.__class__.__name__, compute_id,
@@ -175,32 +205,43 @@ class NoopManager(object):
         LOG.debug("failover %s no-op, failover_preparation, amphora id %s",
                   self.__class__.__name__, amphora.id)
 
-    def plug_port(self, compute_id, port):
-        LOG.debug("Network %s no-op, plug_port compute_id %s, port_id "
-                  "%s", self.__class__.__name__, compute_id, port.id)
-        self.networkconfigconfig[(compute_id, port)] = (
-            compute_id, port, 'plug_port')
+    def plug_port(self, amphora, port):
+        LOG.debug("Network %s no-op, plug_port amphora.id %s, port_id "
+                  "%s", self.__class__.__name__, amphora.id, port.id)
+        self.networkconfigconfig[(amphora.id, port.id)] = (
+            amphora, port, 'plug_port')
 
-    def get_network_configs(self, loadbalancer):
-        LOG.debug("Network %s no-op, get_network_configs loadbalancer id %s ",
-                  self.__class__.__name__, loadbalancer.id)
+    def _get_amp_net_configs(self, amp, amp_configs, vip_subnet, vip_port):
+        vrrp_port = self.get_port(amp.vrrp_port_id)
+        ha_port = self.get_port(amp.ha_port_id)
+        amp_configs[amp.id] = network_models.AmphoraNetworkConfig(
+            amphora=amp,
+            vrrp_subnet=self.get_subnet(
+                vrrp_port.get_subnet_id(amp.vrrp_ip)),
+            vrrp_port=vrrp_port,
+            ha_subnet=self.get_subnet(
+                ha_port.get_subnet_id(amp.ha_ip)),
+            ha_port=ha_port)
+
+    def get_network_configs(self, loadbalancer, amphora=None):
+        amphora_id = amphora.id if amphora else None
+        LOG.debug("Network %s no-op, get_network_configs loadbalancer id "
+                  "%s amphora id: %s", self.__class__.__name__,
+                  loadbalancer.id, amphora_id)
         self.networkconfigconfig[(loadbalancer.id)] = (
             loadbalancer, 'get_network_configs')
+        vip_subnet = self.get_subnet(loadbalancer.vip.subnet_id)
+        vip_port = self.get_port(loadbalancer.vip.port_id)
 
         amp_configs = {}
-        for amp in loadbalancer.amphorae:
-            vrrp_port = self.get_port(amp.vrrp_port_id)
-            ha_port = self.get_port(amp.ha_port_id)
-            amp_configs[amp.id] = network_models.AmphoraNetworkConfig(
-                amphora=amp,
-                vip_subnet=self.get_subnet(loadbalancer.vip.subnet_id),
-                vip_port=self.get_port(loadbalancer.vip.port_id),
-                vrrp_subnet=self.get_subnet(
-                    vrrp_port.get_subnet_id(amp.vrrp_ip)),
-                vrrp_port=vrrp_port,
-                ha_subnet=self.get_subnet(
-                    ha_port.get_subnet_id(amp.ha_ip)),
-                ha_port=ha_port)
+        if amphora:
+            self._get_amp_net_configs(amphora, amp_configs,
+                                      vip_subnet, vip_port)
+        else:
+            for amp in loadbalancer.amphorae:
+                self._get_amp_net_configs(amp, amp_configs,
+                                          vip_subnet, vip_port)
+
         return amp_configs
 
     def wait_for_port_detach(self, amphora):
@@ -220,6 +261,9 @@ class NoopManager(object):
         self.networkconfigconfig[(qos_id, port_id)] = (
             qos_id, port_id, 'apply_qos_on_port')
 
+    def qos_enabled(self):
+        return self._qos_extension_enabled
+
 
 class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
     def __init__(self):
@@ -278,14 +322,26 @@ class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
     def failover_preparation(self, amphora):
         self.driver.failover_preparation(amphora)
 
-    def plug_port(self, compute_id, port):
-        return self.driver.plug_port(compute_id, port)
+    def plug_port(self, amphora, port):
+        return self.driver.plug_port(amphora, port)
 
-    def get_network_configs(self, loadbalancer):
-        return self.driver.get_network_configs(loadbalancer)
+    def get_network_configs(self, loadbalancer, amphora=None):
+        return self.driver.get_network_configs(loadbalancer, amphora)
 
     def wait_for_port_detach(self, amphora):
         self.driver.wait_for_port_detach(amphora)
 
     def apply_qos_on_port(self, qos_id, port_id):
         self.driver.apply_qos_on_port(qos_id, port_id)
+
+    def update_vip_sg(self, load_balancer, vip):
+        self.driver.update_vip_sg(load_balancer, vip)
+
+    def plug_aap_port(self, load_balancer, vip, amphora, subnet):
+        return self.driver.plug_aap_port(load_balancer, vip, amphora, subnet)
+
+    def unplug_aap_port(self, vip, amphora, subnet):
+        self.driver.unplug_aap_port(vip, amphora, subnet)
+
+    def qos_enabled(self):
+        return self.driver.qos_enabled()
