@@ -24,11 +24,10 @@ class HAProxyCompatTestCase(base.TestCase):
     def setUp(self):
         super(HAProxyCompatTestCase, self).setUp()
         self.old_haproxy_global = (
-            "# Configuration for test-lb\n"
+            "# Configuration for loadbalancer sample_loadbalancer_id_1\n"
             "global\n"
             "    daemon\n"
             "    user nobody\n"
-            "    group nogroup\n"
             "    log /dev/log local0\n"
             "    log /dev/log local1 notice\n"
             "    stats socket /var/lib/octavia/sample_listener_id_1.sock"
@@ -38,15 +37,16 @@ class HAProxyCompatTestCase(base.TestCase):
             "    log global\n"
             "    retries 3\n"
             "    option redispatch\n"
-            "    timeout connect 5000\n"
-            "    timeout client 50000\n"
-            "    timeout server 50000\n\n\n\n"
+            "    option splice-request\n"
+            "    option splice-response\n"
+            "    option http-keep-alive\n\n\n\n"
             "frontend sample_listener_id_1\n"
             "    option httplog\n"
             "    maxconn {maxconn}\n"
             "    bind 10.0.0.2:80\n"
             "    mode http\n"
-            "    default_backend sample_pool_id_1\n\n").format(
+            "    default_backend sample_pool_id_1\n"
+            "    timeout client 50000\n\n").format(
             maxconn=constants.HAPROXY_MAX_MAXCONN)
         self.backend_without_external = (
             "backend sample_pool_id_1\n"
@@ -57,11 +57,15 @@ class HAProxyCompatTestCase(base.TestCase):
             "    option httpchk GET /index.html\n"
             "    http-check expect rstatus 418\n"
             "    fullconn {maxconn}\n"
+            "    option allbackups\n"
+            "    timeout connect 5000\n"
+            "    timeout server 50000\n"
             "    server sample_member_id_1 10.0.0.99:82 weight 13 "
             "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
             "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-            "check inter 30s fall 3 rise 2 cookie sample_member_id_2"
-            "\n").format(maxconn=constants.HAPROXY_MAX_MAXCONN)
+            "check inter 30s fall 3 rise 2 cookie "
+            "sample_member_id_2\n").format(
+            maxconn=constants.HAPROXY_MAX_MAXCONN)
         self.backend_with_external = (
             "backend sample_pool_id_1\n"
             "    mode http\n"
@@ -70,14 +74,18 @@ class HAProxyCompatTestCase(base.TestCase):
             "    timeout check 31\n"
             "    option httpchk GET /index.html\n"
             "    http-check expect rstatus 418\n"
-            "    fullconn {maxconn}\n"
             "    option external-check\n"
             "    external-check command /var/lib/octavia/ping-wrapper.sh\n"
+            "    fullconn {maxconn}\n"
+            "    option allbackups\n"
+            "    timeout connect 5000\n"
+            "    timeout server 50000\n"
             "    server sample_member_id_1 10.0.0.99:82 weight 13 "
             "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
             "    server sample_member_id_2 10.0.0.98:82 weight 13 "
-            "check inter 30s fall 3 rise 2 cookie sample_member_id_2"
-            "\n").format(maxconn=constants.HAPROXY_MAX_MAXCONN)
+            "check inter 30s fall 3 rise 2 cookie "
+            "sample_member_id_2\n").format(
+            maxconn=constants.HAPROXY_MAX_MAXCONN)
 
     @mock.patch('subprocess.check_output')
     def test_get_haproxy_versions(self, mock_process):
