@@ -17,9 +17,12 @@ import subprocess
 
 import mock
 import netifaces
+from oslo_config import cfg
+from oslo_config import fixture as oslo_fixture
 
 from octavia.amphorae.backends.agent.api_server import osutils
 from octavia.amphorae.backends.agent.api_server import plug
+from octavia.common import constants
 import octavia.tests.unit.base as base
 
 FAKE_CIDR_IPV4 = '10.0.0.0/24'
@@ -37,8 +40,8 @@ class TestPlug(base.TestCase):
     def setUp(self):
         super(TestPlug, self).setUp()
         self.mock_netifaces = mock.patch.object(plug, "netifaces").start()
-        self.mock_platform = mock.patch("platform.linux_distribution").start()
-        self.mock_platform.return_value = ("Ubuntu",)
+        self.mock_platform = mock.patch("distro.id").start()
+        self.mock_platform.return_value = "ubuntu"
         self.osutil = osutils.BaseOS.get_os_util()
         self.test_plug = plug.Plug(self.osutil)
         self.addCleanup(self.mock_netifaces.stop)
@@ -57,8 +60,7 @@ class TestPlug(base.TestCase):
         self.assertEqual(FAKE_INTERFACE, interface)
 
     def test__interface_by_mac_case_insensitive_rh(self):
-        with mock.patch('platform.linux_distribution',
-                        return_value=['centos', 'Foo']):
+        with mock.patch('distro.id', return_value='centos'):
             osutil = osutils.BaseOS.get_os_util()
             self.test_plug = plug.Plug(osutil)
             interface = self.test_plug._interface_by_mac(
@@ -89,9 +91,17 @@ class TestPlug(base.TestCase):
             'details': 'VIP {vip} plugged on interface {interface}'.format(
                 vip=FAKE_IP_IPV4, interface='eth1')
         }, status=202)
-        mock_nspopen.assert_called_once_with(
-            'amphora-haproxy', ['/sbin/sysctl', '--system'],
-            stdout=subprocess.PIPE)
+        calls = [mock.call('amphora-haproxy', ['/sbin/sysctl', '--system'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy', ['modprobe', 'ip_vs'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy',
+                           ['/sbin/sysctl', '-w', 'net.ipv4.ip_forward=1'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy',
+                           ['/sbin/sysctl', '-w', 'net.ipv4.vs.conntrack=1'],
+                           stdout=subprocess.PIPE)]
+        mock_nspopen.assert_has_calls(calls, any_order=True)
 
     @mock.patch('pyroute2.NSPopen')
     @mock.patch.object(plug, "webob")
@@ -104,6 +114,9 @@ class TestPlug(base.TestCase):
     def test_plug_vip_ipv6(self, mock_makedirs, mock_copytree,
                            mock_check_output, mock_netns, mock_netns_create,
                            mock_pyroute2, mock_webob, mock_nspopen):
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        conf.config(group='controller_worker',
+                    loadbalancer_topology=constants.TOPOLOGY_ACTIVE_STANDBY)
         m = mock.mock_open()
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             self.test_plug.plug_vip(
@@ -117,9 +130,18 @@ class TestPlug(base.TestCase):
             'details': 'VIP {vip} plugged on interface {interface}'.format(
                 vip=FAKE_IP_IPV6_EXPANDED, interface='eth1')
         }, status=202)
-        mock_nspopen.assert_called_once_with(
-            'amphora-haproxy', ['/sbin/sysctl', '--system'],
-            stdout=subprocess.PIPE)
+        calls = [mock.call('amphora-haproxy', ['/sbin/sysctl', '--system'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy', ['modprobe', 'ip_vs'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy',
+                           ['/sbin/sysctl', '-w',
+                            'net.ipv6.conf.all.forwarding=1'],
+                           stdout=subprocess.PIPE),
+                 mock.call('amphora-haproxy',
+                           ['/sbin/sysctl', '-w', 'net.ipv4.vs.conntrack=1'],
+                           stdout=subprocess.PIPE)]
+        mock_nspopen.assert_has_calls(calls, any_order=True)
 
     @mock.patch.object(plug, "webob")
     @mock.patch('pyroute2.IPRoute')
@@ -160,7 +182,7 @@ class TestPlug(base.TestCase):
 class TestPlugNetwork(base.TestCase):
     def setUp(self):
         super(TestPlugNetwork, self).setUp()
-        self.mock_platform = mock.patch("platform.linux_distribution").start()
+        self.mock_platform = mock.patch("distro.id").start()
 
     def __generate_network_file_text_static_ip(self):
         netns_interface = 'eth1234'
@@ -189,7 +211,11 @@ class TestPlugNetwork(base.TestCase):
             'up route add -net {dest1} gw {nexthop} dev {netns_interface}\n'
             'down route del -net {dest1} gw {nexthop} dev {netns_interface}\n'
             'up route add -net {dest2} gw {nexthop} dev {netns_interface}\n'
-            'down route del -net {dest2} gw {nexthop} dev {netns_interface}\n')
+            'down route del -net {dest2} gw {nexthop} dev {netns_interface}\n'
+            'post-up /sbin/iptables -t nat -A POSTROUTING -p udp -o '
+            'eth1234 -j MASQUERADE\n'
+            'post-down /sbin/iptables -t nat -D POSTROUTING -p udp -o eth1234 '
+            '-j MASQUERADE\n')
 
         template_port = osutils.j2_env.get_template('plug_port_ethX.conf.j2')
         text = self.test_plug._osutils._generate_network_file_text(
@@ -240,13 +266,21 @@ class TestPlugNetwork(base.TestCase):
             'down route del -net {dest1} gw {nexthop} dev {netns_interface}\n'
             'up route add -net {dest2} gw {nexthop} dev {netns_interface}\n'
             'down route del -net {dest2} gw {nexthop} dev {netns_interface}\n'
+            'post-up /sbin/iptables -t nat -A POSTROUTING -p udp -o '
+            '{netns_interface} -j MASQUERADE\n'
+            'post-down /sbin/iptables -t nat -D POSTROUTING -p udp -o '
+            '{netns_interface} -j MASQUERADE\n'
             '\n\n# Generated by Octavia agent\n'
             'auto {netns_interface}\n'
             'iface {netns_interface} inet6 static\n'
             'address {fixed_ip_ipv6}\n'
             'broadcast {broadcast_ipv6}\n'
             'netmask {netmask_ipv6}\n'
-            'mtu {mtu}\n')
+            'mtu {mtu}\n'
+            'post-up /sbin/ip6tables -t nat -A POSTROUTING -p udp -o '
+            '{netns_interface} -j MASQUERADE\n'
+            'post-down /sbin/ip6tables -t nat -D POSTROUTING -p udp -o '
+            '{netns_interface} -j MASQUERADE\n')
 
         template_port = osutils.j2_env.get_template('plug_port_ethX.conf.j2')
         text = self.test_plug._osutils._generate_network_file_text(
@@ -265,12 +299,12 @@ class TestPlugNetwork(base.TestCase):
         self.assertEqual(expected_text, text)
 
     def _setup(self, os):
-        self.mock_platform.return_value = [os]
+        self.mock_platform.return_value = os
         self.osutil = osutils.BaseOS.get_os_util()
         self.test_plug = plug.Plug(self.osutil)
 
     def test__generate_network_file_text_static_ip_ubuntu(self):
-        self._setup("Ubuntu")
+        self._setup("ubuntu")
         self.__generate_network_file_text_static_ip()
 
     def test__generate_network_file_text_static_ip_centos(self):
@@ -278,7 +312,7 @@ class TestPlugNetwork(base.TestCase):
         self.__generate_network_file_text_static_ip()
 
     def test__generate_network_file_text_two_static_ips_ubuntu(self):
-        self._setup("Ubuntu")
+        self._setup("ubuntu")
         self.__generate_network_file_text_two_static_ips()
 
     def test__generate_network_file_text_two_static_ips_centos(self):
