@@ -15,6 +15,7 @@
 from oslo_utils import uuidutils
 
 from octavia.amphorae.drivers.noop_driver import driver
+from octavia.common import constants
 from octavia.common import data_models
 from octavia.network import data_models as network_models
 from octavia.tests.unit import base
@@ -44,6 +45,7 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
         super(TestNoopAmphoraLoadBalancerDriver, self).setUp()
         self.driver = driver.NoopAmphoraLoadBalancerDriver()
         self.listener = data_models.Listener()
+        self.listener.id = uuidutils.generate_uuid()
         self.listener.protocol_port = 80
         self.vip = data_models.Vip()
         self.vip.ip_address = "10.0.0.1"
@@ -61,6 +63,21 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
                     vip_subnet=network_models.Subnet(id=self.FAKE_UUID_1))
         }
         self.pem_file = 'test_pem_file'
+        self.agent_config = 'test agent config'
+        self.timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
+                             constants.REQ_READ_TIMEOUT: 2,
+                             constants.CONN_MAX_RETRIES: 3,
+                             constants.CONN_RETRY_INTERVAL: 4}
+
+    def test_update_amphora_listeners(self):
+        amphorae = [self.amphora]
+        self.driver.update_amphora_listeners([self.listener], 0, amphorae,
+                                             self.timeout_dict)
+        self.assertEqual((self.listener, self.amphora.id, self.timeout_dict,
+                          'update_amp'),
+                         self.driver.driver.amphoraconfig[(
+                             self.listener.id,
+                             self.amphora.id)])
 
     def test_update(self):
         self.driver.update(self.listener, self.vip)
@@ -77,11 +94,11 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
                              self.vip.ip_address)])
 
     def test_start(self):
-        self.driver.start(self.listener, self.vip)
-        self.assertEqual((self.listener, self.vip, 'start'),
+        self.driver.start(self.listener, self.vip, amphora='amp1')
+        self.assertEqual((self.listener, self.vip, 'amp1', 'start'),
                          self.driver.driver.amphoraconfig[(
                              self.listener.protocol_port,
-                             self.vip.ip_address)])
+                             self.vip.ip_address, 'amp1')])
 
     def test_delete(self):
         self.driver.delete(self.listener, self.vip)
@@ -131,3 +148,10 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
             (self.amphora.id, self.pem_file, 'update_amp_cert_file'),
             self.driver.driver.amphoraconfig[(
                 self.amphora.id, self.pem_file)])
+
+    def test_update_agent_config(self):
+        self.driver.update_agent_config(self.amphora, self.agent_config)
+        self.assertEqual(
+            (self.amphora.id, self.agent_config, 'update_agent_config'),
+            self.driver.driver.amphoraconfig[(
+                self.amphora.id, self.agent_config)])
