@@ -15,9 +15,10 @@ import mock
 
 from oslo_utils import uuidutils
 
+from octavia_lib.api.drivers import data_models as driver_dm
+from octavia_lib.api.drivers import exceptions
+
 from octavia.api.drivers.amphora_driver import driver
-from octavia.api.drivers import data_models as driver_dm
-from octavia.api.drivers import exceptions
 from octavia.common import constants as consts
 from octavia.network import base as network_base
 from octavia.tests.unit.api.drivers import sample_data_models
@@ -60,7 +61,8 @@ class TestAmphoraDriver(base.TestRpc):
         provider_lb = driver_dm.LoadBalancer(
             loadbalancer_id=self.sample_data.lb_id)
         self.amp_driver.loadbalancer_create(provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id}
+        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
+                   consts.FLAVOR: None}
         mock_cast.assert_called_with({}, 'create_load_balancer', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -193,68 +195,25 @@ class TestAmphoraDriver(base.TestRpc):
         old_provider_pool = driver_dm.Pool(
             pool_id=self.sample_data.pool1_id)
         provider_pool = driver_dm.Pool(
-            pool_id=self.sample_data.pool1_id, name='Great pool')
-        pool_dict = {'name': 'Great pool'}
+            pool_id=self.sample_data.pool1_id, name='Great pool',
+            admin_state_up=True, tls_enabled=True)
+        pool_dict = {'name': 'Great pool',
+                     'enabled': True,
+                     'tls_enabled': True}
         self.amp_driver.pool_update(old_provider_pool, provider_pool)
         payload = {consts.POOL_ID: self.sample_data.pool1_id,
                    consts.POOL_UPDATES: pool_dict}
         mock_cast.assert_called_with({}, 'update_pool', **payload)
 
     # Member
-    @mock.patch('octavia.db.api.get_session')
-    @mock.patch('octavia.db.repositories.PoolRepository.get')
     @mock.patch('oslo_messaging.RPCClient.cast')
-    def test_member_create(self, mock_cast, mock_pool_get, mock_session):
+    def test_member_create(self, mock_cast):
         provider_member = driver_dm.Member(
             member_id=self.sample_data.member1_id)
         self.amp_driver.member_create(provider_member)
         payload = {consts.MEMBER_ID: self.sample_data.member1_id}
         mock_cast.assert_called_with({}, 'create_member', **payload)
 
-    @mock.patch('octavia.db.api.get_session')
-    @mock.patch('octavia.db.repositories.PoolRepository.get')
-    @mock.patch('oslo_messaging.RPCClient.cast')
-    def test_member_create_udp_ipv4(self, mock_cast, mock_pool_get,
-                                    mock_session):
-        mock_lb = mock.MagicMock()
-        mock_lb.vip = mock.MagicMock()
-        mock_lb.vip.ip_address = "192.0.1.1"
-        mock_listener = mock.MagicMock()
-        mock_listener.load_balancer = mock_lb
-        mock_pool = mock.MagicMock()
-        mock_pool.protocol = consts.PROTOCOL_UDP
-        mock_pool.listeners = [mock_listener]
-        mock_pool_get.return_value = mock_pool
-
-        provider_member = driver_dm.Member(
-            member_id=self.sample_data.member1_id,
-            address="192.0.2.1")
-        self.amp_driver.member_create(provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id}
-        mock_cast.assert_called_with({}, 'create_member', **payload)
-
-    @mock.patch('octavia.db.api.get_session')
-    @mock.patch('octavia.db.repositories.PoolRepository.get')
-    @mock.patch('oslo_messaging.RPCClient.cast')
-    def test_member_create_udp_ipv4_ipv6(self, mock_cast, mock_pool_get,
-                                         mock_session):
-        mock_lb = mock.MagicMock()
-        mock_lb.vip = mock.MagicMock()
-        mock_lb.vip.ip_address = "fe80::1"
-        mock_listener = mock.MagicMock()
-        mock_listener.load_balancer = mock_lb
-        mock_pool = mock.MagicMock()
-        mock_pool.protocol = consts.PROTOCOL_UDP
-        mock_pool.listeners = [mock_listener]
-        mock_pool_get.return_value = mock_pool
-
-        provider_member = driver_dm.Member(
-            member_id=self.sample_data.member1_id,
-            address="192.0.2.1")
-        self.assertRaises(exceptions.UnsupportedOptionError,
-                          self.amp_driver.member_create,
-                          provider_member)
-
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_member_delete(self, mock_cast):
         provider_member = driver_dm.Member(
@@ -373,83 +332,6 @@ class TestAmphoraDriver(base.TestRpc):
         payload = {consts.HEALTH_MONITOR_ID: self.sample_data.hm1_id}
         mock_cast.assert_called_with({}, 'delete_health_monitor', **payload)
 
-    @mock.patch('octavia.db.api.get_session')
-    @mock.patch('octavia.db.repositories.PoolRepository.get')
-    @mock.patch('oslo_messaging.RPCClient.cast')
-    def test_member_batch_update_udp_ipv4(self, mock_cast, mock_pool_get,
-                                          mock_session):
-
-        mock_lb = mock.MagicMock()
-        mock_lb.vip = mock.MagicMock()
-        mock_lb.vip.ip_address = "192.0.1.1"
-        mock_listener = mock.MagicMock()
-        mock_listener.load_balancer = mock_lb
-        mock_pool = mock.MagicMock()
-        mock_pool.protocol = consts.PROTOCOL_UDP
-        mock_pool.listeners = [mock_listener]
-        mock_pool.members = self.sample_data.db_pool1_members
-        mock_pool_get.return_value = mock_pool
-
-        prov_mem_update = driver_dm.Member(
-            member_id=self.sample_data.member2_id,
-            pool_id=self.sample_data.pool1_id, admin_state_up=False,
-            address='192.0.2.17', monitor_address='192.0.2.77',
-            protocol_port=80, name='updated-member2')
-        prov_new_member = driver_dm.Member(
-            member_id=self.sample_data.member3_id,
-            pool_id=self.sample_data.pool1_id,
-            address='192.0.2.18', monitor_address='192.0.2.28',
-            protocol_port=80, name='member3')
-        prov_members = [prov_mem_update, prov_new_member]
-
-        update_mem_dict = {'ip_address': '192.0.2.17',
-                           'name': 'updated-member2',
-                           'monitor_address': '192.0.2.77',
-                           'id': self.sample_data.member2_id,
-                           'enabled': False,
-                           'protocol_port': 80,
-                           'pool_id': self.sample_data.pool1_id}
-
-        self.amp_driver.member_batch_update(prov_members)
-
-        payload = {'old_member_ids': [self.sample_data.member1_id],
-                   'new_member_ids': [self.sample_data.member3_id],
-                   'updated_members': [update_mem_dict]}
-        mock_cast.assert_called_with({}, 'batch_update_members', **payload)
-
-    @mock.patch('octavia.db.api.get_session')
-    @mock.patch('octavia.db.repositories.PoolRepository.get')
-    @mock.patch('oslo_messaging.RPCClient.cast')
-    def test_member_batch_update_udp_ipv4_ipv6(self, mock_cast, mock_pool_get,
-                                               mock_session):
-
-        mock_lb = mock.MagicMock()
-        mock_lb.vip = mock.MagicMock()
-        mock_lb.vip.ip_address = "192.0.1.1"
-        mock_listener = mock.MagicMock()
-        mock_listener.load_balancer = mock_lb
-        mock_pool = mock.MagicMock()
-        mock_pool.protocol = consts.PROTOCOL_UDP
-        mock_pool.listeners = [mock_listener]
-        mock_pool.members = self.sample_data.db_pool1_members
-        mock_pool_get.return_value = mock_pool
-
-        prov_mem_update = driver_dm.Member(
-            member_id=self.sample_data.member2_id,
-            pool_id=self.sample_data.pool1_id, admin_state_up=False,
-            address='fe80::1', monitor_address='fe80::2',
-            protocol_port=80, name='updated-member2')
-        prov_new_member = driver_dm.Member(
-            member_id=self.sample_data.member3_id,
-            pool_id=self.sample_data.pool1_id,
-            address='192.0.2.18', monitor_address='192.0.2.28',
-            protocol_port=80, name='member3')
-        prov_members = [prov_mem_update, prov_new_member]
-
-        self.assertRaises(exceptions.UnsupportedOptionError,
-                          self.amp_driver.member_batch_update,
-                          prov_members)
-
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_health_monitor_update(self, mock_cast):
         old_provider_hm = driver_dm.HealthMonitor(
@@ -558,3 +440,47 @@ class TestAmphoraDriver(base.TestRpc):
         payload = {consts.L7RULE_ID: self.sample_data.l7rule1_id,
                    consts.L7RULE_UPDATES: l7rule_dict}
         mock_cast.assert_called_with({}, 'update_l7rule', **payload)
+
+    # Flavor
+    def test_get_supported_flavor_metadata(self):
+        test_schema = {
+            "properties": {
+                "test_name": {"description": "Test description"},
+                "test_name2": {"description": "Another description"}}}
+        ref_dict = {"test_name": "Test description",
+                    "test_name2": "Another description"}
+
+        # mock out the supported_flavor_metadata
+        with mock.patch('octavia.api.drivers.amphora_driver.flavor_schema.'
+                        'SUPPORTED_FLAVOR_SCHEMA', test_schema):
+            result = self.amp_driver.get_supported_flavor_metadata()
+        self.assertEqual(ref_dict, result)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.flavor_schema.'
+                        'SUPPORTED_FLAVOR_SCHEMA', 'bogus'):
+            self.assertRaises(exceptions.DriverError,
+                              self.amp_driver.get_supported_flavor_metadata)
+
+    @mock.patch('jsonschema.validators.requests')
+    def test_validate_flavor(self, mock_validate):
+        ref_dict = {consts.LOADBALANCER_TOPOLOGY: consts.TOPOLOGY_SINGLE}
+        self.amp_driver.validate_flavor(ref_dict)
+
+        # Test bad flavor metadata value is bad
+        ref_dict = {consts.LOADBALANCER_TOPOLOGY: 'bogus'}
+        self.assertRaises(exceptions.UnsupportedOptionError,
+                          self.amp_driver.validate_flavor,
+                          ref_dict)
+
+        # Test bad flavor metadata key
+        ref_dict = {'bogus': 'bogus'}
+        self.assertRaises(exceptions.UnsupportedOptionError,
+                          self.amp_driver.validate_flavor,
+                          ref_dict)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.flavor_schema.'
+                        'SUPPORTED_FLAVOR_SCHEMA', 'bogus'):
+            self.assertRaises(exceptions.DriverError,
+                              self.amp_driver.validate_flavor, 'bogus')
