@@ -16,7 +16,6 @@ import uuid
 
 from barbicanclient.v1 import secrets
 import mock
-from OpenSSL import crypto
 
 import octavia.certificates.common.barbican as barbican_common
 import octavia.certificates.common.cert as cert
@@ -41,11 +40,14 @@ class TestBarbicanManager(base.TestCase):
         )
 
         self.name = 'My Fancy Cert'
-        self.secret = secrets.Secret(
+        self.secret_pkcs12 = secrets.Secret(
             api=mock.MagicMock(),
             payload=sample.PKCS12_BUNDLE
         )
 
+        self.fake_secret = 'Fake secret'
+        self.secret = secrets.Secret(api=mock.MagicMock(),
+                                     payload=self.fake_secret)
         self.empty_secret = mock.Mock(spec=secrets.Secret)
 
         # Mock out the client
@@ -113,7 +115,7 @@ class TestBarbicanManager(base.TestCase):
 
     def test_get_cert(self):
         # Mock out the client
-        self.bc.secrets.get.return_value = self.secret
+        self.bc.secrets.get.return_value = self.secret_pkcs12
 
         # Get the secret data
         data = self.cert_manager.get_cert(
@@ -135,22 +137,6 @@ class TestBarbicanManager(base.TestCase):
         self.assertItemsEqual(sample.X509_IMDS_LIST, data.get_intermediates())
         self.assertIsNone(data.get_private_key_passphrase())
 
-    @mock.patch('OpenSSL.crypto.load_pkcs12')
-    def test_get_cert_bad_pkcs12(self, mock_load_pkcs12):
-
-        mock_load_pkcs12.side_effect = [crypto.Error]
-
-        # Mock out the client
-        self.bc.secrets.get.return_value = self.secret
-
-        # Test bad pkcs12 bundle re-raises UnreadablePKCS12
-        self.assertRaises(exceptions.UnreadablePKCS12,
-                          self.cert_manager.get_cert,
-                          context=self.context,
-                          cert_ref=self.secret_ref,
-                          resource_ref=self.secret_ref,
-                          service_name='Octavia')
-
     def test_delete_cert_legacy(self):
         # Attempt to deregister as a consumer
         self.cert_manager.delete_cert(
@@ -192,3 +178,25 @@ class TestBarbicanManager(base.TestCase):
         self.cert_manager.auth.revoke_secret_access.assert_called_once_with(
             self.context, self.secret_ref
         )
+
+    def test_get_secret(self):
+        # Mock out the client
+        self.bc.secrets.get.side_effect = [self.secret, Exception]
+
+        # Get the secret data
+        data = self.cert_manager.get_secret(
+            context=self.context,
+            secret_ref=self.secret_ref,
+        )
+
+        # 'get_secret' should be called once with the secret_ref
+        self.bc.secrets.get.assert_called_once_with(
+            secret_ref=self.secret_ref
+        )
+
+        self.assertEqual(self.fake_secret, data)
+
+        # Test with a failure
+        self.assertRaises(exceptions.CertificateRetrievalException,
+                          self.cert_manager.get_secret,
+                          context=self.context, secret_ref=self.secret_ref)
