@@ -13,6 +13,7 @@
 # under the License.
 #
 
+from cryptography import fernet
 import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
@@ -20,13 +21,14 @@ from oslo_utils import uuidutils
 
 from octavia.common import constants
 from octavia.common import exceptions
+from octavia.common import utils
 from octavia.controller.worker.tasks import compute_tasks
 from octavia.tests.common import utils as test_utils
 import octavia.tests.unit.base as base
 
 
-AMP_FLAVOR_ID = 10
-AMP_IMAGE_ID = 11
+AMP_FLAVOR_ID = '10'
+AMP_IMAGE_ID = '11'
 AMP_IMAGE_TAG = 'glance_tag'
 AMP_SSH_KEY_NAME = None
 AMP_NET = [uuidutils.generate_uuid()]
@@ -36,7 +38,6 @@ AMPHORA_ID = uuidutils.generate_uuid()
 COMPUTE_ID = uuidutils.generate_uuid()
 LB_NET_IP = '192.0.2.1'
 PORT_ID = uuidutils.generate_uuid()
-AUTH_VERSION = '2'
 SERVER_GRPOUP_ID = uuidutils.generate_uuid()
 
 
@@ -60,18 +61,22 @@ _port.id = PORT_ID
 class TestComputeTasks(base.TestCase):
 
     def setUp(self):
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker", amp_flavor_id=AMP_FLAVOR_ID)
-        conf.config(group="controller_worker", amp_image_id=AMP_IMAGE_ID)
-        conf.config(group="controller_worker", amp_image_tag=AMP_IMAGE_TAG)
-        conf.config(group="controller_worker",
-                    amp_ssh_key_name=AMP_SSH_KEY_NAME)
-        conf.config(group="controller_worker", amp_boot_network_list=AMP_NET)
-        conf.config(group="controller_worker", amp_active_wait_sec=AMP_WAIT)
-        conf.config(group="controller_worker",
-                    amp_secgroup_list=AMP_SEC_GROUPS)
-        conf.config(group="keystone_authtoken", auth_version=AUTH_VERSION)
-        conf.config(group="controller_worker", amp_image_owner_id='')
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        self.conf.config(
+            group="controller_worker", amp_flavor_id=AMP_FLAVOR_ID)
+        self.conf.config(
+            group="controller_worker", amp_image_id=AMP_IMAGE_ID)
+        self.conf.config(
+            group="controller_worker", amp_image_tag=AMP_IMAGE_TAG)
+        self.conf.config(
+            group="controller_worker", amp_ssh_key_name=AMP_SSH_KEY_NAME)
+        self.conf.config(
+            group="controller_worker", amp_boot_network_list=AMP_NET)
+        self.conf.config(
+            group="controller_worker", amp_active_wait_sec=AMP_WAIT)
+        self.conf.config(
+            group="controller_worker", amp_secgroup_list=AMP_SEC_GROUPS)
+        self.conf.config(group="controller_worker", amp_image_owner_id='')
 
         _amphora_mock.id = AMPHORA_ID
         _amphora_mock.status = constants.AMPHORA_ALLOCATED
@@ -89,9 +94,8 @@ class TestComputeTasks(base.TestCase):
     def test_compute_create(self, mock_driver, mock_conf, mock_jinja):
 
         image_owner_id = uuidutils.generate_uuid()
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker",
-                    amp_image_owner_id=image_owner_id)
+        self.conf.config(
+            group="controller_worker", amp_image_owner_id=image_owner_id)
 
         createcompute = compute_tasks.ComputeCreate()
 
@@ -116,130 +120,6 @@ class TestComputeTasks(base.TestCase):
             user_data=None,
             server_group_id=SERVER_GRPOUP_ID)
 
-        # Make sure it returns the expected compute_id
-        assert(compute_id == COMPUTE_ID)
-
-        # Test that a build exception is raised
-        createcompute = compute_tasks.ComputeCreate()
-
-        self.assertRaises(TypeError,
-                          createcompute.execute,
-                          _amphora_mock, config_drive_files='test_cert')
-
-        # Test revert()
-
-        _amphora_mock.compute_id = COMPUTE_ID
-
-        createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
-
-        # Validate that the delete method was called properly
-        mock_driver.delete.assert_called_once_with(
-            COMPUTE_ID)
-
-        # Test that a delete exception is not raised
-
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
-
-        conf.config(group="controller_worker",
-                    amp_image_owner_id='')
-
-    @mock.patch('jinja2.Environment.get_template')
-    @mock.patch('octavia.amphorae.backends.agent.'
-                'agent_jinja_cfg.AgentJinjaTemplater.'
-                'build_agent_config', return_value='test_conf')
-    @mock.patch('octavia.common.jinja.'
-                'user_data_jinja_cfg.UserDataJinjaCfg.'
-                'build_user_data_config', return_value='test_conf')
-    @mock.patch('stevedore.driver.DriverManager.driver')
-    def test_compute_create_user_data(self, mock_driver,
-                                      mock_ud_conf, mock_conf, mock_jinja):
-
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker", user_data_config_drive=True)
-        mock_ud_conf.return_value = 'test_ud_conf'
-        createcompute = compute_tasks.ComputeCreate()
-
-        mock_driver.build.return_value = COMPUTE_ID
-        # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port])
-
-        # Validate that the build method was called properly
-        mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
-            amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
-            image_tag=AMP_IMAGE_TAG,
-            image_owner='',
-            key_name=AMP_SSH_KEY_NAME,
-            sec_groups=AMP_SEC_GROUPS,
-            network_ids=AMP_NET,
-            port_ids=[PORT_ID],
-            config_drive_files=None,
-            user_data='test_ud_conf',
-            server_group_id=None)
-
-        # Make sure it returns the expected compute_id
-        assert(compute_id == COMPUTE_ID)
-
-        # Test that a build exception is raised
-        createcompute = compute_tasks.ComputeCreate()
-
-        self.assertRaises(TypeError,
-                          createcompute.execute,
-                          _amphora_mock, config_drive_files='test_cert')
-
-        # Test revert()
-
-        _amphora_mock.compute_id = COMPUTE_ID
-
-        createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
-
-        # Validate that the delete method was called properly
-        mock_driver.delete.assert_called_once_with(
-            COMPUTE_ID)
-
-        # Test that a delete exception is not raised
-
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker", user_data_config_drive=False)
-
-    @mock.patch('jinja2.Environment.get_template')
-    @mock.patch('octavia.amphorae.backends.agent.'
-                'agent_jinja_cfg.AgentJinjaTemplater.'
-                'build_agent_config', return_value='test_conf')
-    @mock.patch('stevedore.driver.DriverManager.driver')
-    def test_compute_create_without_ssh_access(self, mock_driver,
-                                               mock_conf, mock_jinja):
-
-        createcompute = compute_tasks.ComputeCreate()
-
-        mock_driver.build.return_value = COMPUTE_ID
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker", amp_ssh_access_allowed=False)
-
-        # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
-                                           server_group_id=SERVER_GRPOUP_ID)
-
-        # Validate that the build method was called properly
-        mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
-            amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
-            image_tag=AMP_IMAGE_TAG,
-            image_owner='',
-            key_name=None,
-            sec_groups=AMP_SEC_GROUPS,
-            network_ids=AMP_NET,
-            port_ids=[PORT_ID],
-            config_drive_files={'/etc/octavia/'
-                                'amphora-agent.conf': 'test_conf'},
-            user_data=None,
-            server_group_id=SERVER_GRPOUP_ID)
-
         # Make sure it returns the expected compute_id
         self.assertEqual(COMPUTE_ID, compute_id)
 
@@ -265,6 +145,126 @@ class TestComputeTasks(base.TestCase):
 
         createcompute.revert(COMPUTE_ID, _amphora_mock.id)
 
+    @mock.patch('jinja2.Environment.get_template')
+    @mock.patch('octavia.amphorae.backends.agent.'
+                'agent_jinja_cfg.AgentJinjaTemplater.'
+                'build_agent_config', return_value='test_conf')
+    @mock.patch('octavia.common.jinja.'
+                'user_data_jinja_cfg.UserDataJinjaCfg.'
+                'build_user_data_config', return_value='test_conf')
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_compute_create_user_data(self, mock_driver,
+                                      mock_ud_conf, mock_conf, mock_jinja):
+
+        self.conf.config(
+            group="controller_worker", user_data_config_drive=True)
+        mock_ud_conf.return_value = 'test_ud_conf'
+        createcompute = compute_tasks.ComputeCreate()
+
+        mock_driver.build.return_value = COMPUTE_ID
+        # Test execute()
+        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port])
+
+        # Validate that the build method was called properly
+        mock_driver.build.assert_called_once_with(
+            name="amphora-" + _amphora_mock.id,
+            amphora_flavor=AMP_FLAVOR_ID,
+            image_id=AMP_IMAGE_ID,
+            image_tag=AMP_IMAGE_TAG,
+            image_owner='',
+            key_name=AMP_SSH_KEY_NAME,
+            sec_groups=AMP_SEC_GROUPS,
+            network_ids=AMP_NET,
+            port_ids=[PORT_ID],
+            config_drive_files=None,
+            user_data='test_ud_conf',
+            server_group_id=None)
+
+        # Make sure it returns the expected compute_id
+        self.assertEqual(COMPUTE_ID, compute_id)
+
+        # Test that a build exception is raised
+        createcompute = compute_tasks.ComputeCreate()
+
+        self.assertRaises(TypeError,
+                          createcompute.execute,
+                          _amphora_mock, config_drive_files='test_cert')
+
+        # Test revert()
+
+        _amphora_mock.compute_id = COMPUTE_ID
+
+        createcompute = compute_tasks.ComputeCreate()
+        createcompute.revert(compute_id, _amphora_mock.id)
+
+        # Validate that the delete method was called properly
+        mock_driver.delete.assert_called_once_with(
+            COMPUTE_ID)
+
+        # Test that a delete exception is not raised
+
+        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+
+    @mock.patch('jinja2.Environment.get_template')
+    @mock.patch('octavia.amphorae.backends.agent.'
+                'agent_jinja_cfg.AgentJinjaTemplater.'
+                'build_agent_config', return_value='test_conf')
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_compute_create_without_ssh_access(self, mock_driver,
+                                               mock_conf, mock_jinja):
+
+        createcompute = compute_tasks.ComputeCreate()
+
+        mock_driver.build.return_value = COMPUTE_ID
+        self.conf.config(
+            group="controller_worker", amp_ssh_access_allowed=False)
+        self.conf.config(
+            group="controller_worker", user_data_config_drive=False)
+
+        # Test execute()
+        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
+                                           server_group_id=SERVER_GRPOUP_ID)
+
+        # Validate that the build method was called properly
+        mock_driver.build.assert_called_once_with(
+            name="amphora-" + _amphora_mock.id,
+            amphora_flavor=AMP_FLAVOR_ID,
+            image_id=AMP_IMAGE_ID,
+            image_tag=AMP_IMAGE_TAG,
+            image_owner='',
+            key_name=None,
+            sec_groups=AMP_SEC_GROUPS,
+            network_ids=AMP_NET,
+            port_ids=[PORT_ID],
+            config_drive_files={'/etc/octavia/'
+                                'amphora-agent.conf': 'test_conf'},
+            user_data=None,
+            server_group_id=SERVER_GRPOUP_ID)
+
+        self.assertEqual(COMPUTE_ID, compute_id)
+
+        # Test that a build exception is raised
+        createcompute = compute_tasks.ComputeCreate()
+
+        self.assertRaises(TypeError,
+                          createcompute.execute,
+                          _amphora_mock, config_drive_files='test_cert')
+
+        # Test revert()
+
+        _amphora_mock.compute_id = COMPUTE_ID
+
+        createcompute = compute_tasks.ComputeCreate()
+        createcompute.revert(compute_id, _amphora_mock.id)
+
+        # Validate that the delete method was called properly
+        mock_driver.delete.assert_called_once_with(
+            COMPUTE_ID)
+
+        # Test that a delete exception is not raised
+
+        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+
     @mock.patch('jinja2.Environment.get_template')
     @mock.patch('octavia.amphorae.backends.agent.'
                 'agent_jinja_cfg.AgentJinjaTemplater.'
@@ -272,13 +272,17 @@ class TestComputeTasks(base.TestCase):
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_compute_create_cert(self, mock_driver, mock_conf, mock_jinja):
         createcompute = compute_tasks.CertComputeCreate()
+        key = utils.get_six_compatible_server_certs_key_passphrase()
+        fer = fernet.Fernet(key)
 
         mock_driver.build.return_value = COMPUTE_ID
         path = '/etc/octavia/certs/ca_01.pem'
         self.useFixture(test_utils.OpenFixture(path, 'test'))
-
         # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, 'test_cert',
+        test_cert = fer.encrypt(
+            utils.get_six_compatible_value('test_cert')
+        )
+        compute_id = createcompute.execute(_amphora_mock.id, test_cert,
                                            server_group_id=SERVER_GRPOUP_ID
                                            )
 
@@ -295,13 +299,12 @@ class TestComputeTasks(base.TestCase):
             port_ids=[],
             user_data=None,
             config_drive_files={
-                '/etc/octavia/certs/server.pem': 'test_cert',
+                '/etc/octavia/certs/server.pem': fer.decrypt(test_cert),
                 '/etc/octavia/certs/client_ca.pem': 'test',
                 '/etc/octavia/amphora-agent.conf': 'test_conf'},
             server_group_id=SERVER_GRPOUP_ID)
 
-        # Make sure it returns the expected compute_id
-        assert (compute_id == COMPUTE_ID)
+        self.assertEqual(COMPUTE_ID, compute_id)
 
         # Test that a build exception is raised
         self.useFixture(test_utils.OpenFixture(path, 'test'))
@@ -310,7 +313,7 @@ class TestComputeTasks(base.TestCase):
         self.assertRaises(TypeError,
                           createcompute.execute,
                           _amphora_mock,
-                          config_drive_files='test_cert')
+                          config_drive_files=test_cert)
 
         # Test revert()
 
@@ -326,20 +329,24 @@ class TestComputeTasks(base.TestCase):
 
         createcompute.revert(COMPUTE_ID, _amphora_mock.id)
 
+    @mock.patch('octavia.controller.worker.amphora_rate_limit'
+                '.AmphoraBuildRateLimit.remove_from_build_req_queue')
     @mock.patch('stevedore.driver.DriverManager.driver')
     @mock.patch('time.sleep')
     def test_compute_wait(self,
                           mock_time_sleep,
-                          mock_driver):
+                          mock_driver,
+                          mock_remove_from_build_queue):
 
+        self.conf.config(group='haproxy_amphora', build_rate_limit=5)
         _amphora_mock.compute_id = COMPUTE_ID
         _amphora_mock.status = constants.ACTIVE
         _amphora_mock.lb_network_ip = LB_NET_IP
 
-        mock_driver.get_amphora.return_value = _amphora_mock
+        mock_driver.get_amphora.return_value = _amphora_mock, None
 
-        computewait = compute_tasks.ComputeWait()
-        computewait.execute(COMPUTE_ID)
+        computewait = compute_tasks.ComputeActiveWait()
+        computewait.execute(COMPUTE_ID, AMPHORA_ID)
 
         mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
 
@@ -347,20 +354,26 @@ class TestComputeTasks(base.TestCase):
 
         self.assertRaises(exceptions.ComputeWaitTimeoutException,
                           computewait.execute,
-                          _amphora_mock)
+                          _amphora_mock, AMPHORA_ID)
 
+    @mock.patch('octavia.controller.worker.amphora_rate_limit'
+                '.AmphoraBuildRateLimit.remove_from_build_req_queue')
     @mock.patch('stevedore.driver.DriverManager.driver')
     @mock.patch('time.sleep')
-    def test_compute_wait_error_status(self, mock_time_sleep, mock_driver):
+    def test_compute_wait_error_status(self,
+                                       mock_time_sleep,
+                                       mock_driver,
+                                       mock_remove_from_build_queue):
 
+        self.conf.config(group='haproxy_amphora', build_rate_limit=5)
         _amphora_mock.compute_id = COMPUTE_ID
         _amphora_mock.status = constants.ACTIVE
         _amphora_mock.lb_network_ip = LB_NET_IP
 
-        mock_driver.get_amphora.return_value = _amphora_mock
+        mock_driver.get_amphora.return_value = _amphora_mock, None
 
-        computewait = compute_tasks.ComputeWait()
-        computewait.execute(COMPUTE_ID)
+        computewait = compute_tasks.ComputeActiveWait()
+        computewait.execute(COMPUTE_ID, AMPHORA_ID)
 
         mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
 
@@ -368,7 +381,27 @@ class TestComputeTasks(base.TestCase):
 
         self.assertRaises(exceptions.ComputeBuildException,
                           computewait.execute,
-                          _amphora_mock)
+                          _amphora_mock, AMPHORA_ID)
+
+    @mock.patch('octavia.controller.worker.amphora_rate_limit'
+                '.AmphoraBuildRateLimit.remove_from_build_req_queue')
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    @mock.patch('time.sleep')
+    def test_compute_wait_skipped(self,
+                                  mock_time_sleep,
+                                  mock_driver,
+                                  mock_remove_from_build_queue):
+        _amphora_mock.compute_id = COMPUTE_ID
+        _amphora_mock.status = constants.ACTIVE
+        _amphora_mock.lb_network_ip = LB_NET_IP
+
+        mock_driver.get_amphora.return_value = _amphora_mock, None
+
+        computewait = compute_tasks.ComputeActiveWait()
+        computewait.execute(COMPUTE_ID, AMPHORA_ID)
+
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        mock_remove_from_build_queue.assert_not_called()
 
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_delete_amphorae_on_load_balancer(self, mock_driver):
@@ -404,7 +437,7 @@ class TestComputeTasks(base.TestCase):
             'octavia-lb-123', 'anti-affinity')
 
         # Make sure it returns the expected server group_id
-        assert(sg_id == server_group_test_id)
+        self.assertEqual(server_group_test_id, sg_id)
 
         # Test revert()
         nova_sever_group_obj.revert(sg_id)
