@@ -13,16 +13,11 @@
 # under the License.
 #
 
-import six
+import mock
 
 from octavia.controller.worker.tasks import model_tasks
 import octavia.tests.unit.base as base
 
-if six.PY2:
-    import mock
-else:
-    import unittest.mock as mock
-
 
 class TestObjectUpdateTasks(base.TestCase):
 
@@ -46,4 +41,4 @@ class TestObjectUpdateTasks(base.TestCase):
         update_attr.execute(self.listener_mock,
                             {'name': 'TEST2'})
 
-        assert self.listener_mock.name == 'TEST2'
+        self.listener_mock.update.assert_called_once_with({'name': 'TEST2'})
