@@ -61,7 +61,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         super(TestAllowedAddressPairsDriver, self).setUp()
         with mock.patch('octavia.common.clients.neutron_client.Client',
                         autospec=True) as neutron_client:
-            with mock.patch('octavia.common.clients.nova_client.Client',
+            with mock.patch('stevedore.driver.DriverManager.driver',
                             autospec=True):
                 client = neutron_client(clients.NEUTRON_VERSION)
                 client.list_extensions.return_value = {
@@ -299,123 +299,79 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         show_port.side_effect = neutron_exceptions.PortNotFoundClient
         self.driver.deallocate_vip(vip)
 
-    def test_plug_vip_errors_when_nova_cant_find_network_to_attach(self):
+    def test_plug_aap_errors_when_nova_cant_find_network_to_attach(self):
         lb = dmh.generate_load_balancer_tree()
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = {
-            'subnet': {
-                'id': lb.vip.subnet_id
-            }
-        }
-        list_security_groups = self.driver.neutron_client.list_security_groups
-        lsc_side_effect = [
-            None, {
-                'security_groups': [
-                    {'id': 'lb-sec-grp1'}
-                ]
-            }
-        ]
-        list_security_groups.side_effect = lsc_side_effect
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
 
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.side_effect = nova_exceptions.NotFound(404, "Network")
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = nova_exceptions.NotFound(404, "Network")
         self.assertRaises(network_base.PlugVIPException,
-                          self.driver.plug_vip, lb, lb.vip)
+                          self.driver.plug_aap_port, lb, lb.vip,
+                          lb.amphorae[0], subnet)
 
-    def test_plug_vip_errors_when_neutron_cant_find_port_to_update(self):
+    def test_plug_aap_errors_when_neutron_cant_find_port_to_update(self):
         lb = dmh.generate_load_balancer_tree()
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = {
-            'subnet': {
-                'id': lb.vip.subnet_id
-            }
-        }
-        list_security_groups = self.driver.neutron_client.list_security_groups
-        lsc_side_effect = [
-            None, {
-                'security_groups': [
-                    {'id': 'lb-sec-grp1'}
-                ]
-            }
-        ]
-        list_security_groups.side_effect = lsc_side_effect
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.return_value = t_constants.MOCK_NOVA_INTERFACE
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.return_value = t_constants.MOCK_NOVA_INTERFACE
 
         update_port = self.driver.neutron_client.update_port
         update_port.side_effect = neutron_exceptions.PortNotFoundClient
         self.assertRaises(network_base.PortNotFound,
-                          self.driver.plug_vip, lb, lb.vip)
+                          self.driver.plug_aap_port, lb, lb.vip,
+                          lb.amphorae[0], subnet)
 
-    def test_plug_vip(self):
+    @mock.patch('octavia.network.drivers.neutron.allowed_address_pairs.'
+                'AllowedAddressPairsDriver.update_vip_sg')
+    @mock.patch('octavia.network.drivers.neutron.allowed_address_pairs.'
+                'AllowedAddressPairsDriver.get_subnet')
+    @mock.patch('octavia.network.drivers.neutron.allowed_address_pairs.'
+                'AllowedAddressPairsDriver.plug_aap_port')
+    def test_plug_vip(self, mock_plug_aap, mock_get_subnet,
+                      mock_update_vip_sg):
+        lb = dmh.generate_load_balancer_tree()
+        subnet = mock.MagicMock()
+        mock_get_subnet.return_value = subnet
+        mock_plug_aap.side_effect = lb.amphorae
+        amps = self.driver.plug_vip(lb, lb.vip)
+
+        mock_update_vip_sg.assert_called_with(lb, lb.vip)
+        mock_get_subnet.assert_called_with(lb.vip.subnet_id)
+        for amp in amps:
+            mock_plug_aap.assert_any_call(lb, lb.vip, amp, subnet)
+
+    def test_update_vip_sg(self):
         lb = dmh.generate_load_balancer_tree()
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = {
-            'subnet': {
-                'id': t_constants.MOCK_VIP_SUBNET_ID,
-                'network_id': t_constants.MOCK_VIP_NET_ID
-            }
-        }
-        list_ports = self.driver.neutron_client.list_ports
-        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
-        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
-        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.side_effect = [t_constants.MOCK_VRRP_INTERFACE1,
-                                        t_constants.MOCK_VRRP_INTERFACE2]
         list_security_groups = self.driver.neutron_client.list_security_groups
         list_security_groups.return_value = {
             'security_groups': [
                 {'id': 'lb-sec-grp1'}
             ]
         }
+        self.driver.update_vip_sg(lb, lb.vip)
+
+    def test_plug_aap_port(self):
+        lb = dmh.generate_load_balancer_tree()
+
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
+
+        list_ports = self.driver.neutron_client.list_ports
+        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
+        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
+        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = [t_constants.MOCK_VRRP_INTERFACE1]
         update_port = self.driver.neutron_client.update_port
         expected_aap = {'port': {'allowed_address_pairs':
                                  [{'ip_address': lb.vip.ip_address}]}}
-        amps = self.driver.plug_vip(lb, lb.vip)
-        self.assertEqual(5, update_port.call_count)
-        for amp in amps:
-            update_port.assert_any_call(amp.vrrp_port_id, expected_aap)
-            self.assertIn(amp.vrrp_ip, [t_constants.MOCK_VRRP_IP1,
-                                        t_constants.MOCK_VRRP_IP2])
-            self.assertEqual(lb.vip.ip_address, amp.ha_ip)
-
-    @mock.patch('octavia.network.drivers.neutron.utils.'
-                'convert_port_dict_to_model')
-    def test_plug_vip_amp__port_create_fails(self, mock_convert):
-        lb = dmh.generate_load_balancer_tree()
-
-        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
-                                       network_id=t_constants.MOCK_VIP_NET_ID)
-
-        list_ports = self.driver.neutron_client.list_ports
-        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
-        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
-        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
-        port_create = self.driver.neutron_client.create_port
-        port_create.side_effect = [Exception('Create failure')]
-        self.assertRaises(network_base.PlugVIPException,
-                          self.driver._plug_amphora_vip,
-                          lb.amphorae[0], subnet)
-        mock_convert.assert_not_called()
-        self.driver.neutron_client.delete_port.assert_not_called()
-
-    def test_plug_vip_amp_port_attach_fails(self):
-        lb = dmh.generate_load_balancer_tree()
-
-        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
-                                       network_id=t_constants.MOCK_VIP_NET_ID)
-
-        list_ports = self.driver.neutron_client.list_ports
-        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
-        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
-        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
-        network_attach = self.driver.nova_client.servers.interface_attach
-        network_attach.side_effect = [Exception('Attach failure')]
-        self.assertRaises(network_base.PlugVIPException,
-                          self.driver._plug_amphora_vip,
-                          lb.amphorae[0], subnet)
-        self.driver.neutron_client.delete_port.assert_called_once()
+        amp = self.driver.plug_aap_port(lb, lb.vip, lb.amphorae[0], subnet)
+        update_port.assert_any_call(amp.vrrp_port_id, expected_aap)
+        self.assertIn(amp.vrrp_ip, [t_constants.MOCK_VRRP_IP1,
+                                    t_constants.MOCK_VRRP_IP2])
+        self.assertEqual(lb.vip.ip_address, amp.ha_ip)
 
     def _set_safely(self, obj, name, value):
         if isinstance(obj, dict):
@@ -427,16 +383,12 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             self.addCleanup(setattr, obj, name, current)
             setattr(obj, name, value)
 
-    def test_plug_vip_on_mgmt_net(self):
+    def test_plug_aap_on_mgmt_net(self):
         lb = dmh.generate_load_balancer_tree()
         lb.vip.subnet_id = t_constants.MOCK_MANAGEMENT_SUBNET_ID
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = {
-            'subnet': {
-                'id': t_constants.MOCK_MANAGEMENT_SUBNET_ID,
-                'network_id': t_constants.MOCK_MANAGEMENT_NET_ID
-            }
-        }
+        subnet = network_models.Subnet(
+            id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
+            network_id=t_constants.MOCK_MANAGEMENT_NET_ID)
         list_ports = self.driver.neutron_client.list_ports
         port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
         port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
@@ -445,7 +397,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self._set_safely(t_constants.MOCK_MANAGEMENT_FIXED_IPS2[0],
                          'ip_address', lb.amphorae[1].lb_network_ip)
         list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
-        interface_attach = self.driver.nova_client.servers.interface_attach
+        network_attach = self.driver.compute.attach_network_or_port
         self._set_safely(t_constants.MOCK_VRRP_INTERFACE1,
                          'net_id', t_constants.MOCK_MANAGEMENT_NET_ID)
         self._set_safely(t_constants.MOCK_VRRP_FIXED_IPS1[0],
@@ -454,24 +406,15 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                          'net_id', t_constants.MOCK_MANAGEMENT_NET_ID)
         self._set_safely(t_constants.MOCK_VRRP_FIXED_IPS2[0],
                          'subnet_id', t_constants.MOCK_MANAGEMENT_SUBNET_ID)
-        interface_attach.side_effect = [t_constants.MOCK_VRRP_INTERFACE1,
-                                        t_constants.MOCK_VRRP_INTERFACE2]
-        list_security_groups = self.driver.neutron_client.list_security_groups
-        list_security_groups.return_value = {
-            'security_groups': [
-                {'id': 'lb-sec-grp1'}
-            ]
-        }
+        network_attach.side_effect = [t_constants.MOCK_VRRP_INTERFACE1]
         update_port = self.driver.neutron_client.update_port
         expected_aap = {'port': {'allowed_address_pairs':
                                  [{'ip_address': lb.vip.ip_address}]}}
-        amps = self.driver.plug_vip(lb, lb.vip)
-        self.assertEqual(5, update_port.call_count)
-        for amp in amps:
-            update_port.assert_any_call(amp.vrrp_port_id, expected_aap)
-            self.assertIn(amp.vrrp_ip, [t_constants.MOCK_VRRP_IP1,
-                                        t_constants.MOCK_VRRP_IP2])
-            self.assertEqual(lb.vip.ip_address, amp.ha_ip)
+        amp = self.driver.plug_aap_port(lb, lb.vip, lb.amphorae[0], subnet)
+        update_port.assert_any_call(amp.vrrp_port_id, expected_aap)
+        self.assertIn(amp.vrrp_ip, [t_constants.MOCK_VRRP_IP1,
+                                    t_constants.MOCK_VRRP_IP2])
+        self.assertEqual(lb.vip.ip_address, amp.ha_ip)
 
     def test_allocate_vip_when_port_already_provided(self):
         show_port = self.driver.neutron_client.show_port
@@ -570,38 +513,44 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
 
-    def test_unplug_vip_errors_when_update_port_cant_find_port(self):
+    def test_unplug_aap_port_errors_when_update_port_cant_find_port(self):
         lb = dmh.generate_load_balancer_tree()
         list_ports = self.driver.neutron_client.list_ports
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = t_constants.MOCK_SUBNET
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
         port2 = {
             'id': '4', 'network_id': '3', 'fixed_ips':
             [{'ip_address': '10.0.0.2'}]
         }
+        subnet = network_models.Subnet(
+            id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
+            network_id='3')
         list_ports.return_value = {'ports': [port1, port2]}
         update_port = self.driver.neutron_client.update_port
         update_port.side_effect = neutron_exceptions.PortNotFoundClient
         self.assertRaises(network_base.UnplugVIPException,
-                          self.driver.unplug_vip, lb, lb.vip)
+                          self.driver.unplug_aap_port, lb.vip, lb.amphorae[0],
+                          subnet)
 
-    def test_unplug_vip_errors_when_update_port_fails(self):
+    def test_unplug_aap_errors_when_update_port_fails(self):
         lb = dmh.generate_load_balancer_tree()
-        show_subnet = self.driver.neutron_client.show_subnet
-        show_subnet.return_value = t_constants.MOCK_SUBNET
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
         port2 = {
             'id': '4', 'network_id': '3', 'fixed_ips':
             [{'ip_address': '10.0.0.2'}]
         }
+
+        subnet = network_models.Subnet(
+            id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
+            network_id='3')
+
         list_ports = self.driver.neutron_client.list_ports
         list_ports.return_value = {'ports': [port1, port2]}
 
         update_port = self.driver.neutron_client.update_port
         update_port.side_effect = TypeError
         self.assertRaises(network_base.UnplugVIPException,
-                          self.driver.unplug_vip, lb, lb.vip)
+                          self.driver.unplug_aap_port, lb.vip,
+                          lb.amphorae[0], subnet)
 
     def test_unplug_vip_errors_when_vip_subnet_not_found(self):
         lb = dmh.generate_load_balancer_tree()
@@ -610,30 +559,38 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertRaises(network_base.PluggedVIPNotFound,
                           self.driver.unplug_vip, lb, lb.vip)
 
-    def test_unplug_vip(self):
+    @mock.patch('octavia.network.drivers.neutron.allowed_address_pairs.'
+                'AllowedAddressPairsDriver.unplug_aap_port')
+    def test_unplug_vip(self, mock):
         lb = dmh.generate_load_balancer_tree()
         show_subnet = self.driver.neutron_client.show_subnet
         show_subnet.return_value = t_constants.MOCK_SUBNET
+        self.driver.unplug_vip(lb, lb.vip)
+        self.assertEqual(len(lb.amphorae), mock.call_count)
+
+    def test_unplug_aap_port(self):
+        lb = dmh.generate_load_balancer_tree()
         update_port = self.driver.neutron_client.update_port
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
         port2 = {
             'id': '4', 'network_id': '3', 'fixed_ips':
             [{'ip_address': '10.0.0.2'}]
         }
+        subnet = network_models.Subnet(
+            id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
+            network_id='3')
         list_ports = self.driver.neutron_client.list_ports
         list_ports.return_value = {'ports': [port1, port2]}
         get_port = self.driver.neutron_client.get_port
         get_port.side_effect = neutron_exceptions.NotFound
-        self.driver.unplug_vip(lb, lb.vip)
-        self.assertEqual(len(lb.amphorae), update_port.call_count)
+        self.driver.unplug_aap_port(lb.vip, lb.amphorae[0], subnet)
         clear_aap = {'port': {'allowed_address_pairs': []}}
-        update_port.assert_has_calls([mock.call(port1.get('id'), clear_aap),
-                                      mock.call(port1.get('id'), clear_aap)])
+        update_port.assert_called_once_with(port2.get('id'), clear_aap)
 
     def test_plug_network_when_compute_instance_cant_be_found(self):
         net_id = t_constants.MOCK_NOVA_INTERFACE.net_id
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.side_effect = nova_exceptions.NotFound(
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = nova_exceptions.NotFound(
             404, message='Instance not found')
         self.assertRaises(network_base.AmphoraNotFound,
                           self.driver.plug_network,
@@ -641,8 +598,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
 
     def test_plug_network_when_network_cant_be_found(self):
         net_id = t_constants.MOCK_NOVA_INTERFACE.net_id
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.side_effect = nova_exceptions.NotFound(
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = nova_exceptions.NotFound(
             404, message='Network not found')
         self.assertRaises(network_base.NetworkException,
                           self.driver.plug_network,
@@ -650,16 +607,16 @@ class TestAllowedAddressPairsDriver(base.TestCase):
 
     def test_plug_network_when_interface_attach_fails(self):
         net_id = t_constants.MOCK_NOVA_INTERFACE.net_id
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.side_effect = TypeError
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = TypeError
         self.assertRaises(network_base.PlugNetworkException,
                           self.driver.plug_network,
                           t_constants.MOCK_COMPUTE_ID, net_id)
 
     def test_plug_network(self):
         net_id = t_constants.MOCK_NOVA_INTERFACE.net_id
-        interface_attach = self.driver.nova_client.servers.interface_attach
-        interface_attach.return_value = t_constants.MOCK_NOVA_INTERFACE
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.return_value = t_constants.MOCK_NOVA_INTERFACE
         oct_interface = self.driver.plug_network(
             t_constants.MOCK_COMPUTE_ID, net_id)
         exp_ips = [fixed_ip.get('ip_address')
@@ -687,19 +644,6 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                           self.driver.unplug_network,
                           t_constants.MOCK_COMPUTE_ID, net_id)
 
-    def test_unplug_network_when_interface_detach_fails(self):
-        list_ports = self.driver.neutron_client.list_ports
-        port1 = t_constants.MOCK_NEUTRON_PORT['port']
-        port2 = {
-            'id': '4', 'network_id': '3', 'fixed_ips':
-            [{'ip_address': '10.0.0.2'}]
-        }
-        list_ports.return_value = {'ports': [port1, port2]}
-        interface_detach = self.driver.nova_client.servers.interface_detach
-        interface_detach.side_effect = Exception
-        self.driver.unplug_network(t_constants.MOCK_COMPUTE_ID,
-                                   port2.get('network_id'))
-
     def test_unplug_network(self):
         list_ports = self.driver.neutron_client.list_ports
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
@@ -708,11 +652,11 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             [{'ip_address': '10.0.0.2'}]
         }
         list_ports.return_value = {'ports': [port1, port2]}
-        interface_detach = self.driver.nova_client.servers.interface_detach
+        port_detach = self.driver.compute.detach_port
         self.driver.unplug_network(t_constants.MOCK_COMPUTE_ID,
                                    port2.get('network_id'))
-        interface_detach.assert_called_once_with(
-            server=t_constants.MOCK_COMPUTE_ID, port_id=port2.get('id'))
+        port_detach.assert_called_once_with(
+            compute_id=t_constants.MOCK_COMPUTE_ID, port_id=port2.get('id'))
 
     def test_update_vip(self):
         listeners = [data_models.Listener(protocol_port=80, peer_port=1024,
@@ -944,7 +888,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
     def test_plug_port(self):
         port = mock.MagicMock()
         port.id = self.PORT_ID
-        interface_attach = self.driver.nova_client.servers.interface_attach
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.return_value = t_constants.MOCK_NOVA_INTERFACE
         amphora = data_models.Amphora(
             id=self.AMPHORA_ID, load_balancer_id=self.LB_ID,
             compute_id=self.COMPUTE_ID, status=self.ACTIVE,
@@ -952,25 +897,25 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             ha_ip=self.HA_IP)
 
         self.driver.plug_port(amphora, port)
-        interface_attach.assert_called_once_with(server=amphora.compute_id,
-                                                 net_id=None,
-                                                 fixed_ip=None,
-                                                 port_id=self.PORT_ID)
+        network_attach.assert_called_once_with(compute_id=amphora.compute_id,
+                                               network_id=None,
+                                               ip_address=None,
+                                               port_id=self.PORT_ID)
 
         # NotFound cases
-        interface_attach.side_effect = nova_exceptions.NotFound(
+        network_attach.side_effect = nova_exceptions.NotFound(
             1, message='Instance')
         self.assertRaises(network_base.AmphoraNotFound,
                           self.driver.plug_port,
                           amphora,
                           port)
-        interface_attach.side_effect = nova_exceptions.NotFound(
+        network_attach.side_effect = nova_exceptions.NotFound(
             1, message='Network')
         self.assertRaises(network_base.NetworkNotFound,
                           self.driver.plug_port,
                           amphora,
                           port)
-        interface_attach.side_effect = nova_exceptions.NotFound(
+        network_attach.side_effect = nova_exceptions.NotFound(
             1, message='bogus')
         self.assertRaises(network_base.PlugNetworkException,
                           self.driver.plug_port,
@@ -978,11 +923,11 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                           port)
 
         # Already plugged case should not raise an exception
-        interface_attach.side_effect = nova_exceptions.Conflict(1)
+        network_attach.side_effect = nova_exceptions.Conflict(1)
         self.driver.plug_port(amphora, port)
 
         # Unknown error case
-        interface_attach.side_effect = TypeError
+        network_attach.side_effect = TypeError
         self.assertRaises(network_base.PlugNetworkException,
                           self.driver.plug_port,
                           amphora,
